function [fitCoord] = computeBackbonePixels(fitCoeff, imSize)
% [fitCoord] = computeBackbonePixels(fitCoeff, imSize)
% finds pixel representation of a curve fitted into cell backbone
%
% Inputs
%   fitCoeff    : curve coefficients p1, p2 and p3, which define the curve 
%                 as y = p1*x1^2 + p2*x2 + p3
%   imSize      : size of the image of the cell in the bounding box
%
% Output
%   fitCoord    : array of pixel coordinates (x,y) that represent the curve
%                 in the cell image

% SZ, 2017-10-10: fixed the case when the stationary point is outside image 

xData = (0.5:imSize(2)+0.5)';        
yData = fitCoeff(1)*xData.^2 + fitCoeff(2)*xData + fitCoeff(3);
if abs(fitCoeff(1))<1e-4 && round(yData(1))==round(yData(end))
    % Straight line
    fitCoord = [(1:imSize(2))' round(yData(1))*ones(imSize(2),1)];
else
    % Compute upper and lower bounds for each column in the image
    yDataRound = round(yData);    
    statPt = -fitCoeff(2)/(2*fitCoeff(1));
    if statPt > 0.5 && statPt < imSize(2)
        roundStatPtVal = round(fitCoeff(1)*statPt^2 + fitCoeff(2)*statPt + fitCoeff(3));
        roundStatPt = round(statPt);
        uppBound = zeros(imSize(2),1);
        lowBound = zeros(imSize(2),1);
        if fitCoeff(1)>0                
            uppBound(1:roundStatPt-1) = yDataRound(1:roundStatPt-1);
            uppBound(roundStatPt) = max(yDataRound(roundStatPt:roundStatPt+1));
            uppBound(roundStatPt+1:end) = yDataRound(roundStatPt+2:end);
            lowBound(1:roundStatPt-1) = yDataRound(2:roundStatPt);
            roundDownPoint = find(yDataRound(2:roundStatPt)-yData(2:roundStatPt)==0.5);
            lowBound(roundDownPoint) = lowBound(roundDownPoint)-1;                
            lowBound(roundStatPt) = roundStatPtVal;                
            lowBound(roundStatPt+1:end) = yDataRound(roundStatPt+1:end-1);
        else                
            uppBound(1:roundStatPt-1) = yDataRound(2:roundStatPt);
            uppBound(roundStatPt) = roundStatPtVal;
            uppBound(roundStatPt+1:end) = yDataRound(roundStatPt+1:end-1);
            lowBound(1:roundStatPt-1) = yDataRound(1:roundStatPt-1);
            lowBound(roundStatPt) = max(yDataRound(roundStatPt:roundStatPt+1));
            lowBound(roundStatPt+1:end) = yDataRound(roundStatPt+2:end);
        end
    else
        if (fitCoeff(1)>0 && statPt>imSize(2)) || (fitCoeff(1)<0 && statPt<0.5)
            uppBound = yDataRound(1:end-1);
            lowBound = yDataRound(2:end);
            roundDownPoint = find(lowBound-yData(2:end)==0.5);
            lowBound(roundDownPoint) = lowBound(roundDownPoint)-1; 
        else
            uppBound = yDataRound(2:end);
            lowBound = yDataRound(1:end-1);
        end
    end
    lowBound = max(lowBound,1);
    uppBound = min(uppBound,imSize(1));
    nPixelsPerCol = uppBound-lowBound+1;
    nPixelsPerCol(nPixelsPerCol < 1) = 0;
    fitCoord = zeros(sum(nPixelsPerCol),2);
    idx=0;
    for i=1:length(nPixelsPerCol)
        fitCoord(idx+1:idx+nPixelsPerCol(i),1) = i;
        fitCoord(idx+1:idx+nPixelsPerCol(i),2) = lowBound(i):uppBound(i);
        idx=idx+nPixelsPerCol(i);
    end    
end

end