function backbone = computeCellBackbone(cellImage, distThresh, se, debugFlag)
% backbone = computeCellLength(cellImage, distanceThreshold, se, debugFlag)
%
% Computes cell length by fitting a second-order curve that goes through 
% the cell and represents its backbone.
%
% Input:
%   cellImage  - image of the cell cropped from the segmented image.
%   distThresh - threshold for selecting points inside the cell for
%                curve fitting. Discard points whose distance to the
%                image border in pixels is below the threshold. The default
%                value is 3.
%   se - dilation disk generated by strel(disk), its radius is 0.5*distThresh
%   debugFlag - flag for debugging. Optional, the default value is 0.
% 
% Output:
%   backbone   - struct with fields:
%                   fitCoeff - curve fitting coefficients
%                   length - cell length (in pixels)
%                   coordinates - backbone coordinates in the cell image
%                   poles - 2x2 array, where raws contain poles coordinates
%                           in the cell image.

import Cell.Measure.*

if ~exist('distThresh','var') || isempty(distThresh)
    distThresh = 3;
end

if ~exist('debugFlag','var') || isempty(debugFlag)
    debugFlag = 0;
end

dilRadius = floor(distThresh/2);
if ~exist('se','var') || isempty(se)
    se = strel('disk', dilRadius, 8);
end

% Dilate the cell image and compute Euclidean distance transform
paddedIm = padarray(cellImage, [dilRadius+1 dilRadius+1]);
labelIm = imdilate(paddedIm,se);
distMat = double(bwdist(imcomplement(labelIm)));
labelDist = distMat.* (distMat >= distThresh);
labelDist = ...
    labelDist(dilRadius+2:end-dilRadius-1,dilRadius+2:end-dilRadius-1);

% Get the list of remaining pixels and their value and make the inner
% pixels even more important (squared)   
[yCoords, xCoords, pixelValues]= find(labelDist);
if ~isempty(yCoords)
    % Solve weighted linear least-squares problem (W*A)*x = W*b
    weights = sqrt(pixelValues.^3);
    A = [xCoords.^2 xCoords ones(length(xCoords),1)];
    fitCoeff = (repmat(weights,1,size(A,2)).*A) \ (weights.*yCoords);
else
    backbone.length = -1;
    return
end

% Find pixel representation of the curve inside the cell
imSize = size(cellImage);
fitCoord = computeBackbonePixels(fitCoeff, imSize);    
fitIm = false(imSize);
try
    fitIm(sub2ind(imSize,fitCoord(:,2),fitCoord(:,1))) = 1;
catch
    backbone.length = -1;
    return
end
backboneIm = cellImage.*fitIm;
[backboneCoord(:,2), backboneCoord(:,1)] = find(backboneIm);

if isempty(backboneCoord)
    backbone.length = -1;
    return
end

arcLength = computeArcLength(fitCoeff, backboneCoord(1,1)-0.5, ...
    backboneCoord(end,1)+0.5);

% Compute cell poles as intersection points of the backbone curve with
% the cell boundary
poles(:,1) = [backboneCoord(1,1) - 0.5; backboneCoord(end,1) + 0.5];
poles(:,2) = fitCoeff(1)*poles(:,1).^2 + fitCoeff(2)*poles(:,1) + ...
             fitCoeff(3);

% Update the output structure with the computed values
backbone.fitCoeff = fitCoeff;
backbone.length = arcLength;
backbone.coordinates = backboneCoord;
backbone.poles = poles;

% Debugging
if debugFlag
    figure, hold on,
    imagesc(cellImage),colormap gray, axis image
    xx = 0.5:0.2:(imSize(2)-0.5);
    yy = fitCoeff(1)*xx.^2+fitCoeff(2)*xx+fitCoeff(3);
    plot(xx,yy,'g-');
    plot(poles(:,1),poles(:,2),'*r');
    hold off
end