function computeCellMeasurements(expInfoObj, posName, mFrames)
% computeCellMeasurements(expInfoObj, posName, mFrames)
%
% This function computes cell length, width and poles by fitting a 
% second-order curve into the cell backbone.
%
% Input:
%   expInfoObj   - expInfo object with various info about the experiment, 
%                  including parameter distanceThreshold that defines 
%                  distance from the border to the inside of a cell. 
%                  Typical value 6.
%   posName      - string with position name
%   mFrames      - MFrames object that allows to access MCell objects by
%                  frame and blob index. Optional.
%% Initialization
import Cell.Measure.*

if nargin==2
    mCellsMatFile = expInfoObj.getMCellMatPath(posName);
    [~, mFrames] = Cell.MCell.loadMCells(mCellsMatFile);
end

if ~expInfoObj.getParameters('parforPositions')
    computeCellMeasurements_parfor(expInfoObj, posName, mFrames);
    return;
end

phaseChanName = expInfoObj.getChannelNames('phase');
phaseIndices = expInfoObj.getIndices(posName,phaseChanName);
nFrames = length(phaseIndices);
outputDir = expInfoObj.getPathName('output');
segDir = fullfile(outputDir, posName, 'SegmentedPhase');
segList = dir(fullfile(segDir, '*.tif*'));

% Create dilation disk with a radius defined by the distance threshold
distThreshold = expInfoObj.getParameters('distanceThreshold');
dilationRadius = floor(distThreshold/2);
se = strel('disk', dilationRadius, 8);

%% Main loop
warning('off','MATLAB:rankDeficientMatrix');
for fIdx = 1:nFrames
    currSegImage = imread(fullfile(segDir, segList(fIdx).name));
    blobLabels = mFrames.getBlobsInFrame(fIdx);
    for bIdx = 1:numel(blobLabels)
        blobLabel = blobLabels(bIdx);
        cellsInBlob = mFrames.getCellsInBlob(fIdx, blobLabel);
        c = cellsInBlob(1);
        t = c.getIndex(fIdx);

        % Crop the cell image from the frame in a bounding box
        cellIm = cropCell(currSegImage, c, t);
        c.majorAxisOrientationDegree(t) = computeCellOrientation(cellIm,c.isRotated(t));

        if ~c.badSegmentations(t)
            % Compute cell length by fitting a curve into cell backbone
            backbone = computeCellBackbone(cellIm, distThreshold, se);
            if backbone.length > 0
                c.lengths(t) = backbone.length;
                c.widths(t) = computeCellWidth(cellIm, backbone);
                c.poles{t} = c.convertFromCellCoord(t, backbone.poles, 1);
                c.backbones(:,t) = backbone.fitCoeff;
            end
        else
            c.lengths(t) = -1;
        end
        % Copy measurements to the other cells converging in this blob
        for ocIdx = 2:numel(cellsInBlob)
            oc = cellsInBlob(ocIdx);
            ot = oc.getIndex(fIdx);
            oc.lengths(ot) = c.lengths(t);
            if oc.lengths(ot) > 0
                oc.widths(ot) = c.widths(t);
                oc.backbones(:,ot) = c.backbones(:,t);
            end
        end
    end
end
warning('on','MATLAB:rankDeficientMatrix');