function computeCellMeasurements_parfor(expInfoObj, posName, mFrames)
% computeCellMeasurements_parfor(expInfoObj, posName, mFrames)
%
% This function computes cell length, width and poles by fitting a 
% second-order curve into the cell backbone. 
% The iterations between segmented phase frames are executed in parallel.
%
% Input:
%   expInfoObj   - expInfo object with various info about the experiment, 
%                  including parameter distanceThreshold that defines 
%                  distance from the border to the inside of a cell. 
%                  Typical value 6.
%   posName      - position name
%   mFrames      - MFrames object that allows to access MCell objects by
%                  frame and blob index. Optional.
%% Initialization
import Cell.Measure.*

if nargin==2
    mCellsMatFile = expInfoObj.getMCellMatPath(posName);
    [~, mFrames] = Cell.MCell.loadMCells(mCellsMatFile);
end

phaseChanName = expInfoObj.getChannelNames('phase');
phaseIndices = expInfoObj.getIndices(posName,phaseChanName);
nFrames = length(phaseIndices);
outputDir = expInfoObj.getPathName('output');
segDir = fullfile(outputDir, posName, 'SegmentedPhase');
segList = dir(fullfile(segDir, '*.tif*'));

% Create dilation disk with a radius defined by the distance threshold
distThreshold = expInfoObj.getParameters('distanceThreshold');
dilationRadius = floor(distThreshold/2);
se = strel('disk', dilationRadius, 8);

%% Prepare cell data for parfor loop
warning('off','MATLAB:rankDeficientMatrix');
totalOutData = cell(1,nFrames);
blobLabels = cell(1,nFrames);
cellsInFrame = cell(1,nFrames);
for fIdx = 1:nFrames
    blobLabels{fIdx} = mFrames.getBlobsInFrame(fIdx);
    nBlobsInFrame = numel(blobLabels{fIdx});
    if nBlobsInFrame>0
        cellsInFrame{fIdx}(1,nBlobsInFrame) = Cell.MCell;
        for bIdx = 1:nBlobsInFrame
            blobLabel = blobLabels{fIdx}(bIdx);
            cellsInBlob = mFrames.getCellsInBlob(fIdx, blobLabel);
            cellsInFrame{fIdx}(bIdx) = cellsInBlob(1);
        end
    end
end
%% Main loop
nCores = expInfoObj.getParameters('parNumWorkers');
parfor(fIdx = 1:nFrames, nCores)
    currSegImage = imread(fullfile(segDir, segList(fIdx).name));
    nBlobsInFrame = numel(blobLabels{fIdx});
    outData = struct;
    outData.majorAxisOrientDegree = nan(1,nBlobsInFrame);
    outData.lengths = zeros(1,nBlobsInFrame);
    outData.widths = zeros(1,nBlobsInFrame);
    outData.poles = cell(1,nBlobsInFrame);
    outData.backbones = zeros(3,nBlobsInFrame);
    for bIdx = 1:nBlobsInFrame
        c = cellsInFrame{fIdx}(bIdx);
        t = c.getIndex(fIdx);

        % Crop the cell image from the frame in a bounding box
        cellIm = cropCell(currSegImage, c, t);
        outData.majorAxisOrientDegree(bIdx) = computeCellOrientation(cellIm,c.isRotated(t));

        if ~c.badSegmentations(t)
            % Compute cell length by fitting a curve into cell backbone
            backbone = computeCellBackbone(cellIm, distThreshold, se);
            outData.lengths(bIdx) = backbone.length;
            
            if backbone.length <= 0
                continue
            end
            
            % Compute cell width and store poles and backbone
            outData.widths(bIdx) = computeCellWidth(cellIm, backbone);
            outData.poles{bIdx} = c.convertFromCellCoord(t, backbone.poles, 1);
            outData.backbones(:,bIdx) = backbone.fitCoeff;
        end
    end
    totalOutData{fIdx} = outData;
end
%% Parse data to mCells
for fIdx = 1:nFrames
    nBlobsInFrame = numel(blobLabels{fIdx});
    outData = totalOutData{fIdx};
    for bIdx = 1:nBlobsInFrame
        blobLabel = blobLabels{fIdx}(bIdx);
        cellsInBlob = mFrames.getCellsInBlob(fIdx, blobLabel);
        c = cellsInBlob(1);
        t = c.getIndex(fIdx);
        c.majorAxisOrientationDegree(t) = outData.majorAxisOrientDegree(bIdx);
        if ~c.badSegmentations(t)
            c.lengths(t) = outData.lengths(bIdx);
            if c.lengths(t) <= 0
                continue
            end
            c.widths(t) = outData.widths(bIdx);
            c.poles{t} = outData.poles{bIdx};
            c.backbones(:,t) = outData.backbones(:,bIdx);
        else
            c.lengths(t) = -1;
        end
        % Copy measurements to the other cells converging in this blob
        for ocIdx = 2:numel(cellsInBlob)
            oc = cellsInBlob(ocIdx);
            ot = oc.getIndex(fIdx);
            oc.lengths(ot) = c.lengths(t);
            if oc.lengths(ot) > 0
                oc.widths(ot) = c.widths(t);
                oc.backbones(:,ot) = c.backbones(:,t);
            end
        end
    end
end
warning('on','MATLAB:rankDeficientMatrix');