function [cellWidth] = computeCellWidth(cellImage, backbone)
% [cellWidth] = computeCellWidth(cellImage, backbone) 
%
% Computes the cell width as follows: 
%   - compute normal vectors to the backbone curve
%   - compute their intersection with the cell
%   - find maximum distance between intersection points.
% 
% Input:
%   cellImage - image of the cell cropped from the segmented image.
%   backbone  - structure containing fields to store the curve
%               coefficients, the length of the cell and the
%               coordinates of the backbone of the cell (in pixels).
%
% Output:
%   cellWidth - measured cell width
%
% Last modified: 2017-09-14, Spartak Zikrin

minCoordX = backbone.coordinates(1,1)+0.5;
maxCoordX = backbone.coordinates(end,1)-0.5;

if minCoordX==maxCoordX
    cellWidth = nan;
    return
end

imSize = size(cellImage);
step = ceil((maxCoordX - minCoordX)/15);


% Compute values of the curve and its derivative in the sampled points
backboneInterval = (minCoordX:step:maxCoordX)';
fVal = backbone.fitCoeff(1)*backboneInterval.^2 + ...
       backbone.fitCoeff(2)*backboneInterval + backbone.fitCoeff(3);
fPrime = 2*backbone.fitCoeff(1)*backboneInterval + backbone.fitCoeff(2);

normLineInterval = (0.5:(imSize(2)+0.5))';

tempCellWidth = -ones(size(backboneInterval));    
idx=1;
for xVal = minCoordX:step:maxCoordX
    if abs(fPrime(idx))>1e-4
        normLineVal = fVal(idx) - (normLineInterval-xVal)/fPrime(idx);
        roundedNormLineVal = round(normLineVal);
        if fPrime(idx)<0
            % Consider case of decreasing normal line
            uppNormLinePix = roundedNormLineVal(2:end);
            roundDownPoint = find(roundedNormLineVal(2:end)-normLineVal(2:end)==0.5);
            uppNormLinePix(roundDownPoint) = uppNormLinePix(roundDownPoint)-1;
            uppNormLinePix = min(imSize(1),uppNormLinePix);
            lowNormLinePix = max(1,roundedNormLineVal(1:end-1));

            % Find the bottom left intersection with the cell
            success=0;
            xIdx = find(uppNormLinePix>0,1,'first');
            while ~success && xIdx<=imSize(2)
                yIdx = find(cellImage(lowNormLinePix(xIdx):uppNormLinePix(xIdx),xIdx),1,'first');
                if ~isempty(yIdx)
                    success = 1;
                    lowestPix = [xIdx yIdx+lowNormLinePix(xIdx)-1];
                else
                    xIdx = xIdx+1;
                end
            end
            if success
                % Find the top right intersection with the cell
                success=0;
                xIdx = find(lowNormLinePix<=imSize(1),1,'last');
                while ~success && xIdx>=lowestPix(1)
                    yIdx = find(cellImage(lowNormLinePix(xIdx):uppNormLinePix(xIdx),xIdx),1,'last');
                    if ~isempty(yIdx)
                        success = 1;
                        highestPix = [xIdx yIdx+lowNormLinePix(xIdx)-1];
                    else
                        xIdx = xIdx-1;
                    end
                end                    
            end
        else
            % Consider case of increasing normal line
            uppNormLinePix = min(imSize(1),roundedNormLineVal(1:end-1));
            lowNormLinePix = max(1,roundedNormLineVal(2:end));

            % Find the bottom right intersection with the cell
            success=0;
            xIdx = find(uppNormLinePix>0,1,'last');
            while ~success && xIdx>0
                yIdx = find(cellImage(lowNormLinePix(xIdx):uppNormLinePix(xIdx),xIdx),1,'first');
                if ~isempty(yIdx)
                    success = 1;
                    lowestPix = [xIdx yIdx+lowNormLinePix(xIdx)-1];
                else
                    xIdx = xIdx-1;
                end
            end
            if success
                % Find the top left intersection with the cell
                success=0;
                xIdx = find(lowNormLinePix(1:lowestPix(1))<=imSize(1),1,'first');
                while ~success && xIdx<=lowestPix(1)
                    yIdx = find(cellImage(lowNormLinePix(xIdx):uppNormLinePix(xIdx),xIdx),1,'last');
                    if ~isempty(yIdx)
                        success = 1;
                        highestPix = [xIdx yIdx+lowNormLinePix(xIdx)-1];
                    else
                        xIdx = xIdx+1;
                    end
                end
            end
        end
    else
        % The normal line is (almost) vertical
        xIdx = round(xVal);
        yIdx = find(cellImage(:,xIdx));
        lowestPix = [xIdx yIdx(1)];
        highestPix = [xIdx yIdx(end)];
        success = 1; 
    end

    if success
        % We found intersections of the normal line with the cell
        % border. Compute the distance between intersections
        if abs(highestPix(1)-lowestPix(1))
            tempCellWidth(idx) = sqrt((highestPix(1)-lowestPix(1))^2 ...
                                          +(highestPix(2)-lowestPix(2)+1)^2);
        else
            tempCellWidth(idx) = abs(highestPix(2)-lowestPix(2))+1;
        end
    end
    idx = idx+1;

end

cellWidth = max(tempCellWidth);
    
end