function convertToInternalCoord(mCell, particleIdx, t)
% convertToInternalCoord(mCell, particleIdx, t) 
%
% Converts the coordinates of the dots from image coordinates to the
% internal coordinates: the first coordinate is the length of the arc
% between one origin and the projection of the dot onto the backbone; the
% second coordinate is the signed euclidean distance between the dot and
% its projection onto the backbone.
%
% Input:
%   mCell - MCell object containing info about a tracked cell
%   particleIdx - index of the tracked particle in mCell
%   t - index of particle detection frame in mCell

% Last modified: 2018-01-19, SZ. Fixed a bug in cell to frame coordinates
% transformation

import Cell.Measure.*

firstPoleCellCoord = mCell.convertToCellCoord(t,mCell.poles{t}(1,:),1);
minCoordX = firstPoleCellCoord(1);

for ii = 1:size(mCell.particles(particleIdx).trajImageCoord, 1)

    currFluoCoord = mCell.particles(particleIdx).trajImageCoord(ii,:);

    % If the coordinates are NaN, set the internal to NaN and move on
    if any(isnan(currFluoCoord))
        mCell.particles(particleIdx).trajInternalCoord(ii,:) = [NaN NaN];
        continue;
    end

    % Convert the coordinates to the phase image and then to the
    % cropped cell image
    currImageCoord = mCell.particles(particleIdx).trajPhaseImageCoord(ii,:);
    currCellCoord = mCell.convertToCellCoord(t, currImageCoord, 1);

    fitCoeff = mCell.backbones(:,t);
    [projectedPoint, yCoord] = getProjectedPoint(currCellCoord, fitCoeff);

    % The x-coordinate is the length of the arc between the origin and
    % the projection
    % The y-coordinate is the Euclidean distance between the point and
    % its projection. Needs to be signed properly.        
    if ~mCell.isReoriented(t)
        xCoord = computeArcLength(fitCoeff, minCoordX, projectedPoint(1));
        if projectedPoint(2) < currCellCoord(2)
            yCoord = -yCoord;
        end
    else
        xCoord = computeArcLength(fitCoeff, projectedPoint(1), minCoordX);
        if projectedPoint(2) >= currCellCoord(2)
            yCoord = -yCoord;
        end
    end

    inCoord = [xCoord yCoord];

    % Normalize the coordinates based on cell width and length
    mCell.particles(particleIdx).trajInternalCoord(ii,:) = ...
        normalizeInternalCoord(mCell.lengths(t), mCell.widths(t), inCoord);
end