function [projectedPoint, distToProj] = getProjectedPoint(pointCoord, fitCoeff)
% [projectedPoint, distToProj] = getProjectedPoint(pointCoord, fitCoeff) 
% compute the projected point from a point in the plane onto a polynomial
% curve. The problem is equivalent to solving a cubic equation.
%
% Inputs
%   pointCoord     : Coordinates of the point to be projected onto the
%                    curve.
%   fitCoeff       : Coefficients of the 2nd order curve that fits backbone 
%                    of the cell.
%
% Outputs
%   projectedPoint : Coordinates of the projected point pointCoord onto the
%                    curve.
%   distToProj     : Distance between pointCoord and its projection.
%
% Last modified: 2017-07-03, Spartak Zikrin

p1=fitCoeff(1);
p2=fitCoeff(2);
p3=fitCoeff(3);
x0 = pointCoord(1);
y0 = pointCoord(2);
if abs(p1)>1e-10    
    xStar = roots([2*p1^2 3*p1*p2 2*p1*(p3-y0)+p2^2+1 p2*(p3-y0)-x0]);
    xStar(abs(imag(xStar))<1e-10) = real(xStar(abs(imag(xStar))<1e-10));
    xStar = xStar(~imag(xStar));
    xMid = -p2/(2*p1);
    if x0 > xMid
        projectedPoint(1) = max(xStar);
    else
        projectedPoint(1) = min(xStar);
    end
else
    projectedPoint(1) = x0 - p2*(p3-p2)/(p2^2+1);
end
projectedPoint(2) = p1*projectedPoint(1)^2 + p2*projectedPoint(1) + p3;
distToProj = sqrt(sum((pointCoord - projectedPoint).^2));

end