function orientCellPoles(mCells)
% orientCellPoles(mCells)
%
% This function identifies children cells, checks whether
% their new poles are the closest and swaps the poles if required.
% The new poles are created as a result of cell division. Cell inner 
% coordinate system's x axis should always be directed from the old pole
% to the new pole.
%
% Input:
%   mCells - array of MCell objects containing info about the cells.
%            MCell property *poles* is a cell array of 2x2 matrices, which
%            store the old and new cell pole coordinates in the first and 
%            second rows, respectively. 
%
% Last modified: Spartak Zikrin, 2017-09-14

for cellIdx=1:length(mCells)
    c = mCells(cellIdx);
    if isempty(c.parent)
        c.alignCellPoles();
    end
    if ~isempty(c.descendants)        
        c1 = c.descendants(1);
        c2 = c.descendants(2);
        if c1.lengths(1) > 0 && c2.lengths(1) > 0
            % Find the nearest pair of poles
            distPoles = pdist2cust(c1.poles{1}, c2.poles{1});
            [~, idx] = min(distPoles(:));
            switch idx
                case 1 
                    % Old poles are the closest
                    c1.swapCellPoles(1);
                    c2.swapCellPoles(1);
                case 2 
                    % New pole in cell 1 and old pole in cell 2,
                    c2.swapCellPoles(1);
                case 3 
                    % Old pole in cell 1 and new pole in cell 2
                    c1.swapCellPoles(1);
                % case 4 - new poles are the closest, do nothing
            end
            c1.alignCellPoles();
            c2.alignCellPoles();
        end
    end
end

end