function oList = AppearanceScores(nBlobsPerFrame, parameters)
% Computes scores for events where new cells appear in the image.
%
% The cells can either appear in random places in the image or migrate into
% the field of view. In practice, a cell can appear randomly if it falls
% into focus from suspension or if it is washed into the field of view
% after a media change. The score of an appearance event for a cell is the
% logarithm of the probability that the cell will was not present in the
% previous image.
%
% Inputs:
% nBlobsPerFrame - array, element t is the number of detections in frame t.
% parameters - struct with field pAppear (probability of random appearance)
%
% Outputs:
% oList - N x 4 matrix, where N is the number of returned appearance
%         events. The elements of the matrix are:
%    oList(:,1) - Image index.
%    oList(:,2) - Index of the detection in image oList(:,1).
%    oList(:,3) - Log-probability of the appearance event NOT occurring.
%    oList(:,4) - Log-probability of the appearance event occurring.
%
% Last modified: Spartak Zikrin, 2018-01-12

pAppear = parameters.pAppear;
if ~pAppear
    % There is no mechanism by which cells can disappear, so an empty list
    % is returned.
    oList = zeros(0,4);
    return
end

appearProbs = [1-pAppear pAppear];
appearLogProbs = max(log(appearProbs), -1E6);

% There is no need to find death scores in the last frame.
nSegs = sum(nBlobsPerFrame(2:end));
oList = nan(nSegs,4);
oList(:,3:end) = repmat(appearLogProbs,nSegs,1);
startIndex = 1;
for frameIdx = 2:numel(nBlobsPerFrame)
    lastIndex = startIndex+nBlobsPerFrame(frameIdx)-1;
    oList(startIndex:lastIndex,1) = frameIdx;
    oList(startIndex:lastIndex,2) = (1:nBlobsPerFrame(frameIdx))';
    startIndex = lastIndex+1;
end

end