function oList = CountScores(nBlobsPerFrame, parameters)
% Computes scores for different cell counts in blobs of an image sequence.
%
% The scores are the log-probabilities of the different cell counts.
% The provided settings pCnt0, pCnt1, and pCnt2 define fixed probabilities
% that a blob has 0, 1, or more than 1 cell respectively.Then the 
% likelihoods of 3 to 9 cells is extrapolated from the probability of 3 or 
% more cells. The extrapolation is performed by assuming that the 
% probabilities of different cell counts follow a geometric distribution, 
% as described in [1].
%
% Inputs:
% nBlobsPerFrame - array, element t is the number of detections in frame t.
% parameters - structure with fields pCnt0, pCnt1, pCnt3, pCntExtrap.
%
% Outputs:
% oList- N x 12 matrix where N is the total number of blobs in aBlobSeq.
%        The elements of the matrix are:
%    oList(:,1) - Frame index.
%    oList(:,2) - Index of the blob in frame oList(:,1).
%    oList(:,3:12) - Log-probability that the blob contains 0 to 9 cells.
%
% References:
% [1] Magnusson, K. E. G.; Jald???n, J.; Gilbert, P. M. & Blau, H. M. Global
%     linking of cell tracks using the Viterbi algorithm IEEE Trans. Med.
%     Imag., 2015, 34, 1-19
%
% Last modified: Spartak Zikrin, 2018-01-12

import Cell.Scores.ExtrapProbs

% Use fixed probabilities from settings.
countProbs = [parameters.pCnt0 parameters.pCnt1 parameters.pCnt2];

% Extrapolate up to a cell count of 9.
countProbs = ExtrapProbs(countProbs, 10, parameters.pCntExtrap);
countLogProbs = max(log(countProbs), -1E6);

nSegs = sum(nBlobsPerFrame);
oList = nan(nSegs,size(countProbs,2)+2);
oList(:,3:end) = repmat(countLogProbs,nSegs,1);
startIndex = 1;
for frameIdx = 1:numel(nBlobsPerFrame)
    lastIndex = startIndex+nBlobsPerFrame(frameIdx)-1;
    oList(startIndex:lastIndex,1) = frameIdx;
    oList(startIndex:lastIndex,2) = (1:nBlobsPerFrame(frameIdx))';
    startIndex = lastIndex+1;
end

end