function oList = DeathScores(nBlobsPerFrame, parameters)
% Computes scores for death events in blobs of an image sequence.
%
% The scores are log-probabilities of death events. The log-probabilities
% are set to a fixed value by specifying the setting pDeath. One can also 
% specify the setting deathShift which is a factor that multiplies the 
% death probabilities.
%
% Inputs:
% nBlobsPerFrame - array, element t is the number of detections in frame t.
% parameters - structure with fields pDeath, deathShift

% Outputs:
% oList - N x 4 matrix where N is the total number of blobs in all frames
%         except the last. Only possible death events are included in the
%         list, so if the death probability is set to 0, an empty list will
%         be returned.
%         The elements of the matrix are:
%    oList(:,1) - Frame index.
%    oList(:,2) - Index of the blob in frame oList(:,1).
%    oList(:,3) - Log-probability that no death event takes place in the
%                 blob.
%    oList(:,4) - Log-probability that at least one death event takes place
%                 in the blob.
%
%
% Last modified: Spartak Zikrin, 2018-02-12

pDeath = parameters.pDeath;
if ~pDeath
    % If the death probability is set to 0, we do not add any death
    % events at all.
    oList = zeros(0,4);
    return
end

deathProbs = nan(1,2);
deathProbs(2) = pDeath * parameters.deathShift;
deathProbs(1) = 1-deathProbs(2);
deathLogProbs = max(log(deathProbs), -1E6);

% There is no need to find death scores in the last frame.
nSegs = sum(nBlobsPerFrame(1:end-1));
oList = nan(nSegs,4);
oList(:,3:end) = repmat(deathLogProbs,nSegs,1);
startIndex = 1;
for frameIdx = 1:numel(nBlobsPerFrame)-1
    lastIndex = startIndex+nBlobsPerFrame(frameIdx)-1;
    oList(startIndex:lastIndex,1) = frameIdx;
    oList(startIndex:lastIndex,2) = (1:nBlobsPerFrame(frameIdx))';
    startIndex = lastIndex+1;
end

end