function oList = DisappearanceScores(nBlobsPerFrame, parameters)
% Computes scores for events where cells disappear from the image.
%
% The cells can either disappear randomly or migrate out of the field of
% view. In practice, a cell can disappear randomly if it is washed out of
% the field of view during a media change or if it becomes impossible to
% detect due to for example photo bleaching. The score of a disappearance
% event for a cell is the logarithm of the probability that the cell will
% not be present in the following image.
%
% Inputs:
% nBlobsPerFrame - array, element t is the number of detections in frame t.
% parameters - structure with fields pDisappear (probability of random
%              disappearance)
%
% Outputs:
% oList - N x 4 matrix, where N is the number of returned disappearance
%         events. The elements of the matrix are:
%    oList(:,1) - Image index.
%    oList(:,2) - Index of the detection in image oList(:,1).
%    oList(:,3) - Log-probability of the disappearance event NOT occurring.
%    oList(:,4) - Log-probability of the disappearance event occurring.
%
% Last modified: Spartak Zikrin, 2018-01-12

pDisappear = parameters.pDisappear;
if ~pDisappear
    % There is no mechanism by which cells can disappear, so an empty list
    % is returned.
    oList = zeros(0,4);
    return
end

disappearProbs = [1-pDisappear pDisappear];
disappearLogProbs = max(log(disappearProbs), -1E6);

% There is no need to find death scores in the last frame.
nSegs = sum(nBlobsPerFrame(1:end-1));
oList = nan(nSegs,4);
oList(:,3:end) = repmat(disappearLogProbs,nSegs,1);
startIndex = 1;
for frameIdx = 1:numel(nBlobsPerFrame)-1
    lastIndex = startIndex+nBlobsPerFrame(frameIdx)-1;
    oList(startIndex:lastIndex,1) = frameIdx;
    oList(startIndex:lastIndex,2) = (1:nBlobsPerFrame(frameIdx))';
    startIndex = lastIndex+1;
end

end