function oList = Jaccard(im,imNext,blobs,blobsNext,t)
% oList = Jaccard(im,imNext,blobs,blobsNext,t)
% Computes intersection areas and the Jaccard similarity index between
% blobs in two adjacent frames with the use of regionprops.
%
% The Jaccard similarity index is the intersection of the binary masks,
% divided by the union of the binary masks. The Jaccard similarity index
% can be used as a measure of segmentation accuracy and as a feature or
% likelihood for classification of migration events. The measure is
% symmetric in the two input blobs. The Jaccard index can be raised to the
% power of exponent to account for the fast moving cells with smaller
% overlap.
%
% Input:
%   im - segmented image in frame t
%   imNext - segmented image in frame t+1
%   blobs - cell array of length M. In case of mother-machine, M is the
%           number of growth channels, otherwise M=1. Each cell contains a
%           structure array of blob properties in image im in the
%           corresponding growth channel returned by Labels2Blobs
%   blobsNext - cell array of structure arrays of blob properties in
%               image imNext returned by Labels2Blobs.
%   t - number of acquisition frame of image im.
%
% Output:
%   oList - 1 x M cell array, where each cell contains N x 5 matrix.
%           N is the number of overlapping blobs in the corresponding
%           growth channel and the columns are:
%               1 - Frame count of the first detection in the migration.
%               2 - The index of the detection in image im.
%               3 - The index of the detection in image imNext.
%               4 - The Jaccard similarity index of overlapping blobs.
%               5 - The intersection area of overlapping blobs.
%
% See also: Labels2Blobs

props = regionprops(im,imNext,'PixelValues');

% Create matrix to store pairwise blob intersection areas.
I = zeros(max(max(im)),max(max(imNext)));

for i=1:length(props)
    labels = unique(props(i).PixelValues);
    labels = labels(labels>0);
    for j=1:length(labels)
        I(i,labels(j)) = sum(props(i).PixelValues == labels(j));
    end
end

nChannels = length(blobs);
oList = cell(1,nChannels);
for chIdx=1:nChannels
    bs = blobs{chIdx};
    bsNext = blobsNext{chIdx};
    bsNextLabels = [bsNext.Label];
    maxRow = length(bs)*4;
    tmpList = nan(maxRow,5);
    cnt=1;
    for i = 1:length(bs)
        b1 = bs(i);
        labels = find(I(b1.Label,:));
        for j = 1:length(labels)
            i2 = find(bsNextLabels==labels(j),1);
            if isempty(i2)
                continue, 
            end
            b2 = bsNext(i2);
            intersection = I(b1.Label,b2.Label);
            if intersection > 0
                union = b1.Area+b2.Area-intersection;
                jac = intersection / union;
                tmpList(cnt,:) = [t i i2 jac intersection];
                cnt = cnt+1;
            end
        end
    end
    tmpList(cnt:end,:) = [];
    oList{chIdx} = tmpList;
end