function blobs = Labels2Blobs(im, channels)
% blobs = computeBlobs(im, channels)
% 
% Computes the following properties for each blob in segmented image: 
% blob label, bounding box, area and centroid.
%
% Input:
%   im - segmented image
%   channels - matrix where each row is a 4-element vector with the form
%              [XMIN YMIN WIDTH HEIGHT] that defines the bounding box of a
%              growth channel i. Optional.
%
% Output:
%   blobs - structure array where each element contains properties of a 
%           corresponding blob.  If mother-machine, blobs is a cell array
%           where cell i contains structure array with properties of blobs
%           in growth channel i. Structure fields:
%               Label - the pixel value of the blob in segmented image
%               BoundingBox - the bounding box of the blob image in the
%                             format returned by the function regionprops.
%               Area - the area of the blob in pixels.
%               Centroid - the centroid of the blob pixel region given in
%                          coordinates of the full image.
%          

allBlobs = regionprops(im,'BoundingBox','Centroid','Area','PixelIdxList');

% Remove properties corresponding to missing labels.
allBlobs = allBlobs([allBlobs.Area] > 0);

% Add blob labels and boundaries
[B,L]=bwboundaries(im,4,'noholes');
for i = 1:length(allBlobs)
    allBlobs(i).Label = im(allBlobs(i).PixelIdxList(1));
    idx = L(allBlobs(i).PixelIdxList(1));
    b = B{idx};
    bbox = round(allBlobs(i).BoundingBox); 
    allBlobs(i).Boundary = b(:,[2 1])-bbox(1:2)+1;
end

% Property PixelIdxList was only used to compute blob labels and boundary.
allBlobs = rmfield(allBlobs,'PixelIdxList');

if nargin>1 && ~isempty(channels)
    % Assign blobs to growth channels
    BB = reshape([allBlobs.BoundingBox],4,length(allBlobs))';
%     blobs = cell(1,length(channels));
    blobs = cell(1,size(channels,1));
    for t = 1:size(channels,1)
        areas = rectint(BB,channels(t,:));
        blobs{t} = allBlobs(areas>0);
    end
else
    blobs = {allBlobs};
end