function oList = MergeScores(blobs, migList, parameters)
% Computes a list of cell merge scores.
%
% The function computes mitosis scores for all mitotic events that can take
% place in an image sequence. For each detection, all possible pairs of
% outgoing migrations, given by the list of migration events returned by
% MigrationScores, give rise to mitotic events. In this process,
% migrations are paired with themselves, meaning that the daughter cells
% can end up in the same detection after the mitotic event. The scores are 
% taken to be the log-probabilities of mitosis. 
% Merge scores are based on Jaccard index between two cells in the previous 
% frame and a cell in the current frame.
%
% Input:
%   blobs - cell array, each element is a structure array with properties
%           of blobs which were segmented in the corresponding frame.
%   migList - List of migration scores returned by MigrationScores. Only
%             the first three columns with indices of frames and blobs, as
%             well as the sixth column with intersection areas are used.
%             The actual scores are not needed. The indices are used to
%             determine which mitosis events to return scores for.
%   parameters - structure with fields
%     splitJaccardMin - lower bound on Jaccard index for split event
%     splitJaccardMax - upper bound on Jaccard index and split probability
%     splitJaccardPreCond - scaling factor to compute merging probability
%                           from the Jaccard index
%     minScore - Score threshold below which migrations will not be
%                included in the output.
%
% Output:
%   oList - N x 6 matrix, where N is the number of returned merging events.
%           The elements of the matrix are:
%     oList(:,1) - Frame index of the first detection in the mitotic event.
%     oList(:,2) - Index of the child blob in image oList(:,1)+1.
%     oList(:,3) - Index of the first parent blob in image oList(:,1).
%     oList(:,4) - Index of the second parent blob in image oList(:,1).
%     oList(:,5) - Log-likelihood of the merging event NOT occurring.
%     oList(:,6) - Log-likelihood of the merging event occurring.
%
% See also: MigrationScores, SplitScores
%
% Last modified: Spartak Zikrin, 2020-05-20. Use precomputed intersection
% areas for computing the Jaccard index

if ~size(migList,1)
    % There are no possible migrations and therefore no possible mitotic
    % events.
    oList = zeros(0,6);
    return
end

% Sort the migration list on the time points, the index of the first
% blob, and the index of the second blob, in that priority order.
for i = [2, 3, 1]
    [~, order] = sort(migList(:,i));
    migList = migList(order,:);
end

% There is no need to find mitosis scores in the last frame.
% Allocate an output list that may be too long.
maxMig = 6;
nBlobsPerFrame = cellfun(@length,blobs);
oList = zeros(sum(nBlobsPerFrame(1:end-1))*maxMig, 6);

splitJaccardPreCond = parameters.splitJaccardPreCond;
splitJaccardMin     = parameters.splitJaccardMin;
splitJaccardMax     = parameters.splitJaccardMax;
minScore            = parameters.minScore;

cnt = 1;  % Index of row in oList.
t = nan;
child = nan;
for i = 1:size(migList,1)
    if child ~= migList(i,3) || t ~= migList(i,1)
        % We have gotten to the next parent cell (and row in mergeScores).
        t = migList(i,1);
        child = migList(i,3);
    end
    
    parent1 = migList(i,2);
    
    % Go through all blobs that can be the second child and store scores
    % for the corresponding mitotic events.
    j = i + 1;
    while j <= size(migList,1) && migList(j,1) == t && migList(j,3) == child
        parent2 = migList(j,2);
        childBlob = blobs{t+1}(child);
        parent1Blob = blobs{t}(parent1);
        parent2Blob = blobs{t}(parent2);
        
        intersect = migList(i,6)+migList(j,6);
        union = childBlob.Area+parent1Blob.Area+parent2Blob.Area-intersect;
        jac = intersect / union;
        
        if jac > splitJaccardMin
            jac = jac * splitJaccardPreCond;
            p = min(jac, splitJaccardMax);
            
            oList(cnt,:) = [t, child, parent1, parent2, 1-p, p];
            cnt = cnt + 1;
        end
        j = j + 1;
    end
end

% Crop away unused rows from the output list and compute log-probabilies.
oList(cnt:end,:) = [];
oList(:,5:6) = log(oList(:,5:6));

% Remove mitosis events with too low scores.
scoreDiff = oList(:,6) - oList(:,5);
oList(scoreDiff < minScore, :) = [];