function oList = MigrationScores(jaccardList,parameters)
% Computes migraiton probabilities using the precomputed Jaccard similarity
% index.
%
% The probability of migration between two blobs is assumed to be the
% Jaccard similarity index of the two blob regions. By default, only
% migrations with a probability above 0.1 are included in the returned
% list, and the probabilities are thresholded at 0.99. This function can be
% used to compute migration probabilities when the displacements are so
% the outlines of a cell always overlap in consecutive images.
%
% Input:
%   jaccardList - matrix with Jaccard indices of blob intersections
%                 returned by the function Jaccard.
%   parameters - structure with fields:
%     migJaccardMin - lower bound on probability based on Jaccard index
%     migJaccardMax - upper bound on probability based on Jaccard index
%     migJaccardExp - raise Jaccard index to the power of migJaccardExp.
%
% Output:
%   oList - N x 5 matrix, where N is the number of returned migrations.
%     oList(:,1) - Frame count of the first detection in the migration.
%     oList(:,2) - The index of the detection in image oList(:,1).
%     oList(:,3) - The index of the detection in image oList(:,1)+1.
%     oList(:,4) - Log probability of the migration NOT occuring.
%     oList(:,5) - Log probability of the migration occuring.
%
% See also: Jaccard
%
% Last modified: Spartak Zikrin, 2020-05-20

if parameters.migJaccardMin > 0
    MIN_PROB = parameters.migJaccardMin;
else
    MIN_PROB = 0.1;
end
if parameters.migJaccardMax > 0
    MAX_PROB = parameters.migJaccardMax;
else
    MAX_PROB = 0.99;
end
if parameters.migJaccardExp > 0
    EXP = parameters.migJaccardExp;
else
    EXP = 1;
end

P = jaccardList(:,4).^EXP;
ind = P>MIN_PROB;
P = min(P(ind),MAX_PROB);

oList = [jaccardList(ind,1:3) log(1-P) log(P) jaccardList(ind,5)];