function oList = SplitScores(blobs, migList, parameters)
% Computes a list of cell division scores.
%
% The function computes mitosis scores for all mitotic events that can take
% place in an image sequence. For each detection, all possible pairs of
% outgoing migrations, given by the list of migration events returned by
% MigrationScores, give rise to mitotic events. In this process,
% migrations are paired with themselves, meaning that the daughter cells
% can end up in the same detection after the mitotic event. The scores are 
% taken to be the log-probabilities of mitosis. 
% Split scores are based on Jaccard index between the mother cell and 
% the daughter cells.
%
% Input:
%   blobs - cell array, each element is a structure array with properties
%           of blobs which were segmented in the corresponding frame.
%   migList - List of migration scores returned by MigrationScores. Only
%             the first three columns with indices of frames and blobs, as
%             well as the sixth column with intersection areas are used.
%             The actual scores are not needed. The indices are used to
%             determine which mitosis events to return scores for.
%   parameters - structure with fields
%     splitJaccardMin - lower bound on Jaccard index for split event
%     splitJaccardMax - upper bound on Jaccard index and split probability
%     splitJaccardPreCond - scaling factor to compute split probability
%                           from the Jaccard index
%     minScore - Score threshold below which migrations will not be
%                included in the output.
%
% Output:
%   oList - N x 6 matrix, where N is the number of returned mitotic events.
%           The elements of the matrix are:
%     oList(:,1) - Frame index of the first detection in the mitotic event.
%     oList(:,2) - Index of the parent blob in image oList(:,1).
%     oList(:,3) - Index of the first child blob in image oList(:,1)+1.
%     oList(:,4) - Index of the second child blob in image oList(:,1)+1.
%                  This can be equal to the index of the first child
%                  detection.
%     oList(:,5) - Log-likelihood of the split event NOT occurring.
%     oList(:,6) - Log-likelihood of the split event occurring.
%
% See also: MigrationScores, MergeScores
%
% Last modified: Spartak Zikrin, 2020-05-20. Use precomputed intersection
% areas for computing the Jaccard index

if ~size(migList,1)
    % There are no possible migrations and therefore no possible mitotic
    % events.
    oList = zeros(0,6);
    return
end

% Sort the migration list on the time points, the index of the first
% blob, and the index of the second blob, in that priority order.
% for i = 3:-1:1
%     [~, order] = sort(migList(:,i));
%     migList = migList(order,:);
% end

% There is no need to find mitosis scores in the last frame.
% Allocate an output list that may be too long.
maxMig = 6;
nBlobsPerFrame = cellfun(@length,blobs);
oList = zeros(sum(nBlobsPerFrame(end-1))*maxMig, 6);

splitJaccardPreCond = parameters.splitJaccardPreCond;
splitJaccardMin     = parameters.splitJaccardMin;
splitJaccardMax     = parameters.splitJaccardMax;
minScore            = parameters.minScore;

cnt = 1;  % Index of row in oList.
t = nan;
parent = nan;
for i = 1:size(migList,1)
    if parent ~= migList(i,2) || t ~= migList(i,1)
        % We have gotten to the next parent cell (and row in splitScores).
        t = migList(i,1);
        parent = migList(i,2);
    end
    
    child1 = migList(i,3);
    
    % Go through all blobs that can be the second child and store scores
    % for the corresponding mitotic events.
    j = i + 1;
    while j <= size(migList,1) && migList(j,1) == t && migList(j,2) == parent
        child2 = migList(j,3);
        parentBlob = blobs{t}(parent);
        child1Blob = blobs{t+1}(child1);
        child2Blob = blobs{t+1}(child2);
        intersect = migList(i,6)+migList(j,6);
        union = parentBlob.Area+child1Blob.Area+child2Blob.Area-intersect;
        jac = intersect / union;

        if jac > splitJaccardMin

            jac = splitJaccardPreCond * jac;
            p = min(jac, splitJaccardMax);

            % The mitosis scores only depend on the parent cells. Migration
            % scores from the parent cell to the two daughter cells are
            % added in ViterbiTrackLinking.

            oList(cnt,:) = [t, parent, child1, child2, 1-p, p];
            cnt = cnt + 1;
        end
        j = j + 1;
    end
end

% Crop away unused rows from the output list and compute log-probabilies.
oList = oList(1:cnt-1,:);
oList(:,5:6) = log(oList(:,5:6));

% Remove mitosis events with too low scores.
scoreDiff = oList(:,6) - oList(:,5);
oList(scoreDiff < minScore, :) = [];