function [oBlobs, oList] = computeBlobs(imageList,channels, nCores)
% Wrapper to compute properties of segmented blobs in the images defined by
% imageList. Handle the case of mother-machine, when blobs for each growth
% channel are stored individually. 
% Additionally, intersection areas and the Jaccard index of overlapping
% blobs in adjacent frames are computed. This data will be used to compute
% score matrices for the Viterbi Linking algorithm for cell tracking.
%
% Input:
%   imageList - list of segmented images returned by dir.
%   channels - M x 4 x N matrix where M is the number of growth channels,
%              N is the number of frames. Element (i,:,j) is a rect
%              4-vector with coordinates of bounding box of channel i in
%              frame j. If not mother-machine, channels = [].
%   nCores - number of parallel workers.
%
% Output:
%   oBlobs - cell array where element (i,j) is a structure array with blob
%            properties in frame i and growth channel j returned by 
%            Labels2Blobs. If not mother-machine, number of columns is 1.
%            The blob properties are:
%               Label - the pixel value of blob in segmented image
%               BoundingBox - the bounding box of the blob image in the
%                             format returned by the function regionprops.
%               Area - the area of the blob in pixels.
%               Centroid - the centroid of the blob pixel region given in
%                          coordinates of the full image.
%   oList - cell array where cell j contains a matrix with Jaccard indices
%           of blob intersections in growth channel j returned by Jaccard.
%           If not mother-machine, oList is a matrix with Jaccard indices
%           of blob intersections in growth channel j returned by Jaccard.
%           Matrix columns are:
%               1 - Frame count t.
%               2 - The index of the detection in frame t.
%               3 - The index of the detection in frame t+1.
%               4 - The Jaccard similarity index of overlapping blobs.
%               5 - The intersection area of overlapping blobs.
%
% See also: Jaccard, Labels2Blobs

    nFrames = length(imageList);
    if isempty(channels)
        nChannels=1;
    else
        nChannels = size(channels,1);
    end

    if nCores>0
        nFramesPerJob=ceil(nFrames/nCores);
        tmpBlobs = cell(nCores,1);
        tmpJaccardList = cell(nCores,1);
        parfor(i=1:nCores, nCores)
            frames = nFramesPerJob*(i-1)+1 : min(nFramesPerJob*i,nFrames-1);
            [tmpBlobs{i}, tmpJaccardList{i}] = computeBlobsSingleCore(imageList,channels,frames);
        end
        oBlobs = cell(nFrames,nChannels);
        JaccardList = cell(nFrames-1,nChannels);
        for i=1:nCores
            ind1 = nFramesPerJob*(i-1) + (1:size(tmpBlobs{i},1));
            ind2 = nFramesPerJob*(i-1) + (1:size(tmpJaccardList{i},1));
            oBlobs(ind1,:) = tmpBlobs{i};
            JaccardList(ind2,:) = tmpJaccardList{i};
        end
    else
        [oBlobs, JaccardList] = computeBlobsSingleCore(imageList,channels,1:nFrames-1);
    end

    if nChannels==1
        oList = cell2mat(JaccardList);
    else
        oList = cell(1,nChannels);
        for ch=1:nChannels
            oList{ch} = cell2mat(JaccardList(:,ch));
        end
    end
end

%---------------------------------------- 
function [oBlobs, jacList] = computeBlobsSingleCore(imageList,channels,frames)

    nAllFrames = length(imageList);
    nFrames = length(frames);

    if isempty(channels)
        channels = nan(1,4,nAllFrames);
    end

    nChannels = size(channels,1);
    if max(frames) == nAllFrames-1
        oBlobs = cell(nFrames+1,nChannels);
    else
        oBlobs = cell(nFrames,nChannels);
    end
    jacList = cell(nFrames,nChannels);

    t = frames(1);
    im = tiffread(fullfile(imageList(t+1).folder,imageList(t).name));
    blobs = Cell.Scores.Labels2Blobs(im,channels(:,:,t));
    for i = 1:length(frames)
        t = frames(i);
        imNext = tiffread(fullfile(imageList(t+1).folder,imageList(t+1).name));
        blobsNext = Cell.Scores.Labels2Blobs(imNext,channels(:,:,t+1));
        jacList(i,:) = Cell.Scores.Jaccard(im,imNext,blobs,blobsNext,t);
        oBlobs(i,:) = blobs;
        blobs = blobsNext;
        im = imNext;
    end

    % Store blobs in the last frame
    if max(frames) == nAllFrames-1
        oBlobs(nFrames+1,:) = blobsNext;
    end
end