function findAndLabelConvergingTracks(mFrames)
% findAndLabelConvergintracks(mFrames)
%
% The function looks for cases where two cells share 
% the same segmented blob. This occurs if Baxter
% is allowed to place two cells in the same segmenation
% region.
% 
% Converging tracks are marked by setting 
%   MCell.badSegmentations(frameIdx) = 1 for the blob in question.
%
% If any of the two tracks starts or ends in that blob,
%   MCell.isBadCell = 1;
%
% Input:
%   mFrames - MFrames object that allows to access MCell objects by frame 
%             and blob index

% Last modified: Spartak Zikrin, 2018-03-23. Fixed wrong labelling (5)
% 2020-04-24, SZ. Label cells only if one of them starts or ends in
% a shared blob.

import Cell.Track.Curate.computeJaccardIndex

for frameIdx = 1:mFrames.getNumOfFrames()
    sharedBlobs = mFrames.getSharedBlobs(frameIdx);
    for blobLabel = sharedBlobs
        cellsInBlob = mFrames.getCellsInBlob(frameIdx, blobLabel);
        c1 = cellsInBlob(1);
        c2 = cellsInBlob(2);
        idx11 = c1.getIndex(frameIdx);
        idx21 = c2.getIndex(frameIdx);
        
        if c1.badSegmentations(idx11) == 1 && c2.badSegmentations(idx21) == 1
            continue
        end
            
        nSharedFrames = min(c1.lifeTime-idx11+1, c2.lifeTime-idx21+1);
        blobLabels1 = c1.blobLabels(idx11:idx11+nSharedFrames-1);
        blobLabels2 = c2.blobLabels(idx21:idx21+nSharedFrames-1);

        nSharedBlobs = find(blobLabels1 ~= blobLabels2,1);
        if ~isempty(nSharedBlobs)
            nSharedBlobs = nSharedBlobs - 1;
        else
            nSharedBlobs = nSharedFrames;
        end

        idx12 = idx11 + nSharedBlobs - 1;
        idx22 = idx21 + nSharedBlobs - 1;

        c1.badSegmentations(idx11:idx12) = 1;
        c2.badSegmentations(idx21:idx22) = 1;

        if idx11 > 1 && idx12 < c1.lifeTime && ...
           idx21 > 1 && idx22 < c2.lifeTime

            blob11 = c1.getBlob(idx11-1);
            blob12= c1.getBlob(idx12+1);
            blob22 = c2.getBlob(idx22+1);

            currJaccardIndex = computeJaccardIndex(blob11,blob12);
            newJaccardIndex  = computeJaccardIndex(blob11,blob22);

            if newJaccardIndex > currJaccardIndex
                cellTail1 = c1.split(idx12);
                cellTail2 = c2.split(idx22);
                c1.addCell(cellTail2);
                c2.addCell(cellTail1);
            end
        else
            c1.isBadCell = 1;
            c2.isBadCell = 1;
        end
    end
end