function findAndLabelMisSegmentation(mCells, maxGrowthRate)
% findAndLabelMisSegmentation(mCells, maxGrowthRate)
% 
%  This function will identify all cells that oscillate in
%  in their segmented area. This is quite common close to 
%  division when the segmentation algorithm have problems
%  defining exactly when the cells split.
% 
%  The idea is to identify when the area ratio between cells in consecutive
%  frames is above maxGrowthRate. When this happends there are difference
%  scenarios. First, the area ratio may change again in later frame within
%  the same generation. In this case all segmentations between the two
%  shifts are marked as bad. Second, we check the area ratio between the
%  last frame before the shift and the first frame after the shift. If it
%  is below maxGrowthRate, label the remaining cell track as bad. Third,
%  if there is no second shift in the area ratio until cell division we
%  don't know what to trust and mark the entire cell generation as bad.
% 
%  Bad segmentation is marked setting mCell.badSegmentation(frameIdx) = 2
%  Bad cells are marked by setting mCell.isBadCell = 2
%  
% Input arguments:
%   mCells        - array of MCell objects representing tracked cells.
%   maxGrowthRate - defines the highest allowed area ratio between
%                   two migrating cells in consecutive frames.

% Last modified: SZ, 2018-03-05. Compare area ratio instead of the
% Jaccard index which is expensive to compute

import Cell.Track.Curate.computeJaccardIndex

for i = 1:length(mCells)
    c = mCells(i);
    
    if c.badSegmentations(1) == 1
        startIdx = find(c.badSegmentations == 0, 1);
    else
        startIdx = 1;
    end
    
    growthRates = c.areas(startIdx+1:end)./c.areas(startIdx:end-1);
    
    shiftIdx = false(1,c.lifeTime);
    shiftIdx(startIdx:end-1) = growthRates > maxGrowthRate | growthRates < 1/maxGrowthRate;
    if ~any(shiftIdx)
        continue;
    end

    if mod(sum(shiftIdx),2)
        shiftIdx(end) = true;
        if ~c.isBadCell
            c.isBadCell = 2;
        end
    end
    shiftIdx = find(shiftIdx);
        
    nBadSubtracks  = numel(shiftIdx)/2;
    labelIdx = cell(1,nBadSubtracks);
    for j=1:nBadSubtracks
        k = 2*j-1;
        % Compare cell areas in the last frame before the badly segmented
        % subtrack and in the first frame after
        if shiftIdx(k+1)+1 <= c.lifeTime
            areaRatio = c.areas(shiftIdx(k))/c.areas(shiftIdx(k+1)+1);
            if areaRatio > maxGrowthRate || areaRatio < 1/maxGrowthRate
                % Label the rest as bad
                labelIdx{j} = shiftIdx(k)+1:c.lifeTime;
                if ~c.isBadCell
                    c.isBadCell = 2;
                end
                break
            end
        end
        labelIdx{j} = shiftIdx(k)+1:shiftIdx(k+1);
    end
    labelIdx = cell2mat(labelIdx);
    
    % Avoid relabeling
    isLabeled = c.badSegmentations(labelIdx)>0;
    labelIdx(isLabeled) = [];
    
    c.badSegmentations(labelIdx) = 2;
end