function findAndLabelRapidCMMovements(mCells, pxCutOff)
% findAndLabelRapidCMMovements(mCells, pxCutOff)
%
% This function is used to label cells whose centroid trajectory is
% moving too rapidly. Also, it checks the distance between the parent cell
% centroid in its last frame and the children cell centroids in the
% following frame and deletes parent-children link if the distance is above
% a treshold.
%
% Input:
%   mCells   - array of MCell objects representing cells
%   pxCutOff - treshold, used to label a cell if the Euclidean distance 
%              between its two consequitive centroids is above pxCutOff'

% Last modified: 2018-03-09, SZ. Compute average CMM in the frames with bad
% segmentation labels.

for i = 1:length(mCells)
    c = mCells(i);

    if ~c.isBadCell
        cmTraj = c.centroids;
        badIdxs = [c.badSegmentations > 0, false];
        if badIdxs(1)
            cmTraj = cmTraj(find(~badIdxs,1):end,:);
            badIdxs = badIdxs(find(~badIdxs,1):end);
        end
        startIdx = find(badIdxs,1,'first');
        while ~isempty(startIdx)
            endIdx = startIdx+find(~badIdxs(startIdx:end),1,'first')-2;
            xShift = mean(diff(cmTraj(1:startIdx-1,1)));
            yShift = mean(diff(cmTraj(1:startIdx-1,2)));
            cmTraj(startIdx:endIdx,1) = cmTraj(startIdx-1,1) + xShift * (1:endIdx-startIdx+1);
            cmTraj(startIdx:endIdx,2) = cmTraj(startIdx-1,2) + yShift * (1:endIdx-startIdx+1);
            badIdxs(startIdx:endIdx) = false;
            startIdx = find(badIdxs,1,'first');
        end
        cmMovements = sqrt( sum( diff(cmTraj,1,1).^2, 2) );

        if any(cmMovements > pxCutOff)
            c.isBadCell = 3;
        end

    end
end

for i = 1:length(mCells)
    c = mCells(i);

    if ~isempty(c.descendants)
        c1 = c.descendants(1);
        c2 = c.descendants(2);

        parentCm = c.centroids(find(~c.badSegmentations,1,'last'),:);
        childCm1 = c1.centroids(find(~c1.badSegmentations,1),:);
        childCm2 = c2.centroids(find(~c2.badSegmentations,1),:);

        if ~isempty(parentCm) && ~isempty(childCm1) && ~isempty(childCm2)
            meanChildCm = 0.5*(childCm1 + childCm2);
            if sqrt(sum((parentCm - meanChildCm).^2)) > pxCutOff
                c.removeChildren();
            end
        else
            c.removeChildren();
        end
    end
end
    
end
