function [cellPairs] = findDisappearAppearCells(mCells, nFrames, jacCutOff, pxCutOff)
% [cellPairs] = findDisappearAppearCells(mCells, nFrames, jacCutOff, pxCutOff)
%
% Search for pairs of cells, such that one cell disappears and the other
% appears after one frame in the same place.
%
% The cell tracks must be "good", i.e. isBadCell=0.
%
% The last blob in the disappearing cell and the first blob in the
% appearing cell must satisfy the following conditions:
%   - Euclidean distance between centers of mass is below pxCutOff
%   - Jaccard similarity index is above jacCutOff
%
% Input:
%   mCells - array of MCell objects representing tracked cells.
%   nFrames - number of frames
%   jacCutOff - threshold for Jaccard index, between 0 and 1
%   pxCutOff - threshold for Euclidean distance between blob centroids
%
% Output:
%   cellPairs - 2xn array where the first row is the indices of
%               disappearing cells, and the second row is the indices of
%               appearing cells, correspondingly.

% Last modified: SZ, 2018-06-13. Handle the case of empty mCells.

if isempty(mCells)
    cellPairs = [];
    return;
end

import Cell.Track.Curate.computeJaccardIndex

% Parse info from mCells
nCells = numel(mCells);
firstFrames = [mCells.birthFrame];
lastFrames = [mCells.lastFrame];
isBadCell = [mCells.isBadCell];
hasDescendants  = false(1,nCells);
hasParent  = false(1,nCells);
for cIdx = 1:nCells
    if ~isempty(mCells(cIdx).descendants)
        hasDescendants(cIdx) = true;
    end
    if ~isempty(mCells(cIdx).parent)
        hasParent(cIdx) = true;
    end
end

disappFrames = lastFrames;
disappFrames(hasDescendants | isBadCell>0) = 0;
appFrames = firstFrames;
appFrames(hasParent | isBadCell>0) = 0;

cellPairs = zeros(1,nCells);
for frIdx = 1:nFrames-2
    disappCells = find(disappFrames == frIdx);
    if isempty(disappCells)
        continue;
    end
    appCells = find(appFrames == (frIdx+2));
    if isempty(appCells)
        continue;
    end
    
    % Compute pairwise distance between cell centroids
    cmDisapp = zeros(numel(disappCells),2);
    for i = 1:numel(disappCells)
        cmDisapp(i,:) = mCells(disappCells(i)).centroids(end,:);
    end
    cmApp = zeros(numel(appCells),2);
    for j = 1:numel(appCells)
        cmApp(j,:) = mCells(appCells(j)).centroids(1,:);
    end
    D = pdist2cust(cmDisapp,cmApp);    
    % Sort pairs by distance and cutoff below threshold
    [sortD, sortIdx] = sort(D(:));
    sortD = sortD(sortD<pxCutOff);
    for k=1:numel(sortD)        
        % Restore cell indices
        [i, j] = ind2sub(size(D),sortIdx(k));
        disappIdx = disappCells(i);
        appIdx = appCells(j);
        if ~disappIdx || ~appIdx
            continue
        end        
        % Compute jaccard similarity index
        cellDisapp = mCells(disappIdx);
        cellApp = mCells(appIdx);
        blobDisapp = cellDisapp.getBlob(cellDisapp.lifeTime);
        blobApp = cellApp.getBlob(1);
        jac = computeJaccardIndex(blobApp, blobDisapp);
        if jac > jacCutOff
            % mark that these cells are paired
            cellPairs(disappIdx) = appIdx;
            disappCells(i) = 0;
            appCells(i) = 0;
        end
    end
end

cellPairs = [find(cellPairs); cellPairs(cellPairs>0)];