function mCells = convertViterbiOutput(cellMat, childMat, blobs, nCores)
% mCells = convertViterbiOutput(cellMat, childMat, blobs, nCores)
%
% This function converts the raw output of Baxter algorithm into an array
% of MCell objects representing cell tracks.
%
% Input:
%   cellMat - Matrix which specifies in which blob each cell is in each
%             frame. The matrix has one row for each frame and one column
%             for each cell. Element (i,j) of the matrix has the index of
%             the blob that cell j is in frame i.
%   childMat - Matrix which defines mother-daughter relationships between
%              cells. The matrix has one row for each cell and two
%              columns with indices of daughter cells. The rows for cells
%              without daughter cells have the value NaN.
%   blobs - cell array, element i contains a structure array with
%           properties of blobs in frame i returned by Labels2Blobs.
%   nCores - number of workers to use in the parallel pool.
%            Optional, the default value is 0.
%
% Output:
%   mCells - array of MCell objects representing cell tracks.
%
% See also: Labels2Blobs, convertViterbiOutput_parfor
%
% Last modified: 2020-05-20, Spartak Zikrin. Use blob structure arrays
% instead of Blob objects.
%% Initialization
if ~exist('nCores','var')
    nCores = 0;
end

if nCores>1
    mCells = Cell.Track.convertViterbiOutput_parfor(cellMat, childMat, blobs, nCores);
    return
end

if isempty(cellMat)
    mCells = [];
    return
end
%% Create objects mCells and mFrames
[nFrames, nCells] = size(cellMat);
mFrames = Cell.MCell.mFrames;
mFrames.initialize(nFrames);
mCells(1:nCells,1) = Cell.MCell;

for cIdx = 1:nCells
    firstFrame = find(cellMat(:, cIdx), 1, 'first');
    lastFrame = find(cellMat(:, cIdx), 1, 'last');
    lifeTime = lastFrame-firstFrame+1;
    mCells(cIdx) = Cell.MCell(cIdx, firstFrame, lifeTime);
end
%% Get cell properties using stored Blob objects
for fIdx = 1:nFrames
    curBlobs = blobs{fIdx};
    [~, cIdxs, bIdxs] = find(cellMat(fIdx,:));
    [ubIdxs, ~, iu] = unique(bIdxs);
    nBlobs = numel(ubIdxs);
    if nBlobs>0
        keySet = [curBlobs(ubIdxs).Label];
        valueSet = cell(1,nBlobs);
        for ii = 1:length(cIdxs)
            cIdx = cIdxs(ii);
            bIdx = bIdxs(ii);
            ubIdx = iu(ii);
            b = curBlobs(bIdx);
            c = mCells(cIdx);
            t = c.getIndex(fIdx);
            c.boundingBoxes(:,t) = b.BoundingBox';
            c.boundaries{t} = b.Boundary;
            c.centroids(t,:) = b.Centroid;
            c.blobLabels(t) = b.Label;
            c.areas(t) = b.Area;
            valueSet{ubIdx} = [valueSet{ubIdx}, c];
        end
        mFrames.setPointersInFrame(fIdx,keySet,valueSet);
    end
end
%% Link mother and daughter cells.
cIdxs = find(childMat(:,1))';
for cIdx = cIdxs
    cParent = mCells(cIdx);
    cChildren = mCells(childMat(cIdx,:));
    cParent.addChild(cChildren(1));
    cParent.addChild(cChildren(2));
end