function mCells = convertViterbiOutput_parfor(cellMat, childMat, blobs, nCores)
% mCells = convertViterbiOutput_parfor(cellMat, childMat, blobs, nCores)
%
% This function converts the raw output of Baxter algorithm into an array
% of MCell objects representing cell tracks.
% The code run parallel loop between frames.
%
% Input:
%   cellMat - Matrix which specifies in which blob each cell is in each
%             frame. The matrix has one row for each frame and one column
%             for each cell. Element (i,j) of the matrix has the index of
%             the blob that cell j is in frame i.
%   childMat - Matrix which defines mother-daughter relationships between
%              cells. The matrix has one row for each cell and two
%              columns with indices of daughter cells. The rows for cells
%              without daughter cells have the value NaN.
%   blobs - cell array, element i contains a structure array with
%           properties of blobs in frame i returned by Labels2Blobs.
%   nCores - number of workers to use in the parallel pool.
%            Optional, the default value is 0.
%
% Output:
%   mCells - array of MCell objects representing cell tracks.
%
% See also: Labels2Blobs, convertViterbiOutput
%
% Last modified: 2020-05-20, Spartak Zikrin. Use blob structure arrays
% instead of Blob objects.
%% Initialization

if isempty(cellMat)
    mCells = [];
    return
end

%% Create objects mCells and mFrames
[nFrames, nCells] = size(cellMat);
mFrames = Cell.MCell.mFrames;
mFrames.initialize(nFrames);
mCells(1:nCells,1) = Cell.MCell;
parfor(cIdx = 1:nCells, nCores)
    firstFrame = find(cellMat(:, cIdx), 1, 'first');
    lastFrame = find(cellMat(:, cIdx), 1, 'last');
    lifeTime = lastFrame-firstFrame+1;
    mCells(cIdx) = Cell.MCell(cIdx, firstFrame, lifeTime);
end
%% Get cell properties using stored Blob objects
outData = cell(nFrames,1);
parfor(fIdx = 1:nFrames, nCores)
    curBlobs = blobs{fIdx};
    [~, cIdxs, bIdxs] = find(cellMat(fIdx,:));
    [ubIdxs, ~, iu] = unique(bIdxs);
    nBlobs = numel(ubIdxs);
    frameData = struct;
    frameData.cellIndices = cIdxs;
    frameData.blobIndices = iu;
    frameData.boundaries = cell(nBlobs,1);
    frameData.boundingBoxes = cell(nBlobs,1);
    frameData.centroids = zeros(nBlobs,2);
    frameData.blobLabels = zeros(nBlobs,1);
    frameData.areas  = zeros(nBlobs,1);
    for ii = 1:nBlobs
        b = curBlobs(ubIdxs(ii));
        frameData.boundaries{ii} = b.Boundary;
        frameData.boundingBoxes{ii} = b.BoundingBox';
        frameData.centroids(ii,:) = b.Centroid;
        frameData.blobLabels(ii) = b.Label;
        frameData.areas(ii) = b.Area;
    end
    outData{fIdx} = frameData;
end

%% Parse data to mCells
for fIdx = 1:nFrames
    frameData = outData{fIdx};
    nCells = numel(frameData.cellIndices);
    nBlobs = numel(frameData.blobLabels);
    keySet = frameData.blobLabels;
    valueSet = cell(1,nBlobs);
    for ii=1:nCells
        cIdx = frameData.cellIndices(ii);
        bIdx = frameData.blobIndices(ii);
        c = mCells(cIdx);
        t = c.getIndex(fIdx);
        c.boundaries{t} = frameData.boundaries{bIdx};
        c.boundingBoxes(:,t) = frameData.boundingBoxes{bIdx};
        c.centroids(t,:) = frameData.centroids(bIdx,:);
        c.blobLabels(t) = frameData.blobLabels(bIdx);
        c.areas(t) = frameData.areas(bIdx);
        valueSet{bIdx} = [valueSet{bIdx}, c];
    end
    mFrames.setPointersInFrame(fIdx,keySet,valueSet);
end
%% Link mother and daughter cells.
cIdxs = find(childMat(:,1))';
for cIdx = cIdxs
    cParent = mCells(cIdx);
    cChildren = mCells(childMat(cIdx,:));
    cParent.addChild(cChildren(1));
    cParent.addChild(cChildren(2));
end