function trackCells(expInfoObj, imAdapterObj, posName)
% trackCells(expInfoObj, imAdapterObj, posName)
%
% Wrapper to export blob features from segmented images, compute various 
% scores and link cell tracks using Viterbi algorithm (Baxter Algorithm).
% The obtained cell tracks are converted to mCell objects and then
% additionally curated.
% Requires segmented phase images stored in subfolder SegmentedPhase.
% Cell arrays blobs, jaccardList and matrices cellMat, childMat
% from Baxter algorithm are stored in file outputBaxter.mat.
%
% Input:
%   expInfoObj - expInfo object.
%   imAdapterObj - generidReadAsFrames object containing frame timestamps
%   posName - string with the name of the position to be analysed.
%
% Last modified:
% SZ, 2020-05-20. Compute Jaccard index using regionprops and avoid loading
% blobs.
%% Import libraries
import Cell.Track.*
import Cell.Scores.*
%% Parse parameters and create output folders
if expInfoObj.getParameters('isMothermachine')
    trackCellsMM(expInfoObj, imAdapterObj, posName)
    return
end

outputDir = expInfoObj.getPathName('output');
segDir = fullfile(outputDir, posName, 'SegmentedPhase');
if ~exist(segDir, 'dir')
    error('Provided directory with segmented images does not exist: %s', segDir);
end
segList = dir(fullfile(segDir,'*.tif*'));

param = expInfoObj.getParameters();
if ~param.parforPositions
    nCores = param.parNumWorkers;
else
    nCores = 0;
    param.parNumWorkers = 0;
end

phaseChanName = expInfoObj.getChannelNames('phase');
%% Compute detection features from segmented images
trackOutputFile = fullfile(outputDir, posName, 'outputBaxter.mat');
if exist(trackOutputFile,'file')
    load(trackOutputFile,'blobs','jaccardScores');
else
    [blobs, jaccardScores] = computeBlobs(segList,[],nCores);
    save(trackOutputFile,'blobs','jaccardScores');
end
nBlobsPerFrame = cellfun(@length,blobs);
if ~any(nBlobsPerFrame) % if no detections found
    return
end
%% Compute scores for different linking options.
migrationScores = MigrationScores(jaccardScores,param);
splitScores = SplitScores(blobs, migrationScores, param);
mergeScores = MergeScores(blobs, migrationScores, param);
countScores = CountScores(nBlobsPerFrame, param);
deathScores = DeathScores(nBlobsPerFrame, param);
appearanceScores = AppearanceScores(nBlobsPerFrame, param);
disappearanceScores = DisappearanceScores(nBlobsPerFrame, param);
% Migration scores with low jaccard values were just kept to
% calculate the splitting and merging scores. After this is done
% the low scoring migration should be taken out.
migrationScores = removeLowMigrationScores(migrationScores, splitScores, mergeScores, param);
%% Track linking
[cellMat, childMat, ~] = ViterbiTrackLinking(nBlobsPerFrame,...
    countScores, migrationScores, splitScores, deathScores,...
    appearanceScores, disappearanceScores,0,0,'','');
save(trackOutputFile,'cellMat','childMat','-append');
%% Conversion to the pipeline format and further track curation
rawCells = convertViterbiOutput(cellMat, childMat, blobs, nCores);
mCells = curateTrackingOutput(rawCells, param);
Cell.addAcquireTimes(mCells, imAdapterObj, posName, phaseChanName);
if ~param.saveCellBoundaries
   for cIdx = 1:length(rawCells)
       mCells(cIdx).boundaries = {};
   end
end
%% Save results and delete intermediate files
mCellsMatFile = expInfoObj.getMCellMatPath(posName);
Cell.MCell.saveMCells(mCellsMatFile, mCells);