function trackCellsMM(expInfoObj, imAdapterObj, posName)
% trackCellsMM(expInfoObj, imAdapterObj, posName)
%
% Wrapper to export blob features from segmented images, compute various
% scores and link cell tracks using Viterbi algorithm (Baxter Algorithm).
% Cell tracking is done for each trap in the mother-machine separately, and
% the resulting cell tracks are concatenated into a single array of mCell
% objects and correspong mFrames object.
% The obtained cell tracks are converted to mCell objects and then 
% additionally curated. 
% Requires segmented phase images stored in subfolder SegmentedPhase.
% Cell arrays blobs, jaccardList and matrices cellMat, childMat
% from Baxter algorithm are stored in file outputBaxter.mat.
%
% Input:
%   expInfoObj - expInfo object.
%   imAdapterObj - generidReadAsFrames object containing frame timestamps
%   posName - string with the name of the position to be analysed.
%
% Last modified:
% SZ, 2019-03-14. Parfor loop between the growth channels instead of
% frames.
% SZ, 2020-11-25. Added multiple empty growth channels.
% SZ, 2020-05-20. Compute Jaccard index using regionprops and avoid loading
% blobs.
%% Import libraries
import Cell.Track.*
import Cell.Scores.*
%% Parse parameters and create output folders
outputDir = expInfoObj.getPathName('output');
segDir = fullfile(outputDir, posName, 'SegmentedPhase');
if ~exist(segDir, 'dir')
    error('Provided directory with segmented images does not exist: %s', segDir);
end
segList = dir(fullfile(segDir,'*.tif*'));

param = expInfoObj.getParameters();
if ~param.parforPositions
    nCores = param.parNumWorkers;
else
    nCores = 0;
    param.parNumWorkers = 0;
end
phaseChanName = expInfoObj.getChannelNames('phase');
nFrames = numel(expInfoObj.getIndices(posName,phaseChanName));

channelIndices = 1:param.nGrowthChannels;
channelIndices(param.emptyChannel(2:end)) = [];
% emptyChIdx = find(channelIndices == param.emptyChannel(1));
channels = expInfoObj.getChannelLocations(posName);
%channels = channels([1:emptyChIdx-1 emptyChIdx+1:end],:,:);
nChannels = size(channels,1);
%% Compute detection features from segmented images
trackOutputFile = fullfile(outputDir, posName, 'outputBaxter.mat');
if exist(trackOutputFile,'file')
    load(trackOutputFile,'blobs','jaccardScores');
else
    [blobs, jaccardScores] = computeBlobs(segList,channels,nCores);
    save(trackOutputFile,'blobs','jaccardScores');
end
nBlobs = cellfun(@length,blobs);
%% Cell tracking
outMCells = cell(nChannels,1);
param.parNumWorkers = 0;
cellMat = cell(1,nChannels);
childMat = cell(1,nChannels);
parfor (ch = 1:nChannels, nCores)
% for ch = 1:nChannels
    nBlobsPerFrame = nBlobs(:,ch);
    if any(nBlobsPerFrame) % Make sure there are blobs
        %% Compute scores for different linking options.
        migrationScores = MigrationScores(jaccardScores{ch}, param);
        splitScores = SplitScores(blobs(:,ch), migrationScores, param);
        mergeScores = MergeScores(blobs(:,ch), migrationScores, param);
        countScores = CountScores(nBlobsPerFrame, param);
        deathScores = DeathScores(nBlobsPerFrame, param);
        appearanceScores = AppearanceScores(nBlobsPerFrame, param);
        disappearanceScores = DisappearanceScores(nBlobsPerFrame, param);
        % Migration scores with low jaccard values were just kept to
        % calculate the splitting and merging scores. After this is done
        % the low scoring migration should be taken out.
        migrationScores = removeLowMigrationScores(migrationScores, splitScores, mergeScores, param);
        %% Track linking
        [cellMat{ch}, childMat{ch}, ~] = ViterbiTrackLinking(nBlobsPerFrame,...
            countScores, migrationScores, splitScores, deathScores,...
            appearanceScores, disappearanceScores,0,0,'',''); %#ok<PFOUS>
        rawCells = convertViterbiOutput(cellMat{ch}, childMat{ch}, blobs(:,ch), 0);
        outMCells{ch} = curateTrackingOutput(rawCells, param);
    end
end
save(trackOutputFile,'cellMat','childMat','-append');
mCells = mergeCellTracksInTraps(outMCells,nFrames);
Cell.addAcquireTimes(mCells, imAdapterObj, posName, phaseChanName);
if ~param.saveCellBoundaries
   for cIdx = 1:length(mCells)
       mCells(cIdx).boundaries = {};
   end
end
%% Save results and delete intermediate files
mCellsMatFile = expInfoObj.getMCellMatPath(posName);
Cell.MCell.saveMCells(mCellsMatFile, mCells);