classdef MCell < handle
    % A cell which contains tracking info, cell info, particles info and
    % lineage
    properties
        id                  = [];
        birthFrame          = [];
        lifeTime            = [];
        detectionTimes      = [];
        parent              = [];
        descendants         = [];
        blobLabels          = [];        
        boundaries          = {};
        boundingBoxes       = [];
        backbones           = [];
        centroids           = [];
        poles               = {};
        isReoriented        = [];
        lengths             = [];
        widths              = [];
        areas               = [];
        fluoIntensities     = []; % intensity/cell area at each fluo frame
        isBadCell           = 0;
        % 1 - two cell tracks converge in a blob and at least one of the
        %     tracks ends or starts in that frame
        % 2 - low Jaccard index between blobs in consecutive frames
        % 3 - fast movement of cell centroids in consecutive frames
        % 4 - track with a gap between consecutive frames
        badSegmentations    = [];
        % 1 - two cell tracks converge in this blob
        % 2 - low Jaccard index between blobs in this and next frames
        % 3 - not used
        % 4 - gap, cell is not detected in this frame
        majorAxisOrientationDegree = [];
        growthChannel       = -1;
        particles           = struct('channelName',         [], ...
                                     'trajImageCoord',      {}, ...
                                     'trajPhaseImageCoord', {}, ...
                                     'trajInternalCoord',   {}, ...
                                     'trajImageCov',        {}, ...
                                     'cellDetectionFrames', [], ...
                                     'firstFrame',          [], ...
                                     'numberOfFrame',       [], ...
                                     'fluoTimes',           [], ...
                                     'blobLabels',          []);
    end
    
    properties (Constant)
        mFrames = Cell.MFrames;
    end
    
    properties (Dependent = true)
        lastFrame
        boundaryImageCoord
    end
    
    %% Save/Load methods
    methods (Static)        
        
        function saveMCells(saveFile, mCells)            
            if isa(mCells,'Cell.MCell')
                mFrames = Cell.MCell.mFrames;
                save(saveFile,'mCells','mFrames','-v7');
            else
                disp('Failed to save mCells:');
                disp('Provided variable is not an array of MCell objects');
            end
        end
        
        function [mCells, mFrames] = loadMCells(loadFile)
            if ~exist(loadFile,'file')                
                disp(['Could not find load file ' loadFile]);
                mCells = [];
                mFrames = {};
                return
            end
            load(loadFile,'mCells','mFrames');
            if ~exist('mCells','var') || ~exist('mFrames','var')
                mCells = [];
                mFrames = {};
                return
            end
            % restore links in MFrames to mCells
            h = Cell.MCell.mFrames;
            h.maps = mFrames.maps;
        end
        
        function mFrames = clearMFrames()
            mFrames = Cell.MCell.mFrames;
            mFrames = {};
        end
    end
    
    %%
    methods
        
        function this = MCell(id, birthFrame, lifeTime, varargin)
            if ~nargin
                return
            end
            this.id = id;
            this.birthFrame = birthFrame;
            this.lifeTime = lifeTime;            
            for i = 1 : 2 : length(varargin)
                this.(varargin{i}) = varargin{i+1};
            end            
            if isempty(this.detectionTimes)
                this.detectionTimes = zeros(1,lifeTime);
            end            
            if isempty(this.blobLabels)
                this.blobLabels = zeros(1,lifeTime,'uint16');
            end            
            if isempty(this.poles)
                this.poles = cell(1,lifeTime);
            end            
            if isempty(this.boundaries)
                this.boundaries = cell(1,lifeTime);
            end            
            if isempty(this.boundingBoxes)
                this.boundingBoxes = zeros(4,lifeTime,'uint16');
            end            
            if isempty(this.backbones)
                this.backbones = zeros(3,lifeTime);
            end        
            if isempty(this.centroids)
                this.centroids = zeros(lifeTime,2);
            end            
            if isempty(this.badSegmentations)
                this.badSegmentations = zeros(1,lifeTime,'uint8');
            end            
            if isempty(this.isReoriented)
                this.isReoriented = false(1,lifeTime);
            end            
            if isempty(this.lengths)
                this.lengths = zeros(1,lifeTime);
            end            
            if isempty(this.widths)
                this.widths = zeros(1,lifeTime);
            end            
            if isempty(this.areas)
                this.areas = zeros(1,lifeTime);
            end
            if isempty(this.majorAxisOrientationDegree)
                this.majorAxisOrientationDegree = zeros(1,lifeTime);
            end
        end
        
        function mCell = clone(this)
        % Returns a deep copy of a cell object.
            mCell = Cell.MCell(this.id, ...
                          this.birthFrame, ...
                          this.lifeTime, ...
                          'parent',              this.parent, ...
                          'descendants',         this.descendants, ...
                          'blobLabels',          this.blobLabels, ...
                          'boundaries',          this.boundaries, ...
                          'boundingBoxes',       this.boundingBoxes, ...
                          'backbones',           this.backbones, ...
                          'centroids',           this.centroids, ...
                          'poles',               this.poles, ...
                          'isReoriented',        this.isReoriented, ...
                          'areas',               this.areas, ...
                          'lengths',             this.lengths, ...
                          'widths',              this.widths, ...
                          'fluoIntensities',     this.fluoIntensities, ...
                          'isBadCell',           this.isBadCell, ...
                          'badSegmentations',    this.badSegmentations,...
                          'majorAxisOrientationDegree', this.majorAxisOrientationDegree, ...
                          'particles',           this.particles );
        end
        
        %% Accessors
        
        function blob = getBlob(this,idx)
            blob.boundingBox = this.boundingBoxes(:,idx);
            blob.boundary = this.boundaries{idx};
            blob.area = this.areas(idx);
        end
        
        function lastFrame = get.lastFrame(this)
            lastFrame = this.birthFrame + this.lifeTime - 1;
        end
        
        function t = getIndex(this, frame)
            t = frame - this.birthFrame + 1;
        end
        
        function flag = isRotated(this, t)
            % Determined by dimensions of cell image along x and y
            if this.boundingBoxes(3,t) >= this.boundingBoxes(4,t)
                flag = 0;
            else
                flag = 1;
            end
        end
        
        function backboneCoord = getBackboneCoord(this,t)
            cellPoles = this.poles{t};
            backbone = this.backbones(:,t);
            if isempty(cellPoles) || ~any(backbone)
                backboneCoord = [];
                return;
            end
            
            xCoord = cellPoles(1,1):0.1:cellPoles(2,1);
            yCoord = backbone(1)*xCoord.^2 + backbone(2)*xCoord + backbone(3);
            backboneCoord = [xCoord, yCoord];
        end
        
        %% Modifiers
        
        function mCell = split(this,splitPt)
            % Cuts a cell into two new cells.
            %
            % The time points after splitPoint are removed from the current
            % cell and are then used to create a new cell, which is
            % returned.
            mCell = Cell.MCell([],this.birthFrame + splitPt,...
                this.lifeTime - splitPt,...
                'detectionTimes' ,this.detectionTimes(splitPt+1:end),...
                'descendants', this.descendants,...
                'blobLabels', this.blobLabels(splitPt+1:end),...
                'boundaries', this.boundaries(splitPt+1:end),...
                'boundingBoxes', this.boundingBoxes(:,splitPt+1:end),...
                'backbones', this.backbones(:,splitPt+1:end),...
                'centroids', this.centroids(splitPt+1:end,:),...
                'poles', this.poles(splitPt+1:end),...
                'isReoriented', this.isReoriented(splitPt+1:end),...
                'lengths', this.lengths(splitPt+1:end),...
                'widths', this.widths(splitPt+1:end), ...
                'areas', this.areas(splitPt+1:end),...
                'badSegmentations', this.badSegmentations(splitPt+1:end),...
                'majorAxisOrientationDegree', this.majorAxisOrientationDegree(splitPt+1:end));
            this.lifeTime = splitPt;
            this.detectionTimes = this.detectionTimes(1:splitPt);
            this.blobLabels = this.blobLabels(1:splitPt);
            this.boundaries = this.boundaries(1:splitPt);
            this.boundingBoxes = this.boundingBoxes(:,1:splitPt);
            this.backbones = this.backbones(:,1:splitPt);
            this.centroids = this.centroids(1:splitPt,:);
            this.poles = this.poles(1:splitPt);
            this.isReoriented = this.isReoriented(1:splitPt);
            this.lengths = this.lengths(1:splitPt);
            this.widths = this.widths(1:splitPt);
            this.areas = this.areas(1:splitPt);
            this.badSegmentations = this.badSegmentations(1:splitPt);
            this.majorAxisOrientationDegree = ...
                this.majorAxisOrientationDegree(1:splitPt);
            this.removeChildren();
            
            frame = this.birthFrame + splitPt;
            for blobLabel = mCell.blobLabels
                this.mFrames.removeCellPointer(frame, blobLabel, this.id);
                frame = frame + 1;
            end
        end            
        
        function addCell(this, c)
            % Append cell c after the last frame of the current cell.
            if ~isempty(c.parent)
                fprintf('Trying to append to cell %d a cell %d with parent, aborting', this.id, c.id);
                return
            end
            
            t0 = this.lifeTime;
            t1 = t0 + c.lifeTime;
            this.lifeTime = t1;
            this.detectionTimes(t0+1:t1) = c.detectionTimes;
            this.blobLabels(t0+1:t1) = c.blobLabels;
            this.boundaries(t0+1:t1) = c.boundaries;
            this.boundingBoxes(:,t0+1:t1) = c.boundingBoxes;
            this.backbones(:,t0+1:t1) = c.backbones;
            this.centroids(t0+1:t1,:) = c.centroids;
            this.poles(t0+1:t1) = c.poles;
            this.isReoriented(t0+1:t1) = c.isReoriented;
            this.lengths(t0+1:t1) = c.lengths;
            this.widths(t0+1:t1) = c.widths;
            this.areas(t0+1:t1) = c.areas;
            this.badSegmentations(t0+1:t1) = c.badSegmentations;
            this.majorAxisOrientationDegree(t0+1:t1) = c.majorAxisOrientationDegree;
            
            if ~isempty(c.descendants)
                for i=1:length(c.descendants)
                    this.addChild(c.descendants(i));
                end
                c.descendants = [];
            end
            
            idx = find(c.blobLabels>0);
            if isempty(idx)
                return
            end
            ts = t0+idx;
            frames = this.birthFrame+ts-1;
            for i = 1:numel(idx)
                this.mFrames.addCellPointer(frames(i),this,ts(i));
            end
        end
        
        function addCellWithGap(this, mCell)
            % Append a cell after two frames after the last frame of this
            % cell
            this.lifeTime = this.lifeTime + 1;
            this.isBadCell = 4;
            this.badSegmentations(end+1) = 4;
            addCell(this, mCell);
        end

        function addChild(this, mCell)
            if length(this.descendants) == 2
                disp('This cell already has 2 children. Not adding child')
            elseif this.lastFrame ~= mCell.birthFrame-1                
                disp(['The first frame of the suggested child does '...
                    'not match the last frame of the parent. Not '...
                    'adding child.'])
            else
                this.descendants = [this.descendants mCell];
                mCell.parent = this;
            end
        end
        
        function removeChildren(this)
            for i=1:numel(this.descendants)
                this.descendants(i).parent = [];
            end
            this.descendants = [];
        end
        
        function deleteCell(this)
            if ~isempty(this.parent)
                fprintf('Cell %d has a parent cell. Abort deleting', this.id);
                return
            end
            this.removeChildren();
            for frameNr = this.birthFrame:this.lastFrame
                currBlobLabel = this.blobLabels(frameNr-this.birthFrame+1);
                this.mFrames.removeCellPointer(frameNr, ...
                    currBlobLabel, this.id);
            end
            delete(this);
            
        end
        
        function alignCellPoles(this)
        % Checks pole orientation of cell
            if this.lengths(1) > 0
                prevPoleVec = this.poles{1}(1,:)-this.poles{1}(2,:);
            else
                return;
            end
            for ii=2:length(this.poles)
                % Measure angle between vectors pointed from old poles to
                % new poles in the current and previous cells
                if this.lengths(ii) > 0
                    currPoleVec = this.poles{ii}(1,:) - this.poles{ii}(2,:);
                    dotProd = prevPoleVec * currPoleVec';
                    if dotProd < 0
                        this.swapCellPoles(ii);
                        prevPoleVec = -currPoleVec;
                    else
                        prevPoleVec = currPoleVec;
                    end
                end
            end
        end
        
        function swapCellPoles(this, t)
        % Swaps cell poles at a given cell stage. The old pole should be in 
        % the first row and the new pole in the second row
            this.poles{t} = this.poles{t}([2 1],:);
            this.isReoriented(t) = ~this.isReoriented(t);
        end
        
        %% Converters
        function cellCoord = convertToCellCoord(this, t, imageCoord, rotationFlag)
        % imageCoord is nx2 matrix containing n pairs of (x,y) coordinates
        % in the phase frame's coordinate system
        % 2018-01-19: SZ, fixed bug bb(4)->bb(3)
            bb = this.boundingBoxes(:,t);
            if isa(imageCoord,'float')
                bb = double(bb);
            end
            cellCoord = [imageCoord(:,1) - bb(1) + 1,...
                         imageCoord(:,2) - bb(2) + 1];
            if nargin < 4
                rotationFlag = 0;
            end
            if rotationFlag && this.isRotated(t)
                cellCoord = [cellCoord(:,2), bb(3)-cellCoord(:,1)+1];
            end
        end
        
        function imageCoord = convertFromCellCoord(this, t, cellCoord, rotationFlag)
        % cellCoord is a nx2 matrix containing n pairs of (x,y) coordinates
        % in the cropped cell image's coordinate system
            bb = this.boundingBoxes(:,t);
            if isa(cellCoord,'float')
                bb = double(bb);
            end  
            if nargin < 4
                rotationFlag = 0;
            end
            if rotationFlag && this.isRotated(t)
                cellCoord = [bb(3)-cellCoord(:,2)+1, cellCoord(:,1)];
            end
            imageCoord = [cellCoord(:,1) + bb(1) - 1,...
                          cellCoord(:,2) + bb(2) - 1];
        end
        
    end
end
