classdef MFrames < handle
    % Contains a cell array of map objects, where each map corresponds to a
    % segmented frame and points to CellTrack objects that correspond to
    % frame blobs
    
    % Last modified: SZ, 2018-04-20. Fixed cases of empty or singular blob
    % in a frame
    properties
        maps = {}
    end
    
    %% Private methods
    methods (Access = private)
        function setMap(this,frameIdx,keySet,valueSet)
                if numel(keySet)>1 && numel(keySet) ~= numel(valueSet)
                    error('Number of keys and values must be the same');
                end
                keySet = uint32(keySet);
                if ~isempty(keySet)
                    this.maps{frameIdx} = containers.Map(keySet,valueSet);
                else
                    this.maps{frameIdx} = [];
                end
        end
    end
    
    methods
        % Constuctor, Creates a cell array of empty maps
        function initialize(this, nFrames)
            this.maps = cell(nFrames,1);
            for f=1:nFrames
                this.maps{f} = containers.Map('KeyType','uint32','ValueType','any');
            end
        end
        
        %% Accessors
        function mCells = getCellsInBlob(this, frameIdx, blobLabel)
            if ~isempty(this.maps{frameIdx}) && isKey(this.maps{frameIdx}, blobLabel)
                mCells = this.maps{frameIdx}(blobLabel);
            else
                mCells = [];
            end
        end
        
        function blobLabels = getBlobsInFrame(this, frameIdx)
            if ~isempty(this.maps{frameIdx})
                blobLabels = cell2mat(keys(this.maps{frameIdx}));
            else
                blobLabels = [];
            end
        end
        
        function sharedBlobs = getSharedBlobs(this, frameIdx)
            cellsInFrameMap = this.maps{frameIdx};
            if ~isempty(cellsInFrameMap)
                valueSet = values(cellsInFrameMap);
                blobLabels = keys(cellsInFrameMap);
                idx = cellfun('prodofsize',valueSet) == 2;
                sharedBlobs = cell2mat(blobLabels(idx));
            else
                sharedBlobs = [];
            end
        end
        
        function nFrames = getNumOfFrames(this)
            nFrames = numel(this.maps);
        end
        
        %% Modifiers
        function addCellPointer(this,frameIdx,mCell,trackPt)
            blobLabel = mCell.blobLabels(trackPt);
            key = blobLabel;
            if isKey(this.maps{frameIdx}, key)
                oldValue = this.maps{frameIdx}(key);
                value = [oldValue, mCell];
            else
                value = mCell;
            end
            this.maps{frameIdx}(key) = value;
        end
        
        function setPointersInFrame(this,frameIdx,keySet,valueSet)
            if numel(keySet) ~= numel(valueSet)
                return
            end
            if numel(keySet) == 1
                valueSet = valueSet{1};
            end
            this.setMap(frameIdx,keySet,valueSet);
        end
        
        function removeCellPointer(this, frameIdx, blobLabel, cellId)
            if blobLabel <= 0
                return
            end
            mCells = this.maps{frameIdx}(blobLabel);
            if ~any([mCells.id] == cellId)
                fprintf('Failed to find cell %d in frame %d and blob %d\n',...
                    cellId,frameIdx,blobLabel);
                return
            end
            mCells = mCells([mCells.id] ~= cellId);
            if isempty(mCells)
                remove(this.maps{frameIdx},blobLabel);
            else
                this.maps{frameIdx}(blobLabel) = mCells;
            end
        end
        
    end
end