function mCells = initializeOutputStructure(expInfoObj, imAdapterObj, posName)
% initializeOutputStructure(expInfoObj, imAdapterObj, posName)
% 
% This function is used to initialize MCell objects for each segmented blob
% in phase images in case if no cell tracking is required. The resulting 
% array of MCell objects is saved in "trackedCells.mat" file
%
% Input:
%   expInfoObj - expInfo object.
%   imAdapterObj - generidReadAsFrames object containing frame timestamps
%   posName - string with the name of the position to be analysed.
% 
% Output:
%   mCells - array of MCell objects representing cells.
%% Initializition
if ~expInfoObj.getParameters('parforPositions')
    mCells = Cell.initializeOutputStructure_parfor(expInfoObj, imAdapterObj, posName);
    return;
end

import Cell.MCell

outputDir = expInfoObj.getPathName('output');
phaseChanName = expInfoObj.getChannelNames('phase');
phaseIndices = expInfoObj.getIndices(posName,phaseChanName);
nFrames = length(phaseIndices);
allPhaseTimes = imAdapterObj.getAcquireTimes(posName, phaseChanName);

segDir = fullfile(outputDir, posName, 'SegmentedPhase');
segList = dir(fullfile(segDir,'*.tif*'));
%% Main code
mFrames = MCell.mFrames;
mFrames.initialize(nFrames);
cellArrayOfMCells = cell(nFrames,1);
cIdx = 0;
for fIdx = 1:nFrames
    segIm = imread(fullfile(segDir,segList(fIdx).name));
    blobLabels = unique(segIm);
    blobLabels = blobLabels(2:end);
    nBlobs = numel(blobLabels);
    if nBlobs>0
        mCells(1:nBlobs,1) = MCell;
        currPhaseTime = allPhaseTimes(fIdx);
        for blobIdx = 1:nBlobs
            cIdx = cIdx+1;
            blobLabel = blobLabels(blobIdx);
            stats = regionprops(segIm == blobLabel,'BoundingBox','Area','Centroid');
            bb = uint16(stats.BoundingBox');
            cropIm = segIm(bb(2):bb(2)+bb(4)-1,bb(1):bb(1)+bb(3)-1);
            blobBoundary = bwboundaries(cropIm == blobLabel, 'noholes');
            blobBoundary{1} = blobBoundary{1}(:,[2 1]);
            mCells(blobIdx,1) = Cell.MCell(...
                                    cIdx, fIdx, 1,...
                                    'blobLabels',blobLabel, ...
                                    'detectionTimes', currPhaseTime,...
                                    'boundingBoxes',bb,...
                                    'boundaries', blobBoundary(1),...
                                    'areas', stats.Area, ...
                                    'centroids', stats.Centroid);
            mFrames.addCellPointer(fIdx, mCells(blobIdx), 1);
        end
        cellArrayOfMCells{fIdx} = mCells;
        clear mCells
    else
        mFrames.setPointersInFrame(fIdx,[],{});
    end
end
%% Convert cell array of MCells to array of MCells
nCellsPerFrame = cellfun('length',cellArrayOfMCells);
nAllCells = sum(nCellsPerFrame);
mCells(1:nAllCells,1) = MCell;
if nAllCells>0
    start = 1;
    for fIdx = 1:nFrames
        if nCellsPerFrame(fIdx)>0
            finish = nCellsPerFrame(fIdx)+start-1;
            mCells(start:finish) = cellArrayOfMCells{fIdx};
            start = finish+1;
        end
    end
end
%% Save mCells
saveName = expInfoObj.getMCellMatPath(posName);
MCell.saveMCells(saveName, mCells);