function mCells = initializeOutputStructure_parfor(expInfoObj, imAdapterObj, posName)
% initializeOutputStructure_parfor(expInfoObj, imAdapterObj, posName)
% 
% This function is used to initialize MCell objects for each segmented blob
% in phase images in case if no cell tracking is required. The resulting 
% array of MCell objects is saved in "trackedCells.mat" file.
% The iterations between the segmented frames are run in parallel.
%
% Input:
%   expInfoObj - expInfo object.
%   imAdapterObj - generidReadAsFrames object containing frame timestamps.
%   posName - string with the name of the position to be analysed.
% 
% Output:
%   mCells - array of MCell objects representing cells.
%% Initializition
import Cell.MCell

outputDir = expInfoObj.getPathName('output');
phaseChanName = expInfoObj.getChannelNames('phase');
phaseIndices = expInfoObj.getIndices(posName,phaseChanName);
nFrames = length(phaseIndices);
allPhaseTimes = imAdapterObj.getAcquireTimes(posName, phaseChanName);
nCores = expInfoObj.getParameters('parNumWorkers');

segDir = fullfile(outputDir, posName, 'SegmentedPhase');
segList = dir(fullfile(segDir,'*.tif*'));
%% Get blob properties
outData = cell(nFrames,1);
nBlobs = zeros(nFrames,1);
parfor(fIdx = 1:nFrames, nCores)
    segIm = imread(fullfile(segDir,segList(fIdx).name));
    blobLabels = unique(segIm);
    blobLabels = blobLabels(2:end);
    nBlobs(fIdx) = numel(blobLabels);
    if ~nBlobs(fIdx)
        mCells = [];
        continue
    end
    frameData = struct;
    frameData.blobLabels = blobLabels;
    frameData.boundaries = cell(nBlobs(fIdx),1);
    frameData.boundingBoxes = cell(nBlobs(fIdx),1);
    frameData.centroids = zeros(nBlobs(fIdx),2);
    frameData.areas  = zeros(nBlobs(fIdx),1);
    for ii = 1:nBlobs(fIdx)
        blobLabel = blobLabels(ii);
        stats = regionprops(segIm == blobLabel,'BoundingBox','Area','Centroid');
        bb = uint16(stats.BoundingBox');
        cropIm = segIm(bb(2):bb(2)+bb(4)-1,bb(1):bb(1)+bb(3)-1);
        blobBoundary = bwboundaries(cropIm == blobLabel, 'noholes');
        frameData.boundaries{ii} = {blobBoundary{1}(:,[2 1])};
        frameData.boundingBoxes{ii} = bb;
        frameData.centroids(ii,:) = stats.Centroid;
        frameData.areas(ii) = stats.Area;
    end
    outData{fIdx} = frameData;
end
%% Create mCell objects and parse data
mFrames = MCell.mFrames;
mFrames.initialize(nFrames);
nCells = sum(nBlobs);
mCells(1:nCells,1) = MCell;
cIdx = 0;
for fIdx = 1:nFrames
    frameData = outData{fIdx};
    if ~isempty(frameData)
        phaseTime = allPhaseTimes(fIdx);
        for ii=1:nBlobs(fIdx)
            cIdx = cIdx+1;
            mCells(cIdx,1) = MCell(...
                cIdx, fIdx, 1, ...
                'blobLabels',frameData.blobLabels(ii), ...
                'detectionTimes', phaseTime,...
                'boundingBoxes',frameData.boundingBoxes{ii},...
                'boundaries', frameData.boundaries{ii},...
                'areas', frameData.areas(ii), ...
                'centroids', frameData.centroids(ii,:));
        end
        cIdxs = cIdx-nBlobs(fIdx)+1:cIdx;
        keySet = frameData.blobLabels;
        valueSet = num2cell(mCells(cIdxs));
    else
        keySet = [];
        valueSet = {};
    end
    mFrames.setPointersInFrame(fIdx,keySet,valueSet);
end
%% Save mCell objects in position output folder
saveName = expInfoObj.getMCellMatPath(posName);
MCell.saveMCells(saveName, mCells);
fprintf('Created %d MCell objects.\n', nCells);