function translation = adjustImages(sourceDir, outputDir, param)
% adjustImages(sourceDir, outputDir, param)
%
% Adjusts images from sourceDir as follows: 
%   - reorientation: up/down flip and left/right flip. 
%   - rotation: defined by angular displacement parameter
%   - cropping: defined by cropping region parameter
%
% The parameters are defined in param structure:
%   flipFlags - binary 2-tuple [f1 f2]. If f1=1, images are flipped up to
%               down. If f2=1, images are flipped left to right.
%               If [], images are not flipped.
%   angularDisplacement - counter-clockwise image rotation, in degrees
%   roiInPhase - the region of interest for phase contrast images, in phase
%                pixels. Defined for a phase contrast image after rotation
%                and flipping.
%   barcodeROI - location of a barcode in a reoriented image before
%                cropping, used for frame stabilization. Centers of the
%                detected barcode spots are shifted vertically to be in the
%                middle of barcodeROI. This allows for image stabilization
%                between different positions.
%
% The adjusted images are saved in outputDir. 
%% Initialization
import FastSeg.Preprocess.adjustSingleImage

listIm = dir(fullfile(sourceDir,'*.tif*'));
if isempty(listIm)
    error('The file format is neither .tif or .tiff');    
end
imRange = param.rangePhaseImages;
if ~isempty(imRange)
    listIm = listIm(imRange);
end
nFrames = numel(listIm);
% Read parameters
nCores = param.parNumWorkers;
cropROI = param.roiInPhase;
if any(param.flipFlags)
    flipudFlag = param.flipFlags(1);
    fliplrFlag = param.flipFlags(2);
else
    flipudFlag = 0;
    fliplrFlag = 0;
end
angDisp = param.angularDisplacement;
barcodeROI = param.barcodeROI;
%% Main code
if ~isempty(barcodeROI) % Do frame stabilization
    % Crop subimage with barcode from the first image
    % after reorientation and rotation
    firstIm = tiffread(fullfile(sourceDir,listIm(1).name));
    firstIm = adjustSingleImage(firstIm,flipudFlag,fliplrFlag,angDisp,[]);
    barcodeIm = imcrop(firstIm, barcodeROI);
    cropFirstIm = imcrop(firstIm,cropROI);
    translation = zeros(nFrames,2);
    % Centralize barcode spots vertically in the barcode region
    [~, centers] = getBarcodeLocation(barcodeIm);
     yc = round(median(centers(:,2)));
     yc = 0; %modification for horizontal
     yshift = round((barcodeROI(4)+1)/2)-yc;
     yshift=0;
    if yshift
        translation(1,2) = yshift;
        shiftFirstIm = imtranslate(firstIm,[0 yshift]);
        cropFirstIm = imcrop(shiftFirstIm,cropROI);
        barcodeIm = imcrop(shiftFirstIm, barcodeROI);
    end
    imwrite(cropFirstIm, fullfile(outputDir,listIm(1).name));
    parfor(i = 2:nFrames, nCores)
        im = tiffread(fullfile(sourceDir,listIm(i).name));
        [im, translation(i,:)] = adjustSingleImage(im,flipudFlag,fliplrFlag,angDisp,cropROI,barcodeIm,barcodeROI);
        imwrite(im, fullfile(outputDir,listIm(i).name));
    end
else
    translation = [];
    parfor(i = 1:nFrames, nCores)
        im = tiffread(fullfile(sourceDir,listIm(i).name));
        im = adjustSingleImage(im,flipudFlag,fliplrFlag,angDisp,cropROI);
        imwrite(im, fullfile(outputDir,listIm(i).name));
    end
end