function [im, translation] = adjustSingleImage(im, flipudFlag, fliplrFlag, angularDisplacement, cropROI, barcodeIm, barcodeROI)
% This function preprocess single image. Image can be oreoriented, rotated
% and cropped. Also, translation factor can be computed for image
% stabilization.
%
% Input:
%   im - input image, represented as a uint16 matrix
%   flipudFlag - logical. If true, image is flipped upside down
%   fliplrFlag - logical. If true, image is flipped left to right
%   angularDisplacement - angle in radians. Image is rotated in the
%                       counterclockwise direction by the angle.
%   cropROI - 4-element vector with the form [XMIN YMIN WIDTH HEIGHT].
%             A subimage determined by cropROI is cropped using imcrop
%   barcodeIm - optional, image with barcode dots. Used for stabilization
%               by registering barcodes in all frames to the first frame.
%   barcodeROI - location of barcodeIm in the first frame.
%
% Output:
%   im - adjusted image, represented as a uint16 matrix
%   translation - translation vector between this image and the image in
%                 the first frame.

if flipudFlag
    im = flipud(im);
end

if fliplrFlag
    im = fliplr(im);
end

if angularDisplacement
    im = imrotate(im,angularDisplacement,'bilinear');
end

if nargin<6
    barcodeIm = [];
elseif nargin<7
    error('Missing input argument barcodeROI');
end

% It is assumed that the image rotated so that growth channels are
% horizontal and the barcode is on the top.
if ~isempty(barcodeIm)
    shift = min(barcodeROI(3:4));
    firstRow = max( 1, barcodeROI(2)-shift );
    lastRow = min( sum(barcodeROI([2 4]))+shift, size(im,1) );
    firstCol = max( 1, barcodeROI(1)-shift );
    lastCol = min( sum(barcodeROI([1 3]))+shift, size(im,2));
    subIm = im(firstRow:lastRow,firstCol:lastCol);
    c = normxcorr2(barcodeIm,subIm);
    [~, imax] = max(c(:));
    [ypeak, xpeak] = ind2sub(size(c),imax(1));
    offset = [xpeak-size(barcodeIm,2)+firstCol-1, ...
        ypeak-size(barcodeIm,1)+firstRow-1];
    initOffset = barcodeROI(1:2)-1;
    translation = initOffset-offset;
    im = imtranslate(im,translation);
end

if ~isempty(cropROI)
    im = imcrop(im,cropROI);
end