function [channelLocations, channels] = detectGrowthChannels(im,channelWidth,channelMask,nChannels,minChannelDist,emptyChannels,initChannelLocations,debugFlag)
% channelLocations = detectGrowthChannels(im,channelWidth,channelMask,nChannels,minChannelDist,previousChannels,debugFlag)
% Detects growth channels in a phase contrast image of the mother machine.
%
% Input:
%   im - phase contrast image of growth channels in mother machine.
%   channelWidth - width of a channel, in pixels, must be an odd number!!!
%   channelMask - binary image representing channel with offset.
%   nChannels - number of channels to be detected.
%   minChannelDist - minimum distance between channels, in pixels.
%   emptyChannels - indices of empty channels including barcodes
%   initChannelLocations - channel peak estimates. The default value is [].
%   debugFlag - logical, if 1, plot image with highlighted channels.
%               The default value is 0.
%
% Output:
%   channelLocations - array of locations of channel centerlines, in pixels
%   channels - matrix where row i is a 4-element vector with the form 
%              [XMIN YMIN WIDTH HEIGHT] that defines the bounding box of
%              channel i.
%
% Last modified: 
% SZ, 2017-11-17. Increased max correlation offset using a
% fraction of channel width. Also, more careful with choosing channel
% subimage for registration - use different offsets from above and below.
% SZ, 2019-03-20. Added debugFlag.
% SZ, 2019-11-22. Added option of multiple empty channels (barcodes are
% counted as channels). Removed parameter nExcludedRows.
% SZ, 2020-05-27. Added channels' bounding boxes as output.

if nargin<7
    initChannelLocations = [];
end

if nargin<8
    debugFlag = 0;
end

nRows = size(im,1);
%% Find an estimate of the location of the peaks
if ~isempty(initChannelLocations)
    channelLocations = initChannelLocations;
else
    sumArray = smooth(sum(im,2),channelWidth);
    [~, peaks] = findpeaks(sumArray,'NPeaks',nChannels,...
        'MINPEAKDISTANCE',minChannelDist,'SortStr','descend');
    cutOffPeakLocation = floor(channelWidth/1.5);
    peaks(peaks < cutOffPeakLocation) = [];
    peaks(peaks > nRows-cutOffPeakLocation+1) = [];
    if length(peaks) < nChannels
        fprintf('Detected only %d growth channels out of %d.\n',...
            length(peaks), nChannels);
        channelLocations = NaN(1,nChannels-length(emptyChannels)+1);
        return
    end
    channelLocations = sort(peaks);
    channelLocations(emptyChannels(2:end)) = [];
end
%% Get a finer detection using registration
% max channel position shift is bounded to 50% of channel width
MAX_CORR_OFFSET = ceil(0.5*channelWidth);

nRowsChannelMask = size(channelMask,1);
maskHw = floor(nRowsChannelMask/2);

for channelIdx = 1:length(channelLocations)
    currPeak = channelLocations(channelIdx);
    
    % Check that the channel mask fits to the image for the estimated
    % channel position
    if maskHw > min(currPeak-1,nRows-currPeak)
        continue
    end
    
    % Choose cross-correlation offsets from each side of the channel
    uOffset = min(currPeak-maskHw-1, MAX_CORR_OFFSET);
    dOffset = min(nRows-currPeak-maskHw, MAX_CORR_OFFSET);
    
    % Create sub-image of the current channel      
    subIm = im(currPeak-maskHw-uOffset:currPeak+maskHw+dOffset,:);
           
    % Compute normalised cross correlation between channelMask and sub-image
    c = normxcorr2(channelMask,subIm);
    [ypeak, ~] = find(c==max(c(:)),1);
    corrOffset = ypeak-nRowsChannelMask;
    
    % This is where the center of the mask (along the x-axis) should be
    channelLocations(channelIdx) = currPeak-uOffset+corrOffset;
end
%% Compute bounding box for each growth channel
[maskWidth, ind] = min(size(channelMask));
if size(channelLocations,1)==1
    channelLocations = channelLocations';
end
channels = zeros(length(channelLocations),4);
if ind==1
    channels(:,1) = 1;
    channels(:,2) = channelLocations-maskHw;
    channels(:,3) = size(im,2)-1;
    channels(:,4) = maskWidth-1;
else
    channels(:,1) = channelLocations-maskHw;
    channels(:,2) = 1;
    channels(:,3) = maskWidth-1;
    channels(:,4) = size(im,1)-1;
end
%% Debugging
if debugFlag
    figure
    imshow(im), hold on
    for i=1:length(channelLocations)
        plot([1 size(im,2)], [channelLocations(i) channelLocations(i)],'g-');
    end
    hold off
end