function [Ts, statusFlags] = registerBfToPhase(expInfoObj, posName, bfChanName, offset, debugFlag)
% registerBfToPhase(expInfoObj, posName, bfChanName, offset, debugFlag)
% 
% Registration of bright-field images to preprocessed phase images.
% Requires initial transformation matrix computed for the pair of the first
% frames. Registration algorithm is MATLAB's cross-correlation. The
% computed transformation matrices for each pair of images is saved in
% expInfoObj
% 
% Input:
%   expInfoObj - expInfo object.
%   posName - string with the position name to be analysed.
%   bfChanName - string with bright-field channel name.
%   offset - matrix of translation vectors for frame stabilization, leave
%            empty if no stabilization is done.
%   debugFlag - logical, if 1 shows pairs of phase and transformed
%               bright-field images after the registration.
%
% Output (optional):
%   Ts - 3x3xnFrames array with transformation matrices, where nFrames is
%        the number of bright-field images
%   statusFlags - logical array, 1 indicates a registration error

% Last modified:
% 2018-03-09, SZ. Added frame stabilization using barcodes.
% Reworked how 3-d arrays with transformation matrices are stored. The
% third dimension is for frame indices.
% 2018-09-25. Fixed bug with uint16 to double conversion of fluo/bf roi
% 2019-03-07. Adapted for parfor loops between positions. Moved
% registration of fluo images into a separate function

if ~exist('debugFlag','var')
    debugFlag = 0;
end

sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');
phaseChanName = expInfoObj.getChannelNames('phase');
phaseIdxs = expInfoObj.getIndices(posName,phaseChanName);
bfIdxs = expInfoObj.getIndices(posName,bfChanName);
parameters = expInfoObj.getParameters();
regMax = parameters.registrationMaxOffset;
blackShiftedBF = parameters.blackShiftedBF;
phaseRoi = parameters.roiInPhase;
nCores = parameters.parNumWorkers;

if ~strcmp(parameters.transMatPath, '')
    tmp = load(parameters.transMatPath);
    T0 = tmp.T;
else
    T0 = parameters.transMat;
end

phaseDir = fullfile(outputDir,posName,'PreprocessedPhase');
bfDir = fullfile(sourceDir,posName,bfChanName);
listPhase = dir(fullfile(phaseDir,'*.tif*'));
listBf = dir(fullfile(bfDir,'*.tif*'));

[~, phaseBfIdxs] = intersect(phaseIdxs, bfIdxs);
listPhase = listPhase(phaseBfIdxs);

bfImRange = expInfoObj.getRange(posName, bfChanName);
listBf = listBf(bfImRange);
nBf = length(listBf);

Ts = repmat(T0,1,1,nBf);
if ~isempty(offset)
    offset = offset(phaseBfIdxs,:)';
    Ts(3,1:2,:) = reshape(Ts(3,1:2,:),[2,nBf]) + offset;
end
    
statusFlags = false(nBf,1);
R = imref2d(phaseRoi([4 3])+1);

if ~debugFlag
    parfor (i=1:nBf, nCores)
        phaseIm = imread(fullfile(phaseDir,listPhase(i).name));
        bfIm = tiffread(fullfile(bfDir,listBf(i).name));
        T0 = Ts(:,:,i);
        transBfIm = imwarp(bfIm,affine2d(T0),'OutputView',R,'FillValues',0);
        [statusFlags(i), T] = FastSeg.registerWithCrossCorrelation(...
            phaseIm, transBfIm, T0, regMax, blackShiftedBF);
        Ts(:,:,i) = T;
    end
else
    figure
    for i = 1:nBf
        phaseIm = imread(fullfile(phaseDir,listPhase(i).name));
        bfIm = tiffread(fullfile(bfDir,listBf(i).name));
        T0 = Ts(:,:,i);
        transBfIm = imwarp(bfIm,affine2d(T0),'OutputView',R,'FillValues',0);
        [statusFlags(i), T] = FastSeg.registerWithCrossCorrelation(...
            phaseIm, transBfIm, T0, regMax, blackShiftedBF);
        Ts(:,:,i) = T;
        transBfIm2 = imwarp(bfIm,affine2d(T),...
                           'OutputView',imref2d(size(phaseIm)),...
                           'FillValues',0);
        imshowpair(phaseIm,transBfIm2);
        pause();
    end
end