function fluoTs = registerFluoToPhase(expInfoObj,posName,fluoChanName,bfChanName,offset,bfTs)
% fluoTs = registerFluoToPhase(expInfoObj,posName,fluoChanName,bfChanName,offset,bfTs)
% 
% Registration of fluorescence images to preprocessed phase images.
% 
% Input:
%   expInfoObj - expInfo object.
%   posName - string with the position name to be analysed.
%   fluoChanName - string with fluorescence channel name.
%   bfChanName - string with bright-field channel name. Leave empty, if
%                there is no bright-field channel.
%   offset - matrix of translation vectors for frame stabilization, leave
%            empty if no stabilization is done.
%   bfTs - 3x3xnFrames array with transformation matrices from bright-field
%          to the corresponding phase images. Optional, if not provided,
%          the matrices are extracted from expInfoObj.
%
% Output (optional):
%   fluoTs - 3x3xnFrames array with transformation matrices, where nFrames
%            is the number of fluorescence images

parameters = expInfoObj.getParameters();
if isempty(bfChanName)
    if ~strcmp(parameters.transMatPath, '')
        tmp = load(parameters.transMatPath);
        T0 = tmp.T;
    else
        T0 = parameters.transMat;
    end
    phaseChanName = expInfoObj.getChannelNames('phase');
    phaseIdxs = expInfoObj.getIndices(posName,phaseChanName);
    fluoIdxs = expInfoObj.getIndices(posName,fluoChanName);
    [~, phaseFluoIdxs] = intersect(phaseIdxs, fluoIdxs);
    nFluo = length(fluoIdxs);
    
    fluoTs = repmat(T0,1,1,nFluo);
    if ~isempty(offset)
        offset = offset(phaseFluoIdxs,:)';
        fluoTs(3,1:2,:) = reshape(fluoTs(3,1:2,:),[2,nFluo]) + offset;
    end
else
    nCores = parameters.parNumWorkers;
    if ~exist('bfTs','var')
        bfTs = expInfoObj.getAllT(posName,bfChanName);
    end
    fluoIdxs = expInfoObj.getIndices(posName,fluoChanName);
    bfIdxs = expInfoObj.getIndices(posName,bfChanName);
    [~,idxs] = intersect(bfIdxs,fluoIdxs);
    bfTs = bfTs(:,:,idxs);
    bfRoi = expInfoObj.getRoi(bfChanName);
    bfRoi = bfRoi(1:2);
    if any(isnan(bfRoi))
        bfRoi = [1 1];
    end
    fluoRoi = expInfoObj.getRoi(fluoChanName);
    fluoRoi = fluoRoi(1:2);
    if any(isnan(fluoRoi))
        fluoRoi = [1 1];
    end
    fluo2BfT = getTranslationMatrix(double(fluoRoi) - double(bfRoi));
    fluoTs = nan(size(bfTs));
    parfor (idx = 1:length(fluoIdxs), nCores)
        bfT = bfTs(:,:,idx);
        fluoTs(:,:,idx) = fluo2BfT * bfT;
    end
end