function allChannels = removeMMBackground(expInfoObj, posName)
% allChLocs = removeMMBackground(expInfoObj, posName)
%
% This function is used to preprocess images obtained by mother-machine by
% removing growth channels background. This is done as follows:
%   - detection of channel locations in the first frame
%   - detection of empty channel location using cross-correlation wih the
%     channel mask
%   - removal of channel background in each image using an empty channel. 
% The modified images are renamed with a prefix and replace old images
%
% Input:
%   expInfoObj - expInfo object.
%   posName - string with position name to be analysed
%
% Output (optional):
%   allChannels - Mx4xN matrix where M is the number of growth channels,
%                 N is the number of frames. Element (i,:,j) is a rect
%                 4-vector with coordinates of bounding box of channel i
%                 in frame j. 

% Last modified:
% SZ, 2018-04-23. Added optional parameter emptyChannel. The
% outer loop over positions is moved to FastSeg.preprocessImages.
% SZ, 2018-09-21. Fixed error in computing channel length from phase ROI
% (only if it is provided by user).
% SZ, 2019-03-15. Added optional output of growth channel locations for
% parfor positions loop.
% SZ, 2019-11-25. Added multiple empty channels that can be used to get rid
% of barcodes which are not set to 0 anymore.
%% Initialization
import FastSeg.Preprocess.*

outDir = expInfoObj.getPathName('output');
parameters = expInfoObj.getParameters();
emptyChannelIndices = parameters.emptyChannel;
chW = parameters.channelWidth;
chOffset = parameters.channelOffset;
nCh = parameters.nGrowthChannels;
minChDist = parameters.minChannelDist;
nCores = parameters.parNumWorkers;
barcodeROI = parameters.barcodeROI;
bkgrSubtraction = ~strcmpi(parameters.segmentationAlg,'pranet');
%% Get list of phase images
imDir = fullfile(outDir,posName,'PreprocessedPhase');
imList = dir(fullfile(imDir,'*.tif*'));
%% Compute channel mask
phaseROI = parameters.roiInPhase;
if ~isempty(phaseROI)
    chL = phaseROI(3)+1; % channel length is the image width
    imH = phaseROI(4)+1;
else
    imInfo = imfinfo(fullfile(imDir,imList(1).name));
    chL = imInfo.Width;
    imH = imInfo.Height;
end
chMask = computeChannelMask(chW,chL,chOffset);
halfW = floor(size(chMask,1)/2);
%% Detection of channels in the first frames
firstIm = tiffread(fullfile(imDir,imList(1).name));
[initChLocs, initChannels] = detectGrowthChannels(firstIm, chW,chMask,nCh,minChDist,emptyChannelIndices);
if isempty(emptyChannelIndices)
    emChIdx = findEmptyChannel(firstIm,initChLocs);
else
    channelIndices = 1:nCh;
    channelIndices(emptyChannelIndices(2:end)) = [];
    emChIdx = find(channelIndices == emptyChannelIndices(1));
    nCh = nCh-length(emptyChannelIndices)+1;
end
%% Computation
nFrames = numel(imList);
failedFrames = false(1,nFrames);
% Remove the empty channel and save the images
if ~isempty(barcodeROI)
    allChannels = repmat(initChannels,1,1,nFrames);
else
    nCh = nCh-length(emptyChannelIndices)+1;
    allChannels = zeros(nCh,4,nFrames);
    allChannels(:,1) = initChLocs;
end
parfor(frIdx = 1:nFrames, nCores)
    imFile = fullfile(imDir,imList(frIdx).name);
    im = uint16(tiffread(imFile));
    if frIdx>1 && isempty(barcodeROI)
        [chLocs, channels] = detectGrowthChannels(im,chW,chMask,nCh,minChDist,[],initChLocs);
        allChannels(:,:,frIdx) = channels;
    else
        chLocs = initChLocs;
    end
    if ~any(isnan(chLocs))
        if bkgrSubtraction
            % Make sure that channels are far enough from the image borders
            currHalfW = min([halfW, chLocs(1)-1, imH-chLocs(end)]);
            emChannelLoc = chLocs(emChIdx);
            chRows = emChannelLoc-currHalfW:emChannelLoc+currHalfW;
            currEmChannel = im(chRows,:);
            % Remove the empty channel from each channel and rescale the
            % resulted channel
            newIm = uint16(zeros(size(im)));
            for chIdx = [1:emChIdx-1, emChIdx+1:nCh]
                currChLoc = chLocs(chIdx);
                chRows = currChLoc-currHalfW:currChLoc+currHalfW;
                currChannel = im(chRows,:);
                newChannel = double(currChannel)-double(currEmChannel);
                rescaledChannel = rescaleSingleImage(newChannel);
                newIm(chRows,:) = imadjust(rescaledChannel);
            end
            % Write the output image
            imwrite(newIm,imFile);
        end
    else % Not all growth channels are found
        failedFrames(frIdx) = 1;
    end
end
if any(failedFrames)
    error([posName ': not all channels detected in frames ' ...
        num2str(find(failedFrames))]);
end
if nargout == 0
    expInfoObj.setChannelLocations(posName,allChannels);
end