function [errFlag, errMsg] = reorientImages(sourceDir, opt)
% Function to reorient the images regarding: rotation by an angle of 90
% degrees, up/down flip and left/right flip. Set the different flags for
% the different changes and it will apply them and save the images in the
% output folder.
%
% Input:
%   sourceDir - path to directory with input images
%   opt       - structure with optional arguments:
%                 outputDir      - path to store rescaled images. 
%                                  By default, sourceDir.
%                 imRange        - indices of images to be rescaled. 
%                                  By default, all images in sourceDir.
%                 reorientParams - an array that defines reorientation
%                                  (number of 90 degree rotations, up/down
%                                  flip and left/right flip)
%                                  By default, all are set to 1.
%                 parNumWorkers  - number of workers for parallel pool. 
%                                  By default, use local profile.
% 
% Output:
%   errFlag   - 1 if failed, or 0 otherwise
%   errMsg    - string with error description
%
% Last modified: Spartak Zikrin, 2017-08-22

errFlag = 1;

if nargin == 2 && isfield(opt,'outputDir')
    outputDir = opt.outputDir;
else
    outputDir = sourceDir;
end

listIm = dir(strcat(sourceDir,filesep,'*.tif*'));

if isempty(listIm)
    errMsg = 'The file format is neither .tif or .tiff';
    return;
end

if nargin == 2 && isfield(opt,'imRange')
    imRange = opt.imRange;
    listIm = listIm(imRange);
end

if nargin == 2 && isfield(opt,'parNumWorkers')
    parNumWorkers = opt.parNumWorkers;
else
    localCluster = parcluster('local');
    parNumWorkers = localCluster.NumWorkers;
end

if nargin == 2 && isfield(opt,'orientationParams')
    nRotations = opt.orientationParams(1);
    flipudFlag = opt.orientationParams(2);
    fliplrFlag = opt.orientationParams(3);
else
    nRotations = 1;
    flipudFlag = 1;
    fliplrFlag = 1;
end

parfor(imIdx = 1:length(listIm), parNumWorkers)
    im = imread(strcat(sourceDir,filesep,listIm(imIdx).name));

    if nRotations
        im = rot90(im,nRotations);
    end
    if flipudFlag
        im = flipud(im);
    end
    if fliplrFlag
        im = fliplr(im);
    end

    imwrite(im,strcat(outputDir,filesep,'or_',listIm(imIdx).name));   
    
end

if strcmp(outputDir, sourceDir)
    parfor(imIdx = 1:length(listIm), parNumWorkers)
        delete(strcat(outputDir,filesep,listIm(imIdx).name));
    end
end

errFlag = 0;
errMsg = '';

end