function [errFlag, errMsg] = rescaleImages(sourceDir,opt)
% Rescales the images to use the full range of values for uint16 images.
%
% Input:
%   sourceDir   - path to directory with input images
%   opt         - structure with optional arguments:
%                   outputDir     - path to store rescaled images. 
%                                   By default, sourceDir.
%                   imRange       - indices of images to be rescaled. 
%                                   By default, all images in sourceDir.
%                   parNumWorkers - number of workers for parallel pool. 
%                                   By default, use local profile.
% 
% Output:
%   errFlag     - 1 if failed, or 0 otherwise
%   errMsg      - string with error description
%
% Last modified: Spartak Zikrin, 2017-08-22

errFlag = 1;

if nargin < 1 
    errMsg = 'Input images path is not provided';
    return
end

if nargin == 2 && isfield(opt,'outputDir')
    outputDir = opt.outputDir;
else
    outputDir = sourceDir;
end

listIm = dir(strcat(sourceDir,filesep,'*.tif*'));

if isempty(listIm)
    errMsg = 'The file format is neither .tif or .tiff';
    return;
end

if nargin == 2 && isfield(opt,'imRange')
    imRange = opt.imRange;
    listIm = listIm(imRange);
end

if nargin == 2 && isfield(opt,'parNumWorkers')
    parNumWorkers = opt.parNumWorkers;
else
    localCluster = parcluster('local');
    parNumWorkers = localCluster.NumWorkers;
end

% Compute the maximum and the minimum values in the dataset
maxVal = zeros(length(listIm),1);
minVal = zeros(length(listIm),1);

parfor(imIdx = imRange,parNumWorkers)
    im = uint16(imread(strcat(sourceDir,filesep,listIm(imIdx).name)));
    maxVal(imIdx) = max(max(im));
    minVal(imIdx) = min(min(im));
end

maxVal = max(maxVal);
minVal = min(minVal);
newRatio = computeRescaleRatio(maxVal,minVal);

% Rescale the images and save them as uint16
parfor(imIdx = 1:length(listIm), parNumWorkers)
    im = uint16(imread(strcat(sourceDir,filesep,listIm(imIdx).name)));
    tempIm = (im-minVal)*newRatio;
    imwrite(tempIm,strcat(outputDir,filesep,'rescaled_',listIm(imIdx).name));
end

if strcmp(outputDir,sourceDir)
    parfor(imIdx = 1:length(listIm), parNumWorkers)
        delete(strcat(outputDir,filesep,listIm(imIdx).name));
    end
end

errFlag = 0;
errMsg = '';

end