function [bfTs, fluoTs, failRegs, chLocs] = preprocessImages(expInfoObj, posName)
% This function preprocess phase-contrast images by their reorientation,
% correction of angular displacement and cropping. The preprocessed
% images are used to register bright-field images using cross-correlations 
% and the resulting transformation matrices are stored in expInfo object.
% In case of mother-machine, images are further preprocessed by removing
% growth channels background.
% Executes outer for-loops along positions in parallel.

% Last modified: SZ, 2018-03-09. Frame stabilization
%% Initialization
import FastSeg.Preprocess.*

sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');
bfChanNames = expInfoObj.getChannelNames('bf');
fluoChanNames = expInfoObj.getChannelNames('fluo');
phaseChanName = expInfoObj.getChannelNames('phase');
nChannelsBf = numel(bfChanNames);
nChannelsFluo = numel(fluoChanNames);
parameters = expInfoObj.getParameters();
isMM = parameters.isMothermachine;
%% Create output folder
sourceImDir = fullfile(sourceDir,posName,phaseChanName);
if ~exist(fullfile(outputDir,posName), 'dir')
    mkdir(fullfile(outputDir,posName));
end
outputImDir = fullfile(outputDir,posName,'PreprocessedPhase');
if exist(outputImDir, 'dir')
    rmdir(outputImDir, 's')
end
mkdir(outputImDir);
%% Reorient, rotote, crop, and stabilize phase-contrast images
translation = adjustImages(sourceImDir,outputImDir,parameters);
%% Register bright-field images to phase-contrast images
bfTs = cell(1,nChannelsBf);
failRegs = cell(1,nChannelsBf);
for chIdx = 1:nChannelsBf
    bfChanName = bfChanNames{chIdx};
    [bfTs{chIdx}, failRegs{chIdx}] = registerBfToPhase(expInfoObj,posName,bfChanName,translation);
end
%% Register fluorescence images to phase-contrast images
fluoTs = cell(1,nChannelsFluo);
for chIdx = 1:numel(fluoChanNames)
    fluoChanName = fluoChanNames{chIdx};
    if isempty(bfChanNames)
        bfChanName = '';
        currBfTs = [];
    else
        bfChanName = bfChanNames{chIdx};
        if length(bfChanNames) == 1
            currBfTs = bfTs{chIdx};
        else
            currBfTs = bfTs{1};
        end
    end
    fluoTs{chIdx} = registerFluoToPhase(expInfoObj,posName,fluoChanName,bfChanName,translation,currBfTs);
end
%% Mother-machine background subtraction
if isMM
    %chLocs = removeMMBackground(expInfoObj,posName);
    chLocs = [];
else
    chLocs = [];
end