function [errFlag, transformMatrix, c, xpeak, ypeak] = registerWithCrossCorrelation(im1,im2,initT,regMax,blackShiftedBF)
% transformMatrix = registerWithCrossCorrelation(im1,im2,initT,regMax,blackShiftedBF)
%
% Coorects translation between images im1 and im2 with the use of 
% cross-correlation. im2 is already transformed to im1 by transformation 
% matrix initT.
% Images are of equal size. Since im2 is a result of image transformation,
% a non-zero rectange subimage of im2 is used for registration.
%
% Input:
%   initT - affine transform matrix from im2 to im1
%   im1 - phasecontrast image, in uint16
%   im2 - brightfield image, in uint16
%   regMax - scalar, defines the cross-correlation mask size and limits the
%            max translation step
%   blackShiftedBF - 1, if cells are darker than background, 0 otherwise
% 
% Output:
%   errFlag - 1, if image translation is above max threshold or wrong
%             correlation sign.
%   transformMatrix - 3x3 matrix that transforms brightfield image to 
%                     phasecontrast image
%   c - normalized 2D cross-correlation between im1 and im2
%   xpeak, ypeak - coordinates of max(c) (or min(c), depending on
%                  blackShiftedBF)
%
% Last modified: SZ, 2017-11-17

if any(size(im1) ~= size(im2))
    error('Images must be of the same size');
end

errFlag= false;

% crop subimage from the transformed BF image
[rectROI,subIm2] = FastSeg.Preprocess.cropRectangle(im2);
subIm1 = imcrop(im1,rectROI);


% Cut off 10% of bf image
cutOff = ceil(size(subIm2)*0.1);
subSubIm2 = subIm2(cutOff(1)+1:end-cutOff(1),cutOff(2)+1:end-cutOff(2));

c = normxcorr2(subSubIm2,subIm1);
            
mid_c_coords = round(size(c)/2);
c_mask = false(size(c));
c_mask(mid_c_coords(1)-regMax : mid_c_coords(1)+regMax, ...
       mid_c_coords(2)-regMax : mid_c_coords(2)+regMax) ...
    = true;
c(~c_mask) = 0 ;

if blackShiftedBF
    [val, imax] = max(c(:));
else
    [val, imax] = min(c(:));
end
if val == 0
    errFlag = true;
    warning('Cross-correlation peak value is zero');
end
[ypeak, xpeak] = ind2sub(size(c),imax(1));

corrOffset = [xpeak-size(subSubIm2,2)-cutOff(2) ypeak-size(subSubIm2,1)-cutOff(1)];

if max(abs(corrOffset)) > regMax
    errFlag = true;
    warning('Correlation offset exceeds max limit');
    corrOffset = 0;
end

transformMatrix = initT;

transformMatrix(3,1:2) = transformMatrix(3,1:2) + corrOffset;

% im3 = imtranslate(im2, -corrOffset); 
% figure, 
% imshow(imfuse(im2,im3,'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));
% figure,imshow(im1Orig);
% figure,imshow(im2Orig);
% figure,imshow(imtranslate(im2Orig,-corrOffset));
