function segmentCells(expInfoObj,posName, pipelineDir)
% segmentCells(expInfoObj, posName, pipelineDir)
% 
% Calls python script to run a segmentation algorithm to segment
% cells in phase images. Sequential code execution.
% Supported segmentation algorithms:
% POE - "per object ellipse fit" segmentation.
% CBA - "curvature based approximation" segmentation. Slow, not recommended
%       for time-lapse experiments.
%
% Input:
%   expInfoObj - expInfo object
%   posName - string, current position
%   pipelineDir - path to directory with the ImAnalysis pipeline, optional

% 2019-09-09, SZ: Changed input arguments to use expInfoObj.
% 2019-12-12, SZ: Added deep learning segmentation by pranet

if nargin<3 || isempty(pipelineDir)
    pipelineDir = ImAnalysis_setup();
end

p = expInfoObj.getParameters();

if ~p.parforPositions
    FastSeg.segmentCells_parfor(expInfoObj,posName, pipelineDir);
    return
end

outputDir = expInfoObj.getPathName('output');
segInputDir = fullfile(outputDir,posName,'PreprocessedPhase');
segOutputDir = fullfile(outputDir,posName,'SegmentedPhase');

if exist(segOutputDir,'dir')
    rmdir(segOutputDir,'s');
end
mkdir(segOutputDir);

if strcmp(p.segmentationAlg,'POE')
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsPOE.py');
    sysCommand =  [...
        'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
        'eval "$(pyenv init -)";pyenv activate POE;',...
       'python2.7 "', scriptPath, '" "', segInputDir, '" "' , segOutputDir, '" ', ...
        int2str(p.minWidth), ' ', int2str(p.maxWidth), ' ', ...
        int2str(p.minLength), ' ', int2str(p.maxLength), ' ', ...
        int2str(p.minCellArea), ' ', int2str(p.maxCellArea), ' ', ...
        num2str(p.minEllipseRatio), ' ', num2str(p.maxEllipseRatio), ' ', ...
        num2str(p.segmentationThreshold), ' ', ...
        int2str(p.segmentationThresholdWindow), ' ', ...
        num2str(p.gaussKernSigma), ' ',  ...
        int2str(p.discardCellsOnBorder)];
elseif strcmp(p.segmentationAlg,'CBA')
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsCBA.py');
    sysCommand = [...
        'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
        'eval "$(pyenv init -)";pyenv activate POE;',...
        'python2.7 "', scriptPath, '" "', segInputDir, '" "' , segOutputDir, '" ', ...
        int2str(p.minWidth), ' ', int2str(p.maxWidth), ' ', ...
        int2str(p.minLength), ' ', int2str(p.maxLength), ' ', ...
        int2str(p.minCellArea), ' ', int2str(p.maxCellArea), ' ', ...
        num2str(p.segmentationThreshold), ' ', ...
        num2str(p.gaussKernSigma), ' ',  ...
        int2str(p.discardCellsOnBorder), ' ', ...
        int2str(0)];
elseif strcmpi(p.segmentationAlg,'PraNet')
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsPranet.py');
    sysCommand =  [...
        'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
        'eval "$(pyenv init -)";pyenv activate PraNet;',...
        'python3 "', scriptPath, '" "', segInputDir, '" "' , segOutputDir, '" "', ...
        p.pranetModelPath, '" ', num2str(p.segmentationThreshold), ' ', ...
        int2str(p.minCellArea), ' ', int2str(p.discardCellsOnBorder)];

else
    error('ImAnalysis supports only POE and CBA segmentation algorithms');
end

[status, errMsg] = system(sysCommand);

if status
    error('Segmentation error in %s:\n%s', posName, errMsg);
end