function segmentCells_parfor(expInfoObj, posName, pipelineDir)
% segmentCells_parfor(outputDirPos, param, pipelinePath)
% 
% Calls python script to run a segmentation algorithm to segment 
% cells in phase images. The code is parallelized.
% Supported segmentation algorithms:
% POE - "per object ellipse fit" segmentation. Uses Matlab parfor loop.
% CBA - "curvature based approximation" segmentation. Slow, not recommended
%       for time-lapse experiments. Uses Python's multiprocessing pool to
%       segment in parallel each image. 
%
% Input:
%   expInfoObj - expInfo object
%   posName - string, current position
%   pipelineDir - path to directory with the ImAnalysis pipeline, optional

% 2019-09-09, SZ: Handle the case with no blobs in a frame. Add CBA
% segmentation algorithm (curvature-based approach). The python scripts
% accept both a folder with images and a single image as input arguments.

if nargin<3 || isempty(pipelineDir)
    pipelineDir = ImAnalysis_setup();
end

p = expInfoObj.getParameters();

outputDir = expInfoObj.getPathName('output');
segInputDir = fullfile(outputDir,posName,'PreprocessedPhase');
segOutputDir = fullfile(outputDir,posName,'SegmentedPhase');

if exist(segOutputDir,'dir')
    rmdir(segOutputDir,'s');
end
mkdir(segOutputDir);

minWidth = p.minWidth;
maxWidth = p.maxWidth;
minLength = p.minLength;
maxLength = p.maxLength;
minCellArea = p.minCellArea;
maxCellArea = p.maxCellArea;
minEllipseRatio = p.minEllipseRatio;
maxEllipseRatio = p.maxEllipseRatio;
segmentationThreshold = p.segmentationThreshold;
segmentationThresholdWindow = p.segmentationThresholdWindow;
gaussKernSigma = p.gaussKernSigma;
discardCellsOnBorder = p.discardCellsOnBorder;
model = p.pranetModelPath;
nCores = p.parNumWorkers;

if strcmpi(p.segmentationAlg,'POE')
    imList = dir(fullfile(segInputDir,'*.tif*'));
    imList = {imList.name};
    status = zeros(size(imList));
    errMsgs = cell(size(imList));
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsPOE.py');
    parfor (i=1:numel(imList), nCores)
        imPath = fullfile(segInputDir,imList{i});
        sysCommand =  [...
            'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
            'eval "$(pyenv init -)";pyenv activate POE;',...
            'python2.7 "', scriptPath, '" "', imPath, '" "' , segOutputDir, '" ', ...
            int2str(minWidth), ' ', int2str(maxWidth), ' ', ...
            int2str(minLength), ' ', int2str(maxLength), ' ', ...
            int2str(minCellArea), ' ', int2str(maxCellArea), ' ', ...
            num2str(minEllipseRatio), ' ', num2str(maxEllipseRatio), ' ', ...
            num2str(segmentationThreshold), ' ', int2str(segmentationThresholdWindow), ' ', ...
            num2str(gaussKernSigma), ' ', int2str(discardCellsOnBorder)];
        [status(i), errMsgs{i}] = system(sysCommand);
    end
    if any(status)
        i = find(status,1);
        error('Segmentation error in image %d:\n%s', i, errMsgs{i});
    end
elseif strcmpi(p.segmentationAlg,'CBA')
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsCBA.py');
    sysCommand = [...
        'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
        'eval "$(pyenv init -)";pyenv activate POE;',...
        'python2.7 "', scriptPath, '" "', segInputDir, '" "' , segOutputDir, '" ', ...
        int2str(minWidth), ' ', int2str(maxWidth), ' ', ...
        int2str(minLength), ' ', int2str(maxLength), ' ', ...
        int2str(minCellArea), ' ', int2str(maxCellArea), ' ', ...
        num2str(segmentationThreshold), ' ', ...
        num2str(gaussKernSigma), ' ',  ...
        int2str(discardCellsOnBorder), ' ',...
        int2str(nCores)];
    [status, errMsg] = system(sysCommand);
    if status
        error('Segmentation error:\n%s', errMsg);
    end
elseif strcmpi(p.segmentationAlg,'PraNet')
    scriptPath = fullfile(pipelineDir,'Segmentation','SegmentCellsPranet.py');
    imList = dir(fullfile(segInputDir,'*.tif*'));
    imList = {imList.name};
    status = zeros(size(imList));
    errMsgs = cell(size(imList));
    parfor (i=1:numel(imList), nCores)
        imPath = fullfile(segInputDir,imList{i});
        sysCommand =  [...
            'export PYENV_VIRTUALENV_DISABLE_PROMPT=1;'...
            'eval "$(pyenv init -)";pyenv activate PraNet;',...
            'python3 "',scriptPath,'" "',imPath,'" "',segOutputDir,...
            '" "',model,'" ',num2str(segmentationThreshold),' ', ...
            int2str(minCellArea),' ',int2str(discardCellsOnBorder)];
        [status(i), errMsgs{i}] = system(sysCommand);
    end
    if any(status)
        i = find(status,1);
        error('Segmentation error in %s, frame %d:\n%s', posName, i, errMsgs{i});
    end
else
    error('ImAnalysis supports only POE and CBA segmentation algorithms');
end