% function [dotCoordLSQ,LSQbNS_flag]=...
%    LSQfitDotsSingleFrame(dotCoord0,fluoFrame,fluoOffset,EMgain,ROIwidth)
%
% attempt LSQ fit of the suggested dots in dotCoord0 to a single
% fluorescent image fluoFrame, with mean offset fluoOffset and EM gain
% parameter EMgain (the EMgain parameter is not critical for the fit
% quality, but makes the background and spot amplitude get units close to
% counting photons).
%
% dotCoord0=[x y ...]. Suggested starting points for LSQ fits. Columns > 2
%                      are just passed on unchanged to dotCoordLSQ.
%
% fluoFrame : image to fit spots to.
% fluoOffset: camera offset image
% EMgain    : camera EM gain parameter (av. added counts/photon)
% ROIwidth  : side length (pixels) of the ROI square to use for fit
%
% dotCoordLSQ : same as dotCoord0, but with first two columns updated to
%               x,y positions from the LSQ fit.
% LSQbNS_flag=[bg N S rn exitFlag tFit];, where
%       b  = background parameter
%       N  = gaussian PSF amplitude (approximates #photons)
%       S  = gaussian PSF std
%       rn = residual norm per pixel, from lsqnonlin)
%       exitFlag : -3 means LSQ fit failed with an error, otherwise as
%       returned by lsqnonlin.
% 
% Martin Lindén, bmelinden@gmail.com, 2015-11-12
function [dotCoordLSQ,LSQbNS_flag]=...
    LSQfitDotsSingleFrame(dotCoord0,fluoFrame,fluoOffset,EMgain,ROIwidth)

dotCoordLSQ=dotCoord0;
LSQbNS_flag  =zeros(size(dotCoord0,1),6);
    
LSQopt = optimoptions('lsqnonlin','TolX',1e-6,'MaxIter',300,...
    'TolFun',1e-6,'MaxFunEvals',10000,'Display','none');    

for r=1:size(dotCoord0,1)
    tic
    % construct ROI for LSQ localization
    x0   =dotCoord0(r,1);
    y0   =dotCoord0(r,2);
    
    [riSpot,ciSpot,x0Spot,y0Spot]=LSQrefineDots.ROItransform(x0,y0,ROIwidth,ROIwidth,size(fluoFrame));
    spotROI=double(fluoFrame(riSpot,ciSpot)); % movie raw data
    bgROI=double(fluoOffset(riSpot,ciSpot)); % offset background to be subtracted
    
    % lsq fit
    lsqData=(spotROI-bgROI)/EMgain; % ~ photons + readout noise
    [rows,columns] = size(lsqData);
    lsqData=lsqData(:)';
    [x,y] = meshgrid(1:columns,1:rows);
    x = x(:)';y = y(:)';
    lnpLSQdot=[log(1) log(300) x0-x0Spot y0-y0Spot log(2)];
    PSFtarget=@(p)(lsqData-LSQrefineDots.psfGaussianSymmetric_9ptGQ(x,y,exp(p(1)),exp(p(2)),p(3),p(4),exp(p(5))));
    try
        [lnpLSQdot,resNorm,~,exitFlag]=lsqnonlin(PSFtarget,lnpLSQdot,[],[],LSQopt);
    catch me
        % if something goes wrong already here, we discard
        % the point
        tFit=toc;
        LSQbNS_flag(r,4:6)= [inf -3 tFit];
        continue
    end
    bg=exp(lnpLSQdot(1));
    N =exp(lnpLSQdot(2));
    S =exp(lnpLSQdot(5));
    xLSQ=x0Spot+lnpLSQdot(3);
    yLSQ=y0Spot+lnpLSQdot(4);
    tFit=toc;
    
    dotCoordLSQ(r,1:2)=[xLSQ yLSQ];
    LSQbNS_flag(r,:)  =[bg N S resNorm/ROIwidth^2 exitFlag tFit];
    
    %disp([ int2str([r size(dotCoord0,1)]) ' ' num2str(tFit) ' s'])
end