% [totalDotCoord,allLSQcoord,allLSQpar] = LSQrefineDots_parfor(...
%                totalDotCoord,sourceFolder, fluorChannelName, fluoList,... 
%                                 pathSegImages, segImagesList, parameters)
%
% Filter out unphysical dots from the list in totalDotCoord using 1) cell
% segmentation and 2) the fit parameters of an LSQ fit to each dot with a
% symmetric Gaussian. This version parallellizes the fluo movies using a
% parfor loop.
%
% Input:
% totalDotCoord{k} : list of refined dot positions, [x y frame ...],
%                    column 4+ are just passed on as is
% standard pipeline workflow internal parameters: 
% sourceFolder,fluorChannelName,fluoList, pathSegImages, segImagesList, parameters
% 
% parameters used:
% LSQrefineDots_cellInclusionMargin; % extra margin (in approx bf pixels) for cell inclusion
% LSQrefineDots_ROIwidth   % ROI side length for LSQ fits
% LSQrefineDots_SminPx     % minimum lsq psf width, in fluo pixel units
% LSQrefineDots_SmaxPx     % maximum lsq psf width, in fluo pixel units
% LSQrefineDots_Nmin       % minimum spot amplitude
% LSQrefineDots_drMax      % maximum difference between centroid and lsq positions, in fluo pixel units
% EMgain
% EMCCD_offsetName % name of EMCCD offset file, in sourceFolder/
%
% Output:
% totalDotCoord{k} : list of refined dot positions, [x y frame ...], for
%                    those dots that made it through all the filtration
%                    steps.
% allLSQcoord{k}   : list of putative dot positions, [x y frame ...], where
%                    an LSQ fit was performed, but before the filtering
%                    based on LSQ fit parameters was done. This
%                    intermediate step is useful for diagnosing the LSQ
%                    filter parameters.
% allLSQpar{k}     : Misc fit parameters from all LSQ fits, each line with
%                    [bg N S rn exitFlag tFit dr kept], where
%                       b  = background parameter
%                       N  = gaussian PSF amplitude (approximates #photons)
%                       S  = gaussian PSF std
%                       rn = residual norm per pixel, from lsqnonlin)
%                       exitFlag : -3 means LSQ fit failed with an error,
%                       otherwise as returned by lsqnonlin.
%                       dr = euclidian distance btw LSQ fit and initial
%                       guess (e.g., position in totalDotCoord).
%                       kept = 1 if the point is included in output
%                       totalDotCoord, 0 otherwise.
%
% Martin Lindén, bmelinden@gmail.com, 2015-11-05
function [totalDotCoord,allLSQcoord,allLSQpar] = LSQrefineDots_parfor(totalDotCoord,sourceFolder, fluorChannelName, fluoList, pathSegImages, segImagesList, parameters)

% extract useful parameters
cellInclusionMargin=parameters.LSQrefineDots_cellInclusionMargin; % extra margin (in approx bf pixels) for cell inclusion
ROIwidth=parameters.LSQrefineDots_ROIwidth; % ROI side length for LSQ fits
SminPx= parameters.LSQrefineDots_SminPx;    % minimum lsq psf width, in fluo pixel units
SmaxPx= parameters.LSQrefineDots_SmaxPx;    % maximum lsq psf width, in fluo pixel units
drMax = parameters.LSQrefineDots_drMax;     % maximum difference between centroid and lsq positions, in fluo pixel units
Nmin  = parameters.LSQrefineDots_Nmin;      % minimum spot amplitude (~photons, if offset&EMgain are correct)
EMgain= parameters.EMgain;
EMCCD_offsetName=parameters.EMCCD_offsetName;

allLSQcoord=cell(size(totalDotCoord));
allLSQpar  =cell(size(totalDotCoord));


parfor fluoMovieIdx = 1:length(fluoList)
    t0=tic;
    %% load the necessary images etc
    currentSegmentedImageFile=fullfile(pathSegImages,segImagesList(fluoMovieIdx).name);
    currentSegmentedImage = imread(currentSegmentedImageFile);

    currentFluoMovieFile=fullfile(sourceFolder,fluorChannelName,fluoList(fluoMovieIdx).name);
    % Calc. new variables
    FlInfo = imfinfo(currentFluoMovieFile);
    FlColums = FlInfo(1).Width;
    FlRows = FlInfo(1).Height;
    numberOfFrames= numel(FlInfo);
    
    % compute scaling factors
    fluo2seg_col=size(currentSegmentedImage,2)/FlColums; % x
    fluo2seg_row=size(currentSegmentedImage,1)/FlRows; % y
    
    currentDotCoord = totalDotCoord{fluoMovieIdx}; % coordinates in units of fluo pixels!
    
    %% pruning step 1: remove dots in the outermost pixels, in fluo
    % coordinates. This is because the wavelet spot detection tends to
    % place a lot of false positives a long the edges
    internalPoints=ones(FlRows,FlColums);
    internalPoints(1,:)=0;
    internalPoints(end:end,:)=0;
    internalPoints(:,1)=0;
    internalPoints(:,end)=0;
    currentDotCoord = BuildTrajectories.findPointsInCellNoId(currentDotCoord,1:numberOfFrames,internalPoints,1);
    
    %% pruning step 2: remove dots that are far away from cells, which
    % happens in units of TransformedRCPhase
    currentDotCoord(:,1) = fluo2seg_row*currentDotCoord(:,1);
    currentDotCoord(:,2) = fluo2seg_col*currentDotCoord(:,2);
    
    marginStrel=strel('disk',cellInclusionMargin);
    allCellsWithMargin=imdilate(currentSegmentedImage>0,marginStrel);
    
    % Remove the dots that are not close enugh to any active cell
    currentDotCoord = BuildTrajectories.findPointsInCellNoId(currentDotCoord,1:numberOfFrames,allCellsWithMargin,1);
    
    % scale back to fluo space for further filtering
    currentDotCoord(:,1) = fluo2seg_row\currentDotCoord(:,1);
    currentDotCoord(:,2) = fluo2seg_col\currentDotCoord(:,2);    
    %% pruning step 3, written to only read one frame frame at a time
    % load the necessary images etc
    calibrationOffsetFile=fullfile(sourceFolder,EMCCD_offsetName);
    fluoOffset=double(imread(calibrationOffsetFile)); % the average pixel offset, converted do double    

    currentDotCoordLSQ=currentDotCoord;
    currentDotParLSQ=zeros(size(currentDotCoordLSQ,1),6);
    for frame=1:numberOfFrames
        ind=find(currentDotCoord(:,3)==frame);
        if(~isempty(ind));
            currentFluoFrame=double(imread(currentFluoMovieFile,frame,'info',FlInfo));
            frameCoord=currentDotCoord(ind,:);
            [frameCoordLSQ,frameParLSQ]=LSQrefineDots.LSQfitDotsSingleFrame(frameCoord,currentFluoFrame,fluoOffset,EMgain,ROIwidth);
            currentDotCoordLSQ(ind,:)=frameCoordLSQ;
            currentDotParLSQ(ind,:)  =frameParLSQ;
            
        end
    end
    % filter out LSQ positions based on LSQ convergence, spot size, and
    % fitted position (not) straying too far from the ROI center
    S=currentDotParLSQ(:,3);
    N=currentDotParLSQ(:,2);
    flag=currentDotParLSQ(:,5);
    
    dx=currentDotCoordLSQ(:,1)-currentDotCoord(:,1);
    dy=currentDotCoordLSQ(:,2)-currentDotCoord(:,2);
    dr=sqrt(dx.^2+dy.^2);
    currentDotParLSQ(:,7)=dr;
    
    ind=find((dr<drMax).*(S>SminPx).*(S<SmaxPx).*(flag>=0).*(N>Nmin));
    currentDotParLSQ(:,8)=0;
    currentDotParLSQ(ind,8)=1;
    
    
    tFit=mean(currentDotParLSQ(:,6));
    tTotal=toc(t0);
    LSQfraction=length(ind)/size(currentDotCoord,1);    
    disp('--------------------')
    disp(['LSQ fitted ' int2str(length(ind)) ' dots from ' currentFluoMovieFile ' .'])
    disp(['Average LSQ fit time    : ' num2str(tFit) ' s / spot'])
    disp(['Total LSQ analysis time : ' num2str(tTotal/length(ind)) ' s/spot'])
    disp(['LSQ filter kept ' num2str(LSQfraction*100) ' % of fitted dots.'])
    disp('--------------------')

    % put the final answer back in the cell vector
    allLSQcoord{fluoMovieIdx}=currentDotCoordLSQ;
    allLSQpar{fluoMovieIdx}=currentDotParLSQ;
    totalDotCoord{fluoMovieIdx}=currentDotCoordLSQ(ind,:);
end
