% [totalDotCoord,allMAPcoord,allMAPpar,allMAPcov]=...
%   MAPsgS0refineDots_parfor(totalDotCoord,posName,fluoChanName,expInfoObj)
%
% Filter out unphysical dots from the list in totalDotCoord using 1) cell
% segmentation and 2) the fit parameters of an MAP fit to each dot with a
% symmetric Gaussian, using the symmetric model and prior described in [1].
% The model is a symmetric Gaussian + constant background, parameterized by
% the PSF class PSF.SymGaussS0_logNormBN_expS0. This version parallellizes
% the fluo movies using a parfor loop. 
%
% Input:
% totalDotCoord{k} : list of refined dot positions, [x y frame cell id]
%                    
% standard pipeline workflow internal parameters: 
% totalDotCoord,posName,fluoChanName,expInfoObj
% 
% parameters used:
% LSQrefineDots_cellInclusionMargin; % extra margin (in approx bf pixels) for cell inclusion
% LSQrefineDots_ROIwidth   % ROI side length for MLE fits
% LSQrefineDots_SminPx     % minimum MLE psf width, in fluo pixel units
% LSQrefineDots_SmaxPx     % maximum MLE psf width, in fluo pixel units
% LSQrefineDots_Nmin       % minimum spot amplitude
% LSQrefineDots_drMax      % maximum difference between centroid and MLE
%                            positions, in fluo pixel units 
% EMgain                   % EMCCD gain
% EMCCD_noiseStd           % standard deviation of EMCCD readout noise
% EMCCD_offsetName         % name of EMCCD offset file, in sourceFolder/
% scale                    % fluorescence pixel size [nm]
% pxQuadPoints             % number of quadrature points per pixel
% MAPlambda                % imaging wavelength [nm, or same as pxSize]
% NA                       % Numerical Aperture
% MAPprior                 % [lnB0 lnBstd lnN0 lnNstd S0] : prior parameter
%                            vector for PSF.SymGaussS0_logNormBN_expS0.
%                            lnB0 and lnN0 are also used as initial guess.
%                            For lndS, the initial guess is 1e-2 (i.e.,
%                            spot width close to the minimum).
%
% Output:
% totalDotCoord{k} : list of refined dot positions, indices, and
%                    covariances,  for those dots that made it through all
%                    the filtration steps:
%                    [x y frame cell id varX varY covXY], where frame, cell
%                    are column 3,4 of totalDotCoord{k}. Not present: cond,
%                    the condition number of the matrix inversion used to
%                    compute the XY covariances.
%
% allMLEcoord{k}   : list of putative dot positions, [x y frame ...], where
%                    an MLE fit was performed, but before the filtering
%                    based on MLE fit parameters was done. This
%                    intermediate step is useful for diagnosing the MLE
%                    filter parameters.
% allMLEpar{k}     : Misc fit parameters from all MLE fits, each line with
%                    [bg N S lnL exitFlag tFit dr kept], where
%                       b  = background parameter
%                       N  = gaussian PSF amplitude (approximates #photons)
%                       S  = gaussian PSF std
%                       lnL = log likelihood of the fit
%                       exitFlag : -3 means MLE fit failed with an error,
%                       otherwise as returned by fminunc.
%                       dr = euclidian distance btw MLE fit and initial
%                       guess (e.g., position in totalDotCoord).
%                       kept = 1 if the point is included in output
%                       totalDotCoord, 0 otherwise.
% allMLEcov{k}      : position covariances and inversion condition numbers
%                     for all dots, [varX varY covXY cond]
% Martin Linden, bmelinden@gmail.com, 2017-03-27

% Last modified - SZ, 2017-12-18. 

function [refinedDots,allMAPcoord,allMAPpar,allMAPcov]=MAPsgS0refineDots(expInfoObj, posName,fluoChanName, totalDotCoord)

import LSQrefineDots.MAPsgS0refineDotsSingleMovie
import EMCCDfit.getLogL_EMCCD_LookupTable
import PSF.SymGaussS0_logNormBN_expS0

sourceDir = expInfoObj.getPathName('source');
sourceDirPosFluo = fullfile(sourceDir,posName,fluoChanName);
fluoList = dir(fullfile(sourceDirPosFluo,'*.tif*'));

% extract useful parameters
parameters = expInfoObj.getParameters();
fluoStackRange = parameters.rangeFluoStack;
if isempty(fluoStackRange)
    aMovieFile=fullfile(sourceDirPosFluo,fluoList(1).name);
    flInfo = imfinfo(aMovieFile);
    fluoStackRange = 1:numel(flInfo);
end

%% construct EMCCD likelihood lookup table
[logLobj, fluoOffset] = getLogL_EMCCD_LookupTable(expInfoObj,fluoChanName);
%% construct PSF class
pxSize= parameters.fluoPxSize; % fluorescence pixel size [nm]
lambda= parameters.MAPlambda; % imaging wavelength [nm, or same as pxSize]
lambda_px=lambda/pxSize; % wavelength, in fluo pixel units
NA    = parameters.NA;
priorParameters=parameters.MAPprior;
lnB0 = priorParameters(1); % prior means used as initial guess for lnB,lnN
lnN0 = priorParameters(3);
PSFsymGauss= SymGaussS0_logNormBN_expS0('initialGuess',[0 0 lnB0 lnN0 log(1e-2)],...
    'lambda',lambda_px,'NA',NA,'priorParameters',priorParameters);
%% localization loop over individual movies
refinedDots=cell(size(totalDotCoord));
allMAPcoord=cell(size(totalDotCoord));
allMAPpar  =cell(size(totalDotCoord));
allMAPcov  =cell(size(totalDotCoord));

for fluoIdx=1:length(fluoList)
    fluoMovieFile=fullfile(sourceDirPosFluo,fluoList(fluoIdx).name);
    movieDots = totalDotCoord{fluoIdx};
    [currRefinedDots, currDotsMAP, currDotParMAP, currDotCovMAP] = ...
        MAPsgS0refineDotsSingleMovie(fluoMovieFile,fluoStackRange,movieDots,...
                                 fluoOffset,logLobj,PSFsymGauss,parameters,0,0);
    refinedDots{fluoIdx} = currRefinedDots;
    allMAPcoord{fluoIdx}=currDotsMAP;
    allMAPcov{fluoIdx}=currDotCovMAP;
    allMAPpar{fluoIdx}=currDotParMAP;
end