% [dotCoordMLE,MLEbNS_flag,dotCovMLE]=...
%    MLEfitDotsSingleFrame(dotCoord0,fluoFrame,fluoOffset,logLobj,ROIwidth)
%
% attempt MLE fit of the suggested dots in dotCoord0 to a single
% fluorescent image fluoFrame, with mean offset fluoOffset.
%
% dotCoord0=[x y ...]. Suggested starting points for MLE fits. Columns > 2
%                      are just passed on unchanged to dotCoordMLE.
%
% fluoFrame : image to fit spots to.
% fluoOffset: camera offset image
% logLobj   : logL_EMCCD_lookup object (this is how EMgain and readoutNoise
%             are included in the fit).
% ROIwidth  : side length (pixels) of the ROI square to use for fit
%
% dotCoordMLE : same as dotCoord0, but with first two columns updated to
%               x,y positions from the MLE fit.
% MLEbNS_flag=[bg N S lnL exitFlag tFit];, where
%       b  = background parameter
%       N  = gaussian PSF amplitude (approximates #photons)
%       S  = gaussian PSF std
%       lnL = log Likelihood for the fit
%       exitFlag : -5 means MLE fit failed with an error, otherwise as
%       returned by fminunc.
% dotCovMLE : position covariances, dotCovMLE(t,:)= [varX covXY varY cond],
%             from a Laplace approximation. cond is the Hessian inversion
%             condition number (see help cond).
%
% Martin Lindén, bmelinden@gmail.com, 2015-12-01
function [dotCoordMLE,MLEbNS_flag,dotCovMLE]=...
    MLEfitDotsSingleFrame(dotCoord0,fluoFrame,fluoOffset,logLobj,ROIwidth)

dotCoordMLE=dotCoord0;
dotCovMLE=zeros(size(dotCoordMLE,1),4);
MLEbNS_flag  =zeros(size(dotCoord0,1),6);

MLEopt = optimoptions('fminunc','TolX',1e-6,'MaxIter',500,...
    'TolFun',1e-6,'MaxFunEvals',10000,'Display','notify','algorithm','quasi-newton' );

for r=1:size(dotCoord0,1)
    tic
    % construct ROI for MLE localization
    x0   =dotCoord0(r,1);
    y0   =dotCoord0(r,2);
    
    [riSpot,ciSpot,x0Spot,y0Spot]=LSQrefineDots.ROItransform(x0,y0,ROIwidth,ROIwidth,size(fluoFrame));
    spotROI=double(fluoFrame(riSpot,ciSpot)); % movie raw data
    bgROI=double(fluoOffset(riSpot,ciSpot)); % offset background to be subtracted
    mleData=(spotROI-bgROI);
    
    % MLE fit
    mleObj=LSQrefineDots.logL_EMCCDpsf_symGauss(logLobj,mleData,3);
    mleWprior=@(p)(mleObj.mlogL_psf(p)+0.5*(p(3)/12)^2); % mild bg prior to ensure an invertible hessian
    lnpInit=[x0-x0Spot y0-y0Spot log(10) log(300) log(2)];
    tic
    try
        %[lnpMLEdot,resNorm,~,exitFlag]=lsqnonlin(PSFtarget,lnpMLEdot,[],[],LSQopt);
        [lnpMLEdot,logLMLE,exitFlag,~,~,hessianMLE] = fminunc(mleWprior,lnpInit,MLEopt);
        covMLE=inv(hessianMLE);
        hessRcond=rcond(hessianMLE);
        if( exitFlag<=0 || ~isfinite(hessRcond) || hessRcond<10*eps )
            disp('--------------------------------------------------------')
            warning('MLE convergence problem or ill-conditioned Hessian:')
            disp(['fminunc exitFlag = ' int2str(exitFlag)])
            disp('hessianMLE : ')
            disp(num2str(hessianMLE,4))
            disp(['rcond = ' num2str(hessRcond)])
            %disp('mleData : ')
            %disp(num2str(mleData,5))
            disp('fit parameters :')
            disp(num2str([lnpMLEdot(1:2) exp(lnpMLEdot(3:5))],4))
            %%%warnFileName=['MLEfitDots_badfit_' int2str( ceil(1e8*rand)) '.mat'];
            %%%save(warnFileName)
            %%%disp(['saved local workspace to ' warnFileName])
            disp('--------------------------------------------------------')
        end
    catch me
        % if something goes wrong already here, we discard
        % the point
        tFit=toc;
        MLEbNS_flag(r,4:6)= [-inf -5 tFit];
        continue
    end
    xMLE=x0Spot+lnpMLEdot(1);
    yMLE=y0Spot+lnpMLEdot(2);
    
    bg=exp(lnpMLEdot(3));
    N =exp(lnpMLEdot(4));
    S =exp(lnpMLEdot(5));
    tFit=toc;
    
    dotCoordMLE(r,1:2)=[xMLE yMLE];
    MLEbNS_flag(r,:)  =[bg N S -logLMLE exitFlag tFit];
    dotCovMLE(r,:)=[covMLE(1,1) covMLE(1,2) covMLE(2,2) hessRcond];
    
    if( false && ( covMLE(1,1)<0 || covMLE(2,2)<0 ) )
        % a visual debug block: it seems most fits that encounter this
        % warning are false positives anyway, so the block is not
        % activated.
        disp('--------------------------------------------------------')
        disp(['fminunc exitFlag = ' int2str(exitFlag)])
        disp('hessianMLE : ')
        disp(num2str(hessianMLE,4))
        disp(['rcond = ' num2str(hessRcond)])
        disp('mleData : ')
        disp(num2str(mleData,5))
        disp('fit parameters :')
        disp(num2str([lnpMLEdot(1:2) bg N S],4))
        
        figure(13)
        clf
        subplot(2,1,1)
        hold on
        imagesc(fluoFrame)
        plot(xMLE,yMLE,'ro')
        axis equal, axis tight
        cr=get(gca,'clim');
        
        subplot(2,2,3)
        hold on
        imagesc(mleData)
        plot(lnpMLEdot(1),lnpMLEdot(2),'ro')
        axis equal, axis tight
        set(gca,'clim',cr);
        
        subplot(2,2,4)
        hold on
        Emodel=mleObj.psfModel(lnpMLEdot);
        imagesc(Emodel*logLobj.EMgain)
        plot(lnpMLEdot(1),lnpMLEdot(2),'ro')
        axis equal, axis tight
        set(gca,'clim',cr);
        
        colormap gray
        
        warning('Negative localization error variance encountered')
        disp('--------------------------------------------------------')
    end
    %disp([ int2str([r size(dotCoord0,1)]) ' ' num2str(tFit) ' s'])
end