% [totalDotCoord,allMLEcoord,allMLEpar,allMLEcov]=...
%        MLErefineDots_parfor(totalDotCoord,sourceFolder,fluorChannelName,...
%               fluoList, pathSegImages, segImagesList, parameters)
%
% Filter out unphysical dots from the list in totalDotCoord using 1) cell
% segmentation and 2) the fit parameters of an MLE fit to each dot with a
% symmetric Gaussian. This version parallellizes the fluo movies using a
% parfor loop.
%
% Input:
% totalDotCoord{k} : list of refined dot positions, [x y frame cell],
%                    column 5+ are discarded. If only three columns are
%                    given, then cell=0;
% standard pipeline workflow internal parameters: 
% sourceFolder,fluorChannelName,fluoList, pathSegImages, segImagesList, parameters
% 
% parameters used:
% LSQrefineDots_cellInclusionMargin; % extra margin (in approx bf pixels) for cell inclusion
% LSQrefineDots_ROIwidth   % ROI side length for MLE fits
% LSQrefineDots_SminPx     % minimum MLE psf width, in fluo pixel units
% LSQrefineDots_SmaxPx     % maximum MLE psf width, in fluo pixel units
% LSQrefineDots_Nmin       % minimum spot amplitude
% LSQrefineDots_drMax      % maximum difference between centroid and MLE positions, in fluo pixel units
% EMgain
% EMCCD_offsetName % name of EMCCD offset file, in sourceFolder/
%
% Output:
% totalDotCoord{k} : list of refined dot positions, indices, and
%                    covariances,  for those dots that made it through all
%                    the filtration steps:
%                    [x y frame cell varX varY covXY cond], where frame, cell
%                    are column 3,4 of totalDotCoord{k}, and cond is the
%                    condition number of the matrix inversion used to
%                    compute the XY covariances.
%
% allMLEcoord{k}   : list of putative dot positions, [x y frame ...], where
%                    an MLE fit was performed, but before the filtering
%                    based on MLE fit parameters was done. This
%                    intermediate step is useful for diagnosing the MLE
%                    filter parameters.
% allMLEpar{k}     : Misc fit parameters from all MLE fits, each line with
%                    [bg N S lnL exitFlag tFit dr kept], where
%                       b  = background parameter
%                       N  = gaussian PSF amplitude (approximates #photons)
%                       S  = gaussian PSF std
%                       lnL = log likelihood of the fit
%                       exitFlag : -3 means MLE fit failed with an error,
%                       otherwise as returned by fminunc.
%                       dr = euclidian distance btw MLE fit and initial
%                       guess (e.g., position in totalDotCoord).
%                       kept = 1 if the point is included in output
%                       totalDotCoord, 0 otherwise.
% allMLEcov{k}      : position covariances and inversion condition numbers
%                     for all dots, [varX varY covXY cond]
% Martin Lindén, bmelinden@gmail.com, 2015-12-03
function [totalDotCoord,allMLEcoord,allMLEpar,allMLEcov]=MLErefineDots_parfor(totalDotCoord,sourceFolder,fluorChannelName,fluoList, pathSegImages, segImagesList, parameters)

% extract useful parameters
cellInclusionMargin=parameters.LSQrefineDots_cellInclusionMargin; % extra margin (in approx bf pixels) for cell inclusion
ROIwidth=parameters.LSQrefineDots_ROIwidth; % ROI side length for MLE fits
SminPx= parameters.LSQrefineDots_SminPx;    % minimum lsq psf width, in fluo pixel units
SmaxPx= parameters.LSQrefineDots_SmaxPx;    % maximum lsq psf width, in fluo pixel units
drMax = parameters.LSQrefineDots_drMax;     % maximum difference between centroid and lsq positions, in fluo pixel units
Nmin  = parameters.LSQrefineDots_Nmin;      % minimum spot amplitude (~photons, if offset&EMgain are correct)
EMgain= parameters.EMgain;
EMCCD_noiseStd=parameters.EMCCD_noiseStd;
EMCCD_offsetName=parameters.EMCCD_offsetName;

allMLEcoord=cell(size(totalDotCoord));
allMLEpar  =cell(size(totalDotCoord));
allMLEcov  =cell(size(totalDotCoord));
%% construct EMCCD likelihood lookup table
% first find the intensity range in the data
disp('MLErefineDots_parfor constructing logL lookup table')
calibrationOffsetFile=fullfile(sourceFolder,EMCCD_offsetName);
fluoOffset=double(imread(calibrationOffsetFile)); % the average pixel offset, converted do double

offsetMin=min(fluoOffset(:));
cMax=-inf;
for fluoMovieIdx=1:numel(fluoList)
    currentFluoMovieFile=fullfile(sourceFolder,fluorChannelName,fluoList(fluoMovieIdx).name);
    % movie size
    FlInfo = imfinfo(currentFluoMovieFile);
    FlColums = FlInfo(1).Width;
    FlRows = FlInfo(1).Height;
    numberOfFrames= numel(FlInfo);

    MV=imread(currentFluoMovieFile,1);              % first frame is usually brightest
    cMax=double(max(cMax,max(MV(:))-offsetMin));

    MV=imread(currentFluoMovieFile,numberOfFrames); % last frame is usually dimmest
    cMax=double(max(cMax,max(MV(:))-offsetMin));
end
logLobj=LSQrefineDots.logL_EMCCD_lookup(EMgain,EMCCD_noiseStd,cMax,cMax/EMgain*3);
disp('MLErefineDots_parfor starting MLE fits')
%%
parfor fluoMovieIdx = 1:length(fluoList)
%%%for fluoMovieIdx = 1:length(fluoList) % uncomment for debugging
    t0=tic;
    %% load the necessary images etc
    currentSegmentedImageFile=fullfile(pathSegImages,segImagesList(fluoMovieIdx).name);
    currentSegmentedImage = imread(currentSegmentedImageFile);

    currentFluoMovieFile=fullfile(sourceFolder,fluorChannelName,fluoList(fluoMovieIdx).name);
    % Calc. new variables
    FlInfo = imfinfo(currentFluoMovieFile);
    FlColums = FlInfo(1).Width;
    FlRows = FlInfo(1).Height;
    numberOfFrames= numel(FlInfo);
    
    % compute scaling factors
    fluo2seg_col=size(currentSegmentedImage,2)/FlColums; % x
    fluo2seg_row=size(currentSegmentedImage,1)/FlRows; % y
    
    currentDotCoord = totalDotCoord{fluoMovieIdx}; % coordinates in units of fluo pixels!
    
    %% pruning step 1: remove dots in the outermost pixels, in fluo
    % coordinates. This is because the wavelet spot detection tends to
    % place a lot of false positives a long the edges
    internalPoints=ones(FlRows,FlColums);
    internalPoints(1,:)=0;
    internalPoints(end:end,:)=0;
    internalPoints(:,1)=0;
    internalPoints(:,end)=0;
    currentDotCoord  = BuildTrajectories.findPointsInCellNoId(currentDotCoord,1:numberOfFrames,internalPoints,1);
    
    %% pruning step 2: remove dots that are far away from cells, which
    % happens in units of TransformedRCPhase
    if(~isempty(currentDotCoord))
        currentDotCoord(:,1) = fluo2seg_row*currentDotCoord(:,1);
        currentDotCoord(:,2) = fluo2seg_col*currentDotCoord(:,2);
        
        marginStrel=strel('disk',cellInclusionMargin);
        allCellsWithMargin=imdilate(currentSegmentedImage>0,marginStrel);
        
        % Remove the dots that are not close enugh to any active cell
        currentDotCoord = BuildTrajectories.findPointsInCellNoId(currentDotCoord,1:numberOfFrames,allCellsWithMargin,1);
        
        % scale back to fluo space for further filtering
        if(~isempty(currentDotCoord))
            currentDotCoord(:,1) = fluo2seg_row\currentDotCoord(:,1);
            currentDotCoord(:,2) = fluo2seg_col\currentDotCoord(:,2);
        end
    end
    %% pruning step 3, written to only read one frame frame at a time
    % load the necessary images etc
    
    if(~isempty(currentDotCoord))
        currentDotCoordMLE=currentDotCoord;
        if(size(currentDotCoordMLE,2)<4)
            currentDotCoordMLE(:,4)=0; % default cell=0
        end
        currentDotParMLE=zeros(size(currentDotCoordMLE,1),6);
        currentDotCovMLE=zeros(size(currentDotCoordMLE,1),4);
        for frame=1:numberOfFrames
            ind=find(currentDotCoord(:,3)==frame);
            if(~isempty(ind));
                currentFluoFrame=double(imread(currentFluoMovieFile,frame,'info',FlInfo));
                frameCoord=currentDotCoord(ind,:);
                [frameCoordMLE,frameParMLE,dotCovMLE]=LSQrefineDots.MLEfitDotsSingleFrame(frameCoord,currentFluoFrame,fluoOffset,logLobj,ROIwidth);
                currentDotCoordMLE(ind,:)= frameCoordMLE;
                currentDotParMLE(ind,:)  = frameParMLE;
                currentDotCovMLE(ind,:)  = dotCovMLE;
            end
        end
        % filter out LSQ positions based on LSQ convergence, spot size, and
        % fitted position (not) straying too far from the ROI center
        S=currentDotParMLE(:,3);
        N=currentDotParMLE(:,2);
        flag=currentDotParMLE(:,5);
    
        dx=currentDotCoordMLE(:,1)-currentDotCoord(:,1);
        dy=currentDotCoordMLE(:,2)-currentDotCoord(:,2);
        dr=sqrt(dx.^2+dy.^2);
        currentDotParMLE(:,7)=dr;
    
        xVar=currentDotCovMLE(:,1);
        yVar=currentDotCovMLE(:,3);
        covDet=currentDotCovMLE(:,1).*currentDotCovMLE(:,3)-currentDotCovMLE(:,2).^2;
        covTrace=currentDotCovMLE(:,1)+currentDotCovMLE(:,2);   
        % covDet>0 and covTrace>0 means both eigenvalues are positive, i.e.,
        % covariance matrix is positive definite and thus variances are
        % positive in all coordinate systems. 
        
        ind=find((dr<drMax).*(S>SminPx).*(S<SmaxPx).*(flag>=0).*(N>Nmin)...
        .*(xVar>0).*(yVar>0).*(covDet>0).*(covTrace>0));
    
        currentDotParMLE(:,8)=0;
        currentDotParMLE(ind,8)=1;
        
        tFit=mean(currentDotParMLE(:,6));
        tTotal=toc(t0);
        MLEfraction=length(ind)/size(currentDotCoord,1);
        disp('--------------------')
        disp(['MLE fitted ' int2str(length(ind)) ' dots from ' currentFluoMovieFile ' .'])
        disp(['Average MLE fit time    : ' num2str(tFit,4) ' s/spot'])
        disp(['Total MLE analysis time : ' num2str(tTotal/length(ind),4) ' s/spot'])
        disp(['MLE filter kept ' num2str(MLEfraction*100) ' % of fitted dots.'])
        disp('--------------------')
        
        % put the final answer back in the cell vector
        allMLEcoord{fluoMovieIdx}=currentDotCoordMLE;
        allMLEcov{fluoMovieIdx}=currentDotCovMLE;
        allMLEpar{fluoMovieIdx}=currentDotParMLE;
        
        totalDotCoord{fluoMovieIdx}=[currentDotCoordMLE(ind,1:4) currentDotCovMLE(ind,1:4)];
    else % just make the resulting cell vectro entries empty
        % put the final answer back in the cell vector
        allMLEcoord{fluoMovieIdx}=[];
        allMLEcov{fluoMovieIdx}=[];
        allMLEpar{fluoMovieIdx}=[];
        
        totalDotCoord{fluoMovieIdx}=[];
    end
end
