function [totalDotCoord,numDots,maxNumLater,frameStart]=dotNumCurves(totalDotCoord,posIdx,fluoChanName,expInfoObj,mFrames)
%[totalDotCoord,numDots,maxNumLater]=dotNumCurves(totalDotCoord,sourceFolder,...
% fluorChannelName, fluoList, pathSegImages, segImagesList,maxDots,segDilate)
%
% 1) Compute the number of dots per frame in each cells, and filter out
% frames with future dot number above some threshold 
% 2) Add cell number entries to column four of each remaining dots

% input:
% totalDotCoord     : dot coordinates. For dot m in movie k,
%                     totalDotCoord{k}=[x y frame cell ...]; Fluo pixel
%                     units.
% sourceFolder, fluoChannelName, fluoList, pathSegImages, segImagesList: 
% As in the usual image analsyis pipeline.
% maxDots           : maximum number of future dots, a dot in frame t is
%                     discarded if the number of dots in the cell exceeds
%                     this number at frame t or later. 
%                     (Default -1 : no dots discarded).
% segDilate         : cell dilation margin (segmentation pixel units). When
%                     determining if a dot is in a cell or not, the cell
%                     segmentation image is first dilated by a disk of this
%                     radius. (Default 0, no dilation). NOTE:
%                     segDilateion>0 can lead to some dots being present
%                     multiple times with different cell indices.
% output: 
% totalDotCoord     : Filtered dot coordinates, with cell id in the fourth
%                     column, 5th and later columns left untouched.
% numSpots          : numSpots{m}{c}(t) is the number of spots in cell c,
%                     movie m, frame t.
% maxNumLater       : maxNumLater{m}{k}(t) is the maximum number of spots
%                     in cell k, movie k, at frame t or later.
% frameStart{m}(k)  : the first frame in movie m, cell k, for which
%                     dot positions where kept due to the maxDots
%                     parameter. Inf means that no frames with few enough
%                     dots were found.
%
% ML 2015-11-24
% ML 2016-02-10 : gracefully handle case of no dots in some movies
% SZ 2017-09-25 : 

sourceDir = expInfoObj.sourceFolderPath;
outputDir = expInfoObj.outputFolderPath;
posName = expInfoObj.getPositionList(posIdx);
imRangeFluo = expInfoObj.getImRangeFluo(posIdx);
phaseIdxs = expInfoObj.getPhaseIndices(posIdx);
fluoIdxs  = expInfoObj.getFluoIndices(posIdx);

parameters = expInfoObj.parameters;
fluoStackRange = parameters.rangeFluoStack;
maxDots = parameters.LSQrefineDots_trjMaxDots;
segDilate = parameters.LSQrefineDots_cellInclusionMargin;

sourceDirPos = fullfile(sourceDir,posName);
sourceDirPosFluo = fullfile(sourceDirPos,fluoChanName);
fluoList = dir(fullfile(sourceDirPosFluo,'*.tif*'));
pathSegImages = fullfile(outputDir,posName,'SegmentedPhase');
segImagesList = dir(fullfile(pathSegImages,'*.tif*'));

if(~exist('segDilate','var') || isempty(segDilate) )
    segDilate=0;
end
se=strel('disk',segDilate);

if ~isempty(imRangeFluo)
    fluoList = fluoList(imRangeFluo);
end
numDots=cell(1,length(fluoList));
maxNumLater=cell(1,length(fluoList));
frameStart=cell(1,length(fluoList));
% parfor m = 1:length(fluoList)
for fluoIdx = 1:length(fluoList)
    tic
    newPhaseCoords=[];
    phaseIdx = find(phaseIdxs == fluoIdxs(fluoIdx));
    currSegmentedImage = imread(fullfile(pathSegImages,...
        segImagesList(phaseIdx).name));    
    
    blobs = mFrames.getBlobsInFrame(phaseIdx);
        
    % determine cell identity of all spots, and discard spots outside cells
    if(~isempty(totalDotCoord{fluoIdx}))
        fluoCoords=totalDotCoord{fluoIdx};
        
        % Convert to phase coordinates
        phaseCoords = expInfoObj.convertCoordToPhase(...
            posName, fluoChanName, fluoIdx, fluoCoords);
        
        for blobIdx=1:length(blobs)
            c = blobs(blobIdx);
            % first determine cell identity of all spots, and discard spots outside cells
            segmentedImage=double(imdilate(currSegmentedImage==c,se))*c;            
            [~, dotIdxs] = BuildTrajectories.findPointsInCell(phaseCoords(:,1:3),fluoStackRange,segmentedImage,c,[]);
            if any(dotIdxs)
                dotsInCell = phaseCoords(dotIdxs,:);            
                dotsInCell(:,4) = c;
                newPhaseCoords=[newPhaseCoords;dotsInCell];
            end
        end
        if(~isempty(newPhaseCoords))
            newFluoCoords = expInfoObj.convertCoordToFluo(...
                posName, fluoChanName, fluoIdx, newPhaseCoords);
        end
        totalDotCoord{fluoIdx}=newFluoCoords;
    end
    
    for blobIdx=1:length(blobs)
        c = blobs(blobIdx);
        if(~isempty(totalDotCoord{fluoIdx}))
            coordsInThisCell=totalDotCoord{fluoIdx}(totalDotCoord{fluoIdx}(:,4)==c,3);
        else
            coordsInThisCell=[];
        end
            
        [numDots{fluoIdx}{c}]=hist(coordsInThisCell,fluoStackRange);
        
        maxNumLater{fluoIdx}{c}=zeros(size(fluoStackRange));
        maxNum=0;
        while(true)
            maxNum=maxNum+1;
            % last index with maxNum spots in it
            ind=find(numDots{fluoIdx}{c}==maxNum,1,'last');
            if ~isempty(ind)
                maxNumLater{fluoIdx}{c}(1:ind)=maxNum;
            else
                break
            end
        end
    end
    % filter out dots from crowded frames
    newCoords=[];
    frameStart{fluoIdx}=ones(1,max(blobs));
    for blobIdx=1:length(blobs)
        c = blobs(blobIdx);
        newFrameStart=fluoStackRange(find(maxNumLater{fluoIdx}{c}<=maxDots,1));
        if(isempty(newFrameStart))
            % in this case, no frames with few enough spots were found, and
            % hence no coordinates should be kept. We mark this by setting
            frameStart{fluoIdx}(c)=inf;     
        else
            frameStart{fluoIdx}(c)=newFrameStart;
            if(~isempty(totalDotCoord{fluoIdx}))
                coordsInThisCell=totalDotCoord{fluoIdx}(totalDotCoord{fluoIdx}(:,4)==c,:);
                newCoords=[newCoords; coordsInThisCell(coordsInThisCell(:,3)>=frameStart{fluoIdx}(c),:)];
            end
        end
    end
    totalDotCoord{fluoIdx}=newCoords;
end

end