% [trjRaw,covRaw,trjRot,covRot,trjInt,fmc]=getCellTrjs(trackPath,cellFilePattern,pxScale,mvRange)
% extract trajectory and precision information from ImAnalysis pipeline
% output.
%
% input :
% trackPath         : path to the folder with pipeline output. Default: pwd
% cellFilePattern   : output file pattern, for use with dir(). Default: 'outCells_*.mat'
% pxLength          : scale factor for trajectory length units. Rescaling:
%                     trjRaw -> trjRaw*pxLength
%                     covRaw -> covRaw*pxLength^2
%                     trjRot -> trjRaw*pxLength
%                     covRot -> covRaw*pxLength^2
%                     Default 1.
% mvRange           : range of files to include. Default: all
%
% output:
% trjRaw    : raw position trajectories [x y]
% covRaw    : raw position posterior covariances [varx covxy vary]
% trjRot, covRot : rotated to have x coordinates measure position along
%                  cell long axis (using the
%                  majorAxisOrientationDegree entry of the cell
%                  objects), and with mean position removed.
% trjInt    : position trajectory in internal cell coordinates
% fmc       : [frame fileNumber cellId ]. fileNumber as ordered by dir(),
%             presumably matching the fluorescence movie file number.
%
% Missing points are reported with x,y=NaN, varx,covxy,vary=inf
%
% ML 2017-04-03

% SZ, 2018-09-21: handle the case of empty trajectory.

function [trjRaw,covRaw,trjRot,covRot,trjInt,fmc]=getCellTrjs(trackPath,cellFilePattern,pxScale,mvRange)

% parse parameters
if(~exist('trackPath','var') || isempty(trackPath) )
    trackPath=pwd;
end
if(~exist(trackPath,'dir'))    
    error(['Tracking path not found at: ' trackPath])
end
if(~exist('cellFilePattern','var') || isempty(cellFilePattern) )
    cellFilePattern='outCells_*.mat';
end
if(~exist('pxScale','var') || isempty(pxScale) )
    pxScale=1;
end
% output files (one for each movie)
outFile=dir(fullfile(trackPath,cellFilePattern));
if(isempty(outFile))
    error(['No files matching ' fullfile(trackPath,cellFilePattern) ])
end
% range of files to include
if(~exist('mvRange','var') || isempty(mvRange) )
    mvRange=1:numel(outFile);
end

% count number of trajectories
numTrj=0;
for f=mvRange
    C=load(fullfile(trackPath,outFile(f).name));
    for c=1:numel(C.cellsArray)        
        numTrj=numTrj+numel(C.cellsArray(c).particles);
    end
end
%trj{r}(t,:)=[x y t mv cellID varx covxy vary ];
trjRaw=cell(1,numTrj);
covRaw=cell(1,numTrj);
trjRot=cell(1,numTrj);
covRot=cell(1,numTrj);
trjInt=cell(1,numTrj);
fmc   =cell(1,numTrj);

tr=0; % trajectory number
for m=mvRange % loop over movies
    C=load(fullfile(trackPath,outFile(m).name));
    for c=1:numel(C.cellsArray)
        for p=1:numel(C.cellsArray(c).particles)
            tr=tr+1; % this trajectory number
            pos0=C.cellsArray(c).particles(p).trajImageCoord;
            if isempty(pos0)
                continue;
            end
            cov0=C.cellsArray(c).particles(p).trajImageCov;
            missing=~isfinite(pos0(:,1))|~isfinite(pos0(:,2));
            pos0(missing,:)=nan;
            cov0(missing,:)=inf;
            
            % raw trajectory with missing points properly marked
            trjRaw{tr}=pos0;
            covRaw{tr}=cov0;
            
            % frames, cellId, and movie number
            currFrames=C.cellsArray(c).particles(p).firstFrame-1+(1:size(pos0,1))';
            fmc{tr}   = [currFrames m*ones(size(currFrames)) c*ones(size(currFrames))];

            % internal coordinates
            posI=C.cellsArray(c).particles(p).trajInternalCoord;
            posI(missing,:)=nan;
            trjInt{tr}=posI;

            % rotated and centered trajectories            
            v=C.cellsArray(c).majorAxisOrientationDegree*pi/180; % cell orientation [radian]
            Rv=[cos(v) -sin(v) ; % rotation matrix
                sin(v)  cos(v)];
            xy0=mean(pos0(~missing,:),1);
            posR=nan(size(pos0));
            covR=inf(size(cov0));
            for t=find(~missing)'
                xyt=(pos0(t,:)-xy0)'; % subtract average position
                posR(t,:)= (Rv*xyt)'; % rotation rule for positions
                
                covXYt=[cov0(t,1) cov0(t,2);cov0(t,2) cov0(t,3)];
                covXYtR= Rv*covXYt*(Rv'); % rotation rule for covariance matrices
                covR(t,:)=[covXYtR(1,1) covXYtR(1,2) covXYtR(2,2)];
            end
            trjRot{tr}=posR;
            covRot{tr}=covR;
        end
    end
end
