classdef logL_EMCCDpsf_symGauss
    
    properties
        logLfun=[];
        
        % the image
        image=[];
        imX=[];
        imY=[];
        imSize=[];
        
        % quadrature quantities
        Xquad=[];
        Yquad=[];
        W=[];        
    end
    methods
        function obj=logL_EMCCDpsf_symGauss(logLobj,C,Nquad)
            % create a fast dot fit object for symmetric Gaussian PSF
            % model, from the components:
            % logLobj   : a log likelihood object, e.g., a logL_EMCCD_lookup
            % C         : an image, consistent with the logLobj camera model
            % Nquad     : number of quadrature points in each dimension
            %             (i.e., there will be Nquad^2 PSFmodel evaluations
            %             per pixel). Quadrature points are equally spaced.
            %
            % Methods
            % 
            % lnL=logL_psf(obj,p) : fast interpolated log likelihood for a
            % symmetric Gaussian PSF model with parameters 
            % p=[mu_x mu_y ln(bg) ln(N) ln(sPSF)]. 
            %
            % [lnL,dlnLdp]=logL_psf(p) : log likelihood + gradient for the
            % above PSF model. NOTE: only if the logLobj has a lnL_dE
            % function!
            %            
            % [E,C]=psfModel(obj,p) : evaluate the PSF intensity image E
            % corresponding to parameters p (as above), and return the
            % image data C.
            % ML 2015-11-20
            
            % store a reference to the log likelohood function
            obj.logLfun=logLobj;
            
            % store image data
            obj.imSize=size(C);
            rows=1:size(C,1);
            cols=1:size(C,2);
            [X,Y]=meshgrid(cols,rows);
            obj.image=reshape(C,numel(C),1);
            obj.imX  =reshape(X,numel(X),1);
            obj.imY  =reshape(Y,numel(Y),1);
            
            % precompute pixel integration weights and
            qPts=linspace(-0.5,0.5,2*Nquad+1);
            qPts=qPts(2:2:end-1);
            [dx,dy]=meshgrid(qPts,qPts);            
            dx=reshape(dx,1,Nquad^2);
            dy=reshape(dy,1,Nquad^2);
            
            % grid for pixel quadrature
            [a,b]=meshgrid(dx,obj.imX);
            obj.Xquad=a+b;
            [a,b]=meshgrid(dy,obj.imY);
            obj.Yquad=a+b;
            obj.W=1/Nquad^2; % same weight for all quadrature points here
        end
        %% log likelihood
        function [E,C]=psfModel(obj,p)
            % compute the model corresponding to parameters p
            E=LSQrefineDots.psf_diff_psfGaussian(obj.Xquad,obj.Yquad,p(1),p(2),p(3),p(4),p(5));
            E=reshape(sum(E,2)*obj.W,obj.imSize(1),obj.imSize(2));
            C=reshape(obj.image,obj.imSize(1),obj.imSize(2));
        end
        function mlnL=mlogL_psf(obj,p)
            % compute negative log-likelihood for symmetric Gaussian psf
            % model parameterixzed by p=[mu_x mu_y ln(bg) ln(N) ln(sPSF)]
 
            E=LSQrefineDots.psf_diff_psfGaussian(obj.Xquad,obj.Yquad,p(1),p(2),p(3),p(4),p(5));
            % quadrature over single pixels
            E=sum(E,2)*obj.W;
            % compute likelihood
            lnLi=obj.logLfun.lnL(obj.image,E);
            mlnL=-sum(lnLi);
        end
        function [mlnL,dmlnLdp]=mlogL_diff_psf(obj,p)
            % compute negative log-likelihood and gradient for symmetric
            % Gaussian psf model parameterixzed by p=[mu_x mu_y ln(bg)
            % ln(N) ln(sPSF)]
            
            % evaulate model intensity and all gradients
            [E,dE_dmuX,dE_dmuY,dE_dlnBG,dE_dlnN,dE_dlnS]=...
                LSQrefineDots.psf_diff_psfGaussian(obj.Xquad,obj.Yquad,p(1),p(2),p(3),p(4),p(5));
            % quadrature over single pixels
            E=sum(E,2)*obj.W;
            dE_dmuX=sum(dE_dmuX,2)*obj.W;
            dE_dmuY=sum(dE_dmuY,2)*obj.W;
            dE_dlnBG=sum(dE_dlnBG,2)*obj.W;
            dE_dlnN=sum(dE_dlnN,2)*obj.W;
            dE_dlnS=sum(dE_dlnS,2)*obj.W;
            
            % log likelihood and derivative
            [lnLi,dlnLi_dE]=obj.logLfun.lnL_dE(obj.image,E);
            mlnL=-sum(lnLi);
            
            dmlnLdp  = p;
            dmlnLdp(1)  = -sum(dlnLi_dE.*dE_dmuX );
            dmlnLdp(2)  = -sum(dlnLi_dE.*dE_dmuY );
            dmlnLdp(3)  = -sum(dlnLi_dE.*dE_dlnBG);
            dmlnLdp(4)  = -sum(dlnLi_dE.*dE_dlnN );
            dmlnLdp(5)  = -sum(dlnLi_dE.*dE_dlnS );
        end        
    end
end
