% This code computes PSF using a 3*3-point Gauss quadrature rule.
%
% E = psfGaussianSymmetric_9ptGQ(x,y,b,A,mu_x,mu_y,sigmaS,pxL)
% 
% % Input : 
%     - x : data (vector format) that corresponds to the x-variable of
%           pixel mid-points
%     - y : data (vector format) that corresponds to the y-variable of
%           pixel mid-points
%     - b : background
%     - A : expected number of photons per frame
%     - mu_x : x_position of the centre of PSF
%     - mu_y : y_position of the centre of PSF
%     - sigmaS : PSF standard deviation
%     - pxL : linear pixel size (default 1)
%
% Output:
%     - psfGaussianSymmetric : PSF function (vector format), approximately
%       integrated over every pixel.
% Author: Vladimir Curic, 2015-01-22, vladacuric@gmail.com
%         Martin Lindén, 2015-08-21, bmelinden@gmail.com, added multipoint
%         version.
% 
% ref: https://en.wikipedia.org/wiki/Gaussian_quadrature
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  E = psfGaussianSymmetric_9ptGQ(x,y,b,A,mu_x,mu_y,sigmaS,pxL)

% default parameter
if(nargin<8)
    pxL=1;
end
% check
if(numel(b)>1 || numel(A)>1 || numel(mu_x)>1 || numel(mu_y)>1 || numel(sigmaS)>1)
   error(' psfGaussianSymmetric_refined cannot handle more than one set of shape parameters at once')
end


% node points: +-1/sqrt(3) on the [-1,1] interval, translates to +-
% 1/2/sqrt(3)) on the [-0.5,0.5] interval (
dx=pxL/2*[0 -1 1 0 -1 1 0 -1 1]*sqrt(3/5);
dy=pxL/2*[0 0 0 -1 -1 -1 1 1 1]*sqrt(3/5);
w=(pxL/2/9)^2*[64;40;40;40;25;25;40;25;25];% [8 5 5  8 5 5 8 5 5]'.*[8 8 8 5 5 5 5 5 5]';

% compute refined 
N = numel(x);
E = zeros(1,N);
for i = 1:N
    E(i) = (b+A*(1./(2*pi*sigmaS^2))*(exp(-((x(i)+dx-mu_x).^2+(y(i)+dy-mu_y).^2)/(2*sigmaS^2))))*w;
end
