% [MLEtrj,MLEtrjRot,param]=trjAddMLEuncertainties(LSQcoordFile,pathSegImages)
% reinsert position uncertainties in trajectories made from MLE positions
% from a pipeline intermediate file (usually called LSQdotCoordinates.mat)
% produced by the LSQrefineDots pipeline option, and
% LSQrefineDots_doMLE=true.
%
% input :
%
% LSQcoordFile : LSQrefineDots intermediate file. 
%                Default: fullfile(pwd,'LSQdotCoordinates.mat'), if an
%                empty matrix is given.
% pathSegImages: path to the segmented images. 
%                Default: the values saved in the LSQcoordFile. Not using
%                the default might be necessary when running the analysis
%                on a different computer than was used to run the image
%                analysis pipeline, in which case the paths might differ.
%
% output:
% MLEtrj, MLEtrjRot : tracked trajectories, of the form
% MLEtrj{r}(t,:)=[x y t mv ci varx covxy vary], where
% x,y are positions, whose localization errors are characterized by a
% posterior covariance matrix [varx covxy;covxy vary].
% t = frame number, mv=movie number, ci = cell ID number.
% MLEtrj are given in original coordinates, MLEtrjRot in coordinates that
% are rotated to make align the cell long axes with the x axis.
% 
% param : a struct with some fields including paths to the fluo movie and
% segmentation
%
% Missing points are reported with x,y=NaN, varx,vary=inf, covxy=0
%
% Two useful special cases: 
% 1) When called from the outputFolder on the computer where
% the image analysis pipeline ran:
% [MLEtrj,MLEtrjRot,param]=LSQrefineDots.trjAddMLEuncertainties(); 
% 2) When called from the outputFolder on a different computer:
% [MLEtrj,MLEtrjRot,param]=LSQrefineDots.trjAddMLEuncertainties([],'SegmentedImages');
%
% ML 2015-12-07
function [MLEtrj,MLEtrjRot,param]=trjAddMLEuncertainties(LSQcoordFile,pathSegImages)
maxMismatch=1e-13; % margin of error for matching dot positions.

% parse parameters
inputParameters=struct;
if(exist('LSQcoordFile','var') && ~isempty(LSQcoordFile) )
    inputParameters.LSQcoordFile=LSQcoordFile;
else
    inputParameters.LSQcoordFile=fullfile(pwd,'LSQdotCoordinates.mat');
end
if(exist('pathSegImages','var') && ~isempty(pathSegImages) )
    inputParameters.pathSegImages=pathSegImages;
else
    inputParameters.pathSegImages=[];
end
% load pipeline intermediates
load(inputParameters.LSQcoordFile);
if(isempty(inputParameters.pathSegImages))
    inputParameters.pathSegImages=pathSegImages;
end

% parameters
param=struct;
param.inputParameters=inputParameters;
param.parameters=parameters;
param.sourceFolder =sourceFolder ;
param.fluorChannelName =fluorChannelName ;
param.fluoList = fluoList;
param.pathSegImages =pathSegImages ;
param.segImagesList =segImagesList ;


% count number of trajectories
numTrj=0;
for m=1:length(finalImageTraj)
    numTrj=numTrj+length(finalImageTraj{m});
end

%MLEtrj{r}(t,:)=[x y t mv cellID varx covxy vary ];
MLEtrj=cell(1,numTrj);
MLEtrjRot=cell(1,numTrj);
tr=0; % trajectory number
for m=1:length(finalImageTraj)          % loop over movies
    % figure out cell orientations from segmented images
    currSegIm=imread(fullfile(inputParameters.pathSegImages,segImagesList(m).name));
    %currSegIm=imread(fullfile('SegmentedImages',segImagesList(m).name));
    rp=regionprops(currSegIm,'orientation','centroid');
    
    for r=1:length(finalImageTraj{m})    % loop over trajectories
        tr=tr+1;
        MLEtrj{tr}=zeros(size(finalImageTraj{m}{r},1),8);
        MLEtrjRot{tr}=zeros(size(finalImageTraj{m}{r},1),8);
        for t=1:size(finalImageTraj{m}{r},1) % loop over frames
            fr=finalImageTraj{m}{r}(t,4);    % frame number
            ci=finalImageTraj{m}{r}(t,5);     % cell ID number
            xt=finalImageTraj{m}{r}(t,1);% /allFluoToSegScaling(m,1);    % not needed now that trj are in fluo units
            yt=finalImageTraj{m}{r}(t,2);% /allFluoToSegScaling(m,2);    %
            % cell mid-point
            xc=rp(ci).Centroid(1)/allFluoToSegScaling(m,1);
            yc=rp(ci).Centroid(2)/allFluoToSegScaling(m,2);
            % cell orientation
            v=rp(ci).Orientation*pi/180;
            Rv=[cos(v) -sin(v) ; % rotation matrix
                sin(v)  cos(v)];
            
            if(isnan(xt) && isnan(yt) ) % then this point is a gap
                MLEtrj{tr}(t,:)   =[NaN NaN fr m ci inf 0 inf ];
                MLEtrjRot{tr}(t,:)=[NaN NaN fr m ci inf 0 inf ];
            else % find this point in the coordinate array
                ind=find(allMLEcoord{m}(:,3)==fr); % points in this frame
                
                % replace == with some small tolerance ~1e-13 ?
                [dr,k]=min(abs(xt-allMLEcoord{m}(ind,1))...
                          +abs(yt-allMLEcoord{m}(ind,2)));                
                if(dr<maxMismatch)
                    ii=ind(k);
                    covXY=[allMLEcov{m}(ii,1) allMLEcov{m}(ii,2);allMLEcov{m}(ii,2) allMLEcov{m}(ii,3)];
                    MLEtrj{tr}(t,:)=[allMLEcoord{m}(ii,1:2) fr m ci covXY([1 2 4])];
                    
                    % rotate coordinates and covarince matrix
                    dXY=[xt-xc;yt-yc];
                    covXYr=Rv*covXY*(Rv'); % rotate xy-covariance matrix
                    XYr=Rv*dXY +[xc;yc];
                    MLEtrjRot{tr}(t,:)=[XYr' fr m ci covXYr([1 2 4])];
                    
                else
                   % something strange happened 
                   errFileName=['trjAddMLEuncertainties_error_' int2str(1e6*rand)];
                   error(['Position mismatch ' num2str(dr) ' too large. Saving workspace to ' errFileName ])
                end                
            end
        end        
    end
end

