function dotCoords = computeSpotCentroids(mask, im, minSpotArea, maxAxesRatio, showDots)
% dotCoords = computeSpotCentroids(mask, im, minSpotArea, maxAxesRatio, showDots)
%
% This function computes weighted centroids for each spot defined by the 
% spot mask.
% It detects overlapping spots by computing aspect ratio between major and 
% minor axes. If the ratio > maxAxesRatio, split the spot into two spots 
% using the minor axis that passes through the weighted centroid. For each
% new spot, compute its weighted centroid.
% This function also discards dim spots if
% spot area in pixels < minSpotArea.
%
% Input:
%   mask - logical matrix, 1 if pixel belongs to a spot.
%   im - fluorescence image.
%   minSpotArea - integer, discard a spot if its area (number of pixels) 
%                 is below this threshold.
%   maxAxesRatio - scalar, a spot is an overlap of two spots if the ratio
%                  between major and minor axes is above this threshold
%   showDots - if 1, plot heatmap of spots with axes ratio and plot
%              fluorescence image with spot centroids.
%
% Output:
%   dotCoords - matrix with two columns [xcoord ycoord], 
%               where xcoord and ycoord are the coordinates of the 
%               weighted centroids.

if ~exist('minSpotArea', 'var') || isempty(minSpotArea)
    minSpotArea = 0;
end

if ~exist('maxAxesRatio', 'var') || isempty(maxAxesRatio)
    maxAxesRatio = 1.5;
end

if ~exist('showDots', 'var') || isempty(showDots)
    showDots = 0;
end

stats = regionprops(mask, im, 'WeightedCentroid','MajorAxisLength',...
    'MinorAxisLength','Area','Orientation','PixelList');
spotAreas = [stats.Area];
stats = stats(spotAreas >= minSpotArea);
spotAreas = spotAreas(spotAreas >= minSpotArea);
axesRatio = [stats.MajorAxisLength] ./ [stats.MinorAxisLength];
orientation = [stats.Orientation];
dotCoords = [stats.WeightedCentroid];
dotCoords = [dotCoords(1:2:end); dotCoords(2:2:end)]';

overlapIndices = find(axesRatio > maxAxesRatio & spotAreas >= 2*minSpotArea);
if isempty(overlapIndices)
    return
end

splitDotCoords = cell(length(overlapIndices),1);
for i = overlapIndices
    angle = 90-orientation(i);
    x0 = dotCoords(i,1);
    y0 = dotCoords(i,2);
    xs = stats(i).PixelList(:,1);
    ys = stats(i).PixelList(:,2);
    minorAxisYcoords = round(tand(angle)*(xs-x0)+y0);
    ind = sub2ind(size(im),stats(i).PixelList(:,2),stats(i).PixelList(:,1));
    ws = double(im(ind));
    ind = find(ys <= minorAxisYcoords);
    if sum(ind) >= minSpotArea
        xc1 = sum(xs(ind).*ws(ind)) / sum(ws(ind));
        yc1 = sum(ys(ind).*ws(ind)) / sum(ws(ind));
    else
        xc1 = [];
        yc1 = [];
    end
    ind = find(ys > minorAxisYcoords);
    if sum(ind) >= minSpotArea
        xc2 = sum(xs(ind).*ws(ind)) / sum(ws(ind));
        yc2 = sum(ys(ind).*ws(ind)) / sum(ws(ind));
    else
        xc2 = [];
        yc2 = [];
    end
    splitDotCoords{i} = [xc1 yc1;xc2 yc2];
end
splitDotCoords = cell2mat(splitDotCoords);

if showDots
    
    ratioMap = zeros(size(mask));
    for i=1:numel(stats)
        ind = sub2ind(size(im),stats(i).PixelList(:,2),stats(i).PixelList(:,1));
        ratioMap(ind) = axesRatio(i);
    end
    
    figure, hold on
    imagesc(ratioMap),
    axis image; set(gca,'Ydir','reverse','YTick',[],'XTick',[]);
    plot(dotCoords(:,1),dotCoords(:,2),'r.');
    plot(splitDotCoords(:,1), splitDotCoords(:,2), 'b.');
    hold off

    bb = bwboundaries(mask,'noholes');
    figure, hold on
    the_clim = quantile(double(im(:)), [0 0.99]);
    imagesc(im, the_clim); colormap gray;
    axis image; set(gca,'Ydir','reverse','YTick',[],'XTick',[]);
    for i=1:numel(bb)
        plot(bb{i}(:,2),bb{i}(:,1),'g-')
    end
    plot(dotCoords(:,1),dotCoords(:,2),'r.');
    plot(splitDotCoords(:,1), splitDotCoords(:,2), 'b.');
    hold off
end

dotCoords(overlapIndices,:) = [];
dotCoords = [dotCoords; splitDotCoords];