function [totalDotCoord] = detectDots(expInfoObj,posName,fluoChanName)
% [totalDotCoord] = detectDots(expInfoObj,posName,fluoChanName)
%
% Detects dots for all fluo images/movies in a single position. 
% Two algorithms are supported, wavelet-based and radial-symmetry based.
%
% Input:
%   expInfoObj   - expInfo object with various info about the experiment
%   posName      - position name
%   fluoChanName - fluo channel name
%
% Output
%   totalDotCoord - cell array, where each cell corresponds to a fluo 
%                   frame and contains a matrix where each row 
%                   represents a detected dot with columns
%                       [xcoord ycoord movieFrame]. 
%                   If detection algorithm is "wavelet" and 
%                   noiseThresh = -1, dots are not detected and 
%                   totalDotCoord = [].

% Last modified:
% SZ, 2018-08-22. Improved dot detection for the wavelet algorithm that
% handles the case of overlapping fluorescence spots. Discard dim spots 
% based on their area.
% SZ, 2018-08-09. Support different parameters for multiple fluo channels. 
% Only for the wavelet detection algorithm.
% SZ, 2018-06-05. Bugfix, wrong computation of the image
% size when the fluor ROI is provided.

%% Initialization
parameters = expInfoObj.getParameters();
fluoRange = expInfoObj.getRange(posName, fluoChanName);
fluoIdxs  = expInfoObj.getIndices(posName, fluoChanName);
phaseChanName = expInfoObj.getChannelNames('phase');
phaseIdxs = expInfoObj.getIndices(posName, phaseChanName);
sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');

%% Read parameters
nCores = parameters.parNumWorkers;
if parameters.parforPositions
    nWorkersMovies = 0;
    nWorkersMovieFrames = 0;
else
    if length(phaseIdxs)>1
        nWorkersMovies = nCores;
        nWorkersMovieFrames = 0;
    else
        nWorkersMovies = 0;
        nWorkersMovieFrames = nCores;
    end
end
rangeFluoStack = parameters.rangeFluoStack;
dotDetectionAlg = parameters.dotDetectionAlg;

if ~any(strcmp(dotDetectionAlg, {'wavelet','radialSymmetry'}))
    error('The parameter for choosing the dot detection algorithm is incorrectly set')
end

% Choose the right parameters for the current fluo channel
fluoChanId = strcmp(fluoChanName, expInfoObj.fluoChannelNames); 
if size(parameters.noiseThreshold, 1)>1
    noiseThreshold = parameters.noiseThreshold(fluoChanId); 
else
    noiseThreshold = parameters.noiseThreshold;
end

if size(parameters.minSpotArea, 1) > 1
    minSpotArea = parameters.minSpotArea(fluoChanId);
else
    minSpotArea = parameters.minSpotArea;
end

if size(parameters.maxAxesRatio, 1) > 1
    maxAxesRatio = parameters.maxAxesRatio(fluoChanId);
else
    maxAxesRatio = parameters.maxAxesRatio;
end
    
if size(parameters.radii, 1)>1
    radii = parameters.radii(fluoChanId,:); 
else
    radii = parameters.radii;
end

if size(parameters.alpha, 1)>1
    alphaParam = parameters.alpha(fluoChanId); 
else
    alphaParam = parameters.alpha;
end

if size(parameters.beta, 1)>1
    betaParam = parameters.beta(fluoChanId); 
else
    betaParam = parameters.beta;
end

if size(parameters.radThreshold, 1)>1
    radThreshold = parameters.radThreshold(fluoChanId); 
else
    radThreshold = parameters.radThreshold;
end

if size(parameters.radSymThreshold, 1)>1
    radSymThreshold = parameters.radSymThreshold(fluoChanId); 
else
    radSymThreshold = parameters.radSymThreshold;
end

if size(parameters.gaussKernRadFactor, 1)>1
    gaussKernRadFactor = parameters.gaussKernRadFactor(fluoChanId); 
else
    gaussKernRadFactor = parameters.gaussKernRadFactor;
end

if strcmp(dotDetectionAlg,'wavelet') && noiseThreshold == -1
    totalDotCoord = [];
    return    
end
%% Prepare shared variables for parfor loop
fluoDir = fullfile(sourceDir, posName, fluoChanName);
fluoList = dir(fullfile(fluoDir,'*.tif*'));
fluoList = fluoList(fluoRange);
segDir = fullfile(outputDir, posName,'SegmentedPhase');
segList = dir(fullfile(segDir,'*.tif*'));

fluoImPaths = fullfile(fluoDir,{fluoList.name});

if strcmp(dotDetectionAlg,'wavelet')
    roiInFluo = parameters.roiInFluo;
    if ~isempty(roiInFluo)
        fluoImSize = roiInFluo([4 3])+1;
    else
        fluoImInfo = imfinfo(fluoImPaths{1});
        fluoImSize = [fluoImInfo(1).Height fluoImInfo(1).Width];
    end
    Ts = expInfoObj.getAllT(posName,fluoChanName);
    [~, phaseFluoIdxs] = intersect(phaseIdxs, fluoIdxs);
    segImPaths = fullfile(segDir,{segList(phaseFluoIdxs).name});
end

%% Main code
totalDotCoord = cell(1,length(fluoList));
if strcmp(dotDetectionAlg, 'wavelet')
    parfor(i = 1:length(fluoRange), nWorkersMovies)
        segIm = imread(segImPaths{i});
        segMask = Spot.Detect.makeMaskFromSegmentedImage(segIm,fluoImSize,Ts(:,:,i));
        dotCoord = BuildTrajectories.getDots(fluoImPaths{i}, noiseThreshold,...
            rangeFluoStack, segMask, minSpotArea, maxAxesRatio, 0, nWorkersMovieFrames);
        if ~isempty(dotCoord)
            totalDotCoord{i} = dotCoord;
        end
    end
elseif strcmp(dotDetectionAlg, 'radialSymmetry')
    parfor(i = 1:length(fluoRange), nWorkersMovies)
        dotCoord = BuildTrajectories.getDotsFRS(fluoImPaths{i}, radii, ...
            alphaParam, betaParam, radThreshold, radSymThreshold, rangeFluoStack, ...
            gaussKernRadFactor, 0, nWorkersMovieFrames);
        if ~isempty(dotCoord)
            totalDotCoord{i} = dotCoord(:,1:3);
        end
    end
end