function [curatedTrajs, curatedDots] = curateDuplicatedDots(dotTrajs,blobsArray,trackedDots,frameRange,parameters)
% [curatedTrajs, curatedDots] = curateDuplicatedDots(dotTrajs,blobsArray,trackedDots,frameRange,parameters)
% 
% Detects duplicated dots in trajectories and keeps the ones that belong
% the longest trajectory. This might happen if we use a dilated cell image
% to detect dots in the cell. Each dot is represented by row vector
% [x y frame cell id], and optionally [var(x) var(y) var(xy)]
% 
% Input:
%   dotTrajs    - cell array with detected trajectories in cells. 
%   blobsArray  - row vector with labels of blobs (cells) in ROI, same
%                 length as dotTrajs
%   trackedDots - matrix where each row represents info about a dot:
%   frameRange  - index array of fluo images in tiff stack of interest
%   parameters  - structure with dot detection parameters

% Last modified:
% SZ, 2018-09-27. Handled the case of empty trajectories (false segmentation)

curatedTrajs = dotTrajs;
curatedDots = trackedDots;

% Get all dots from the trajectories
tmpTrajs = horzcat(dotTrajs{:});
if isempty(tmpTrajs)
    return;
end
allTrajs = vertcat(tmpTrajs{:});
[~, ~, ic] = unique(allTrajs(:,5),'stable');
valCounts = histcounts(double(ic),max(ic));

curatedTrajs = dotTrajs;
curatedDots = trackedDots;
rebuildTraj = false(1,length(blobsArray));
for idx = find(valCounts > 1)
    conflictDots = allTrajs(ic == idx,:);
    trajLength = zeros(1,size(conflictDots,1));
    currDotId = conflictDots(1,5);
    % Find which trajectory they belong to
    for dotIdx = 1:size(conflictDots,1)
        currCell = conflictDots(dotIdx,4);
        currBlob = (blobsArray == currCell);
        currCellTrajs = dotTrajs{currBlob};
        
        for trajIdx = 1:length(currCellTrajs)
            currTraj = currCellTrajs{trajIdx};
            if any(currTraj(:,5) == currDotId)
                trajLength(dotIdx) = size(currTraj,1);
                break
           end
        end
    end

    [~, maxIdx] = max(trajLength);

    % Remove the dot from the shorter trajectories and rebuild the trajectories 
    for dotIdx = [1:maxIdx-1 maxIdx+1:size(conflictDots,1)]
        currDotId = conflictDots(dotIdx,5);
        currBlob = conflictDots(dotIdx,4);
        % Remove the conflicting point
        conflictIdx = (curatedDots(:,4) == currBlob) & (curatedDots(:,5) == currDotId);
        curatedDots(conflictIdx,:) = [];
        rebuildTraj(blobsArray == currBlob) = true;
    end
    
end

for blobIdx = find(rebuildTraj)
    currBlob = blobsArray(blobIdx);
    dotsInCurrCell = curatedDots(curatedDots(:,4) == currBlob,:);
    curatedTrajs{blobIdx} = BuildTrajectories.buildTrajectories(...
        dotsInCurrCell,frameRange,parameters);
end

end