function processDots(expInfoObj, posName, fluoChanName, mFrames)
% processDots(expInfoObj, posName, fluoChanName, mFrames)
%
% Detects and tracks dots in fluorescence images that belong to cells, 
% adds them to the corresponding MCell objects.
%
% Input:
%   expInfoObj   - expInfo object
%   posName      - string, position name
%   fluoChanName - string, fluor channel name
%   mFrames      - MFrames object that allows to access MCell objects by 
%                  frame and blob index. Optional.

% Last modified:
% SZ, 2018-06-16. Code parallelization.
% SZ, 2018-08-09. Support different parameters for multiple fluo channels. 
% Only for the wavelet detection algorithm.
% SZ, 2018-09-21. Parse nCores parameter for parfor loops. Load LSQrefine
% results.
% SZ, 2019-03-15. Created two versions of the code: serial and parallel.
%% Initialization
import Spot.*

if nargin==3
    mCellsMatFile = expInfoObj.getMCellMatPath(posName);
    [~, mFrames] = Cell.MCell.loadMCells(mCellsMatFile);
end

parameters = expInfoObj.getParameters();

if ~parameters.parforPositions
    processDots_parfor(expInfoObj, posName, fluoChanName, mFrames);
    return
end

imRangeFluo = expInfoObj.getRange(posName,fluoChanName);
if isempty(imRangeFluo)
    return
end
sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');
phaseChanName = expInfoObj.getChannelNames('phase');
phaseIdxs = expInfoObj.getIndices(posName,phaseChanName);
fluoIdxs  = expInfoObj.getIndices(posName,fluoChanName);
[~,phaseFluoIdxs] = intersect(phaseIdxs, fluoIdxs);

posDir = fullfile(outputDir, posName);
fluoDir = fullfile(sourceDir, posName, fluoChanName);
fluoList = dir(fullfile(fluoDir,'*.tif*'));
fluoList = fluoList(imRangeFluo);
segDir = fullfile(outputDir, posName, 'SegmentedPhase');
segList = dir(fullfile(segDir,'*.tif*'));
segList = {segList.name};
segList = segList(phaseFluoIdxs);
Ts = expInfoObj.getAllT(posName,fluoChanName);

fluoROI = parameters.roiInFluo;
if ~isempty(fluoROI)
    fluoImSize = [fluoROI(2)+fluoROI(4)-1, fluoROI(1)+fluoROI(3)-1];
else
    fluoImInfo = imfinfo(fullfile(fluoDir,fluoList(1).name));
    fluoImSize = [fluoImInfo(1).Height fluoImInfo(1).Width];
    fluoROI = [1 1 fluoImInfo(1).Width fluoImInfo(1).Height];
end

rangeFluoStack = parameters.rangeFluoStack;
if isempty(rangeFluoStack)
    if ~exist('fluoImInfo','var')
        fluoImInfo = imfinfo(fullfile(fluoDir,fluoList(1).name));
    end
    rangeFluoStack = 1:length(fluoImInfo);
end

if parameters.cellInclusionMargin > 0
    marginStrel = strel('disk',parameters.cellInclusionMargin);
else
    marginStrel = [];
end

%% Detect all dots in fluo images
totalDotCoord = Spot.Detect.detectDots(expInfoObj,posName,fluoChanName);
if isempty(totalDotCoord)
    return
end
saveDir = fullfile(outputDir, posName);
saveName =  ['dotCoord_' fluoChanName '.mat'];
save(fullfile(saveDir,saveName), 'totalDotCoord');
%% Detect dots inside (tracked) cells in fluo ROI
excludeCellsFluorBoundary = parameters.excludeCellsFluorBoundary;
blobsInRoi = cell(1,length(fluoList));
for fluoIdx = 1:length(fluoList)
    dotsInFluo = totalDotCoord{fluoIdx};
    currSegIm = imread(fullfile(segDir,segList{fluoIdx}));
    phaseFrame = phaseFluoIdxs(fluoIdx);
    T = Ts(:,:,fluoIdx);    
    % Filter out dots on the image border    
    dotsInFluo = Spot.Refine.filterDotsOnBorder(dotsInFluo,fluoImSize);
    % Filter out dots by fluo ROI
    dotsInFluo = Spot.Refine.filterDotsByFluoRoi(dotsInFluo,fluoROI);
    % Filter out cells by fluo ROI
    if excludeCellsFluorBoundary
        blobsInRoi{fluoIdx} = Spot.Refine.filterCellsByFluoRoi(mFrames,phaseFrame,fluoROI,T);
    else
        blobsInRoi{fluoIdx} = mFrames.getBlobsInFrame(phaseFrame);
    end
    if ~isempty(blobsInRoi{fluoIdx})
        % Remove the dots that are not close enough to any active cell
        allCellsIm = false(size(currSegIm));
        for blob = blobsInRoi{fluoIdx}
            allCellsIm(currSegIm == blob) = 1;
        end
        dotsInPhase = convertCoord(dotsInFluo,T);
        dotsInPhase = BuildTrajectories.findPointsInCell(...
            dotsInPhase,rangeFluoStack,allCellsIm,1,marginStrel);
        dotsInFluo = convertCoord(dotsInPhase,T,1);
        totalDotCoord{fluoIdx} = dotsInFluo;
    else
        totalDotCoord{fluoIdx} = [];
    end
end
saveName = ['dotsInROI_' fluoChanName '_par.mat'];
save(fullfile(saveDir,saveName),'totalDotCoord','blobsInRoi');
%% Dot refinement, by ML
if parameters.doLSQrefineDots == 1
%    disp('Running dot refinement based on MAP EMCCD localization and symmetric Gaussian PSF model');
    LSQsaveFile = fullfile(posDir,  ['LSQdotCoordinates_' fluoChanName '.mat']);
    if exist(LSQsaveFile, 'file')
        savedLSQdata = load(LSQsaveFile,'totalDotCoord0','totalDotCoord');
        totalDotCoord0 = savedLSQdata.totalDotCoord0;
    else
        totalDotCoord0 = [];
    end
    if 0%isequal(totalDotCoord, totalDotCoord0)
        totalDotCoord = savedLSQdata.totalDotCoord;
    else
        totalDotCoord0 = totalDotCoord;
        [totalDotCoord,allMAPcoord,allMAPpar,allMAPcov] = ...
            LSQrefineDots.MAPsgS0refineDots(expInfoObj,posName,fluoChanName,totalDotCoord);
        save(LSQsaveFile,'totalDotCoord0','totalDotCoord','allMAPcoord','allMAPpar','allMAPcov');
    end
end
%% Build dot trajectories
maxFluoFrame = rangeFluoStack(end);
maxDotsInCell = parameters.maxDotsInCell;
dotTrackingAlg = parameters.dotTrackingAlg;
for fluoIdx = 1:length(fluoList)
    dotsInFluo = totalDotCoord{fluoIdx};
    phaseFrame = phaseFluoIdxs(fluoIdx);
    if isempty(dotsInFluo)
        continue;
    end
    T = Ts(:,:,fluoIdx);
    currSegIm = imread(fullfile(segDir,segList{fluoIdx}));
    % Determine cell identity of all spots, and discard spots outside cells
    nBlobs = length(blobsInRoi{fluoIdx});
    dotsInPhase = convertCoord(dotsInFluo,T);
    dotTrajs = cell(nBlobs,1);
    dotData = [];
    for blobIdx = 1:nBlobs
        currBlob = blobsInRoi{fluoIdx}(blobIdx);
        % Select dots in the cells
        cellDotsInPhase = BuildTrajectories.findPointsInCell(...
            dotsInPhase(:,1:3),[],currSegIm,currBlob,marginStrel,dotsInPhase(:,6:end));        
        if isempty(cellDotsInPhase)
            continue
        end
        cellDotsInFluo = convertCoord(cellDotsInPhase,T,1);
        % Discard movie frames with number of dots in cell above threshold
        cellDotsInFluo = Spot.Refine.filterDotsInCell(cellDotsInFluo,maxFluoFrame,maxDotsInCell);
        dotData = [dotData; cellDotsInFluo];
        % Build dot trajectories
        if ~isempty(dotTrackingAlg)
            dotTrajs{blobIdx} = BuildTrajectories.buildTrajectories(cellDotsInFluo, rangeFluoStack, parameters);
        else
            dotTrajs{blobIdx} = num2cell(cellDotsInFluo,2);
        end          
    end
    if parameters.removeDuplicatedDots
        dotTrajs = Spot.Refine.curateDuplicatedDots(dotTrajs,blobsInRoi{fluoIdx},dotData,rangeFluoStack,parameters);
    end
    %% Update MCell objects
    for blobIdx = 1:nBlobs
        currBlob = blobsInRoi{fluoIdx}(blobIdx);  
        cellsInBlob = mFrames.getCellsInBlob(phaseFrame, currBlob);
        c = cellsInBlob(1);
        if isempty(dotTrajs)
            dotTrajsInCell = [];
        else
            dotTrajsInCell = dotTrajs{blobIdx};
        end
        if ~isempty(dotTrajsInCell)           
            nTrajs = numel(dotTrajsInCell);
            for trajIdx = 1:nTrajs
                currTraj = dotTrajsInCell{trajIdx};
                trajLength = size(currTraj,1);
                pIdx = length(c.particles) + 1;
                c.particles(pIdx).channelName = fluoChanName;
                c.particles(pIdx).trajImageCoord = currTraj(:,1:2);
                c.particles(pIdx).trajPhaseImageCoord = convertCoord(currTraj(:,1:2),T);
                c.particles(pIdx).cellDetectionFrames = repmat(phaseFrame,trajLength,1);
                c.particles(pIdx).blobLabels = repmat(currBlob,trajLength,1);
                c.particles(pIdx).firstFrame = currTraj(1,3);
                c.particles(pIdx).numberOfFrame = trajLength;
                if size(currTraj,2) == 8             
                    c.particles(pIdx).trajImageCov = currTraj(:,6:8);
                end
            end
        else
             % Mark that cell is inside fluo roi but has no dots
            nTrajs = 1;
            pIdx = length(c.particles)+1;
            c.particles(pIdx).channelName = fluoChanName;
            c.particles(pIdx).trajImageCoord = [];
            c.particles(pIdx).cellDetectionFrames = phaseFrame;
            c.particles(pIdx).blobLabels = currBlob;
            c.particles(pIdx).numberOfFrame = 0;
        end
        % Copy particles to other cell tracks that share this blob
        ncP = numel(c.particles);
        pIdxs = ncP-nTrajs+1:ncP;
        for ocIdx = 2:numel(cellsInBlob)
            oc = cellsInBlob(ocIdx);
            oc.particles(end+1:end+nTrajs) = c.particles(pIdxs);
        end
    end
end