import PerObjectEllipsefit as poe
import numpy as np
import scipy
import scipy.ndimage as nd
import time

import os
import sys
import shutil
import Queue
from threading import Thread
from subprocess import call
import multiprocessing
from multiprocessing import Process
import math
# from Tifffile import tifffile
import cv2

class POE_Parallel(object):
    def __init__(self):
        pass

    def run(self, InputDir, \
            OutputDir, \
            area_interval_min = 700, \
            area_interval_max = 4000, \
            ellipsefit_thr = 0.88, \
            major_interval_min = 30, \
            major_interval_max = 200, \
            minor_interval_min = 15, \
            minor_interval_max = 30, \
            major_minor_ratio_interval_min = 1.5, \
            major_minor_ratio_interval_max = 10, \
            brightObjects = True, \
            resample = False, \
            resampleMax = 255, \
            thresholdwindow = 255,\
            sigmaVal = 1.5\
            discardBorderCells = 1):
            
        self.area_interval_min = area_interval_min
        self.area_interval_max = area_interval_max
        self.ellipsefit_thr = ellipsefit_thr
        self.major_interval_min = major_interval_min
        self.major_interval_max = major_interval_max
        self.minor_interval_min = minor_interval_min
        self.minor_interval_max = minor_interval_max
        self.major_minor_ratio_interval_min = major_minor_ratio_interval_min
        self.major_minor_ratio_interval_max = major_minor_ratio_interval_max
        self.brightObjects = brightObjects
        self.resample = resample
        self.resampleMax = resampleMax
        self.thresholdwindow = thresholdwindow
        self.sigmaVal = sigmaVal
        
        nProc = multiprocessing.cpu_count()
        self.files = [name for name in os.listdir(InputDir) if os.path.isfile(os.path.join(InputDir, name))]
        self.nFiles = len(self.files)
        nProc = min(12, self.nFiles)
        self.imagesPerThread = int(math.ceil(self.nFiles / float(nProc)))
        
        self.InputDir = InputDir
        self.InputDir = self.InputDir.replace('\\', '/')
        if not self.InputDir.endswith('/'):
            self.InputDir = self.InputDir + '/'
            
        self.OutputDir = OutputDir
        self.OutputDir = self.OutputDir.replace('\\', '/')
        if not self.OutputDir.endswith('/'):
            self.OutputDir = self.OutputDir + '/'
        if not os.path.isdir(self.OutputDir):
            os.mkdir(self.OutputDir)
                
        procs = [Process(target=self.worker, args=(ii,)) for ii in range(0, nProc)]
        
        for proc in procs:
            proc.start()
            
        # join all threads
        for proc in procs:
            proc.join()
        
    def gauss_kern(self, Img, sigmaVal):
        """ Returns a normalized 2D gauss kernel array for convolutions """
        h2,h1 = Img.shape    
        x, y = np.mgrid[0:h2, 0:h1]
        x = x-h2/2
        y = y-h1/2
        g = np.exp( -( x**2 + y**2 ) / (2*sigmaVal**2) );
        return g / g.sum()  
    
    def enhanceContrast(self, orimg):
    
        orimg = np.float32(orimg)
        #orimg = orimg.max()-orimg # uncomment for fluorescent data
        
        # find the eigen value of gaussian smoothed hessian image
        Img = np.copy(orimg)
        gau = self.gauss_kern(Img, self.sigmaVal)
        Imgfft = np.fft.rfft2(Img)
        gfft = np.fft.rfft2(gau)
        fftimage = np.multiply(Imgfft, gfft)
        Img_smooth =np.real(np.fft.ifftshift( np.fft.irfft2(fftimage)))
        Iy, Ix = np.gradient(Img_smooth)
        Ixy, Ixx = np.gradient(Ix)
        Iyy, Iyx = np.gradient(Iy)
        
        eigvallam = np.zeros((2,Ixx.shape[0],Ixx.shape[1] ))
        trhessian = Ixx+Iyy
        dethessian = Ixx*Iyy-Ixy*Ixy
        # find eigen values of hessian
        D = trhessian*trhessian - (4*dethessian)
        D[D<0] = 0
        eigvallam[0,:,:] = 0.5*(trhessian + np.sqrt(D))
        eigvallam[1,:,:] = 0.5*(trhessian - np.sqrt(D))
        eigh1 = eigvallam.min(0)
        
        etemp = np.copy(eigh1)
        etemp = etemp-etemp.min()
        etemp = etemp/etemp.max()
        etemp = np.uint8(etemp*255)
        
        return(etemp)

    def clearBorder(img):
        h2 = np.size(img,0)
        h1 = np.size(img,1)
        imBorder = np.concatenate((img[0], img[:,0],img[:,h1-1],img[h2-1]))
        borderBlobs = np.unique(imBorder)
        borderBlobs = borderBlobs[borderBlobs>0]
        for blob in borderBlobs:
            img[img==blob] = 0

        return(img)
        
    def worker(self, t):
		if t is None:
			return
			
		poeInstance = poe.PerObjectEllipsefit()
		firstI = self.imagesPerThread * t
		lastI = min(self.imagesPerThread * (t+1), self.nFiles)
	
		for i in range(firstI, lastI):
			filename = self.files[i]
			inputImageName = self.InputDir + filename
			base, ext = filename.rsplit('.',1)
			outputImage = self.OutputDir + base + "_resEF." + ext
			orimg = cv2.imread(inputImageName,-1)
			inputImage = self.enhanceContrast(orimg)
			pixel_data = np.asarray(inputImage)                
				
			if not self.brightObjects:
				maxp = np.amax(pixel_data)
				pixel_data[:] = [maxp - x for x in pixel_data]
			if self.resample:
				maxp = np.amax(pixel_data)
				if maxp > self.resampleMax:
					scaleFactor = self.resampleMax/float(maxp)
					pixel_data[:] = [x * scaleFactor for x in pixel_data]
					pixel_data = pixel_data.astype(int)

			#Extract objects
			res = poeInstance.run(pixel_data, self.area_interval_min, self.area_interval_max, self.ellipsefit_thr, self.major_interval_min, self.major_interval_max, self.minor_interval_min, self.minor_interval_max, self.major_minor_ratio_interval_min, self.major_minor_ratio_interval_max, self.thresholdwindow)

			#Fill holes
			res = nd.binary_fill_holes(res).astype(int)

			#Label objects
			(res, nObj) = nd.measurements.label(res)

			#Discard border cells
			if self.discardBorderCells:
				res = clearBorder(res)

			#Save image
			cv2.imwrite(outputImage,np.uint16(res))
