# Project: pipeline v0.1
# Author: Praneeth Karempudi
# Email: praneeth.karempudi@gmail.com

##############################################################################
# File containing classes for helping build Unet
# consolidating
# 1. double conv - batchnorm - relu 
# 2. concatenation and upconv

import torch
import torch.nn as nn
import torch.nn.functional as F

class double_conv(nn.Module):
    '''
    Combining conv1- batch - relu - conv batch relu parts of UNet
    
    Features: No change in size of the image (maintained using padding)
    '''
    
    def __init__(self, input_channels, output_channels):
        ''' Just initializes the conv - batch - relu layers twice 
            with no reduction in image size (don't change padding , stride)
        '''
        super(double_conv, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(input_channels, output_channels, 3, stride = 1, padding = 1),
            nn.BatchNorm2d(output_channels),
            nn.ReLU(inplace = True),
            nn.Conv2d(output_channels, output_channels, 3, stride = 1, padding =1),
            nn.ReLU(inplace = True)
        )

    def forward(self, x):
        x = self.conv(x)
        return x



class up_conv(nn.Module):
    '''
    UpConv + concatenation of features during downscaling
    '''

    def __init__(self, input_channels, output_channels, model_expansion):
        super(up_conv, self).__init__()

        if model_expansion == 'upsample':
            self.up = nn.Upsample(scale_factor=2, mode='bilinear', align_corners=True)
        elif model_expansion == 'transpose_conv':
            self.up = nn.ConvTranspose2d(input_channels//2, input_channels//2, 2, stride=2)

        self.conv = double_conv(input_channels, output_channels)

    def forward(self, from_bottom, from_side):

        x1 = self.up(from_bottom)

        x2 = torch.cat([from_side, x1], dim = 1)

        x2 = self.conv(x2)
 
        return x2

