function diffCoeff = getMSD(outputFolders,nFitPoints,timeStep,pixelSize,minTrajLength,splitLength,throughZero,nBins,debugFlag)
% This function computes apparent diffusion coefficients for trajectories
% and creates a histogram plot for diffusion coefficients.
%
% Input:
%   outputFolders - either cell array with several paths or string with 
%                   a path to folder that contain the pipeline results
%   nFitPoints - number of points to fit to get the slope.
%   timeStep - time between frames [in ms], typical value is 20.
%   pixelSize - size of the pixels [in um], typical value is 0.08
%
% Optional parameters:
%   minTrajLength - minimum trajectory length, discard trajectories shorter
%                   than minTrajLength.  The default value is 0.
%   splitLength - Chop up trajectories to sub-trajectories with length
%                 <= splitLength, i.e 13 = 5+5+3 for splitLength=5. 
%                 Needed if particle experience different diffusional 
%                 states within one trajectory. If [], no splitting.
%                 The default value is [].
%   throughZero - if 1, force that localization error is zero, i.e., 
%                 fit two first points such that fit goes through (0,0).
%                 The default value is 0.
%   nBins - number of bins for histogram plot. The default value
%           is determined by function histogram.
%   debugFlag - if 1, plot MSD fit for individual trajectories.
%               The default value is 0.
%
% Output:
%   diffCoeff - list of diffusion coefficients
%
% See also: getMSDSingleTraj
%
% Author : Vladimir Curic, 2015-01-05
%
% 2020-04-08, SZ. Adapted the code to the current version of image analysis
% pipeline. Deal with nan-coordinates. Added new optional parameters.
%% Parse paramerers
if ~iscell(outputFolders)
    outputFolders = {outputFolders};
end

if nargin<5 || isempty(minTrajLength)
    minTrajLength = 0;
end
    
if nargin<6
    splitLength = [];
end
if ~isempty(splitLength) && splitLength < nFitPoints+1
    error('Choose such parameters that splitLength >= nFitPoints+1');
end

if nargin<7
    throughZero = 0;
end

if nargin<8
    nBins = [];
end

if nargin<9
    debugFlag = 0;
end
if debugFlag
    figure
    ax=gca;
else
    ax=[];
end
%% Compute MSD and diffusion coefficients

diffCoeff = [];
for ii=1:length(outputFolders)
    outputFolder = outputFolders{ii};
    expInfoObj = loadExpInfo(outputFolder);
    positionList = expInfoObj.getPositionList();
    
    for posIdx = 1:length(positionList)
        posName = positionList{posIdx};
        cellMatFile = expInfoObj.getMCellMatPath(posName);
        mCells = Cell.MCell.loadMCells(cellMatFile);
        if isempty(mCells)
            continue;
        end
        trajList = [mCells.particles];
        trajLengths = [trajList.numberOfFrame];
        trajList = trajList(trajLengths >= minTrajLength);

        for trajIdx = 1:length(trajList)
            coord = trajList(trajIdx).trajImageCoord * pixelSize;
            nData = size(coord,1);
            % Chop trajectory
            if isempty(splitLength)
                curSplitLength = nData;
            else
                curSplitLength = splitLength;
            end
            nSegments = ceil(nData/curSplitLength);
            if mod(nData,curSplitLength)>0 && ...
               mod(nData,curSplitLength)< nFitPoints +1
                nSegments = nSegments-1;
            end
            for ll=1:nSegments
                startFrame = (ll-1)*curSplitLength+1;
                endFrame = min(nData,ll*curSplitLength);
                curCoord = coord(startFrame:endFrame,:);
                if debugFlag
                    curDiffCoeff = getMSDSingleTraj(ax,curCoord,nFitPoints,timeStep,throughZero);
                    pause;
                else
                    curDiffCoeff = getMSDSingleTraj(curCoord,nFitPoints,timeStep,throughZero);
                end
                diffCoeff = [diffCoeff curDiffCoeff];
            end
        end
    end
end
%% Make histogram plot
if ~isempty(nBins)
    figure, histogram(diffCoeff,nBins)
else
    figure, histogram(diffCoeff);
end
xlabel('Apparent diffusion coefficient [\mum^2s^{-1}]');
ylabel('Number of trajectories')