function getMSDFit(finalTraj,timeStep,pixelSize,minTrajLength,maxTrajLength,frameSpacing)
% This function computes MSD for all trajectories and estimates MSD and
% plots the result
%
% Input:
%   finalTraj - cell array of all trajectories
%   timeStep - time between frames [in ms]
%   pixelSize - size of the pixels [in um] (typical value is 0.08)
%   minTrajLength - minimal allowed length of trajectory
%   maxTrajLength - maximal allowed lenght of trajectory
%   frameSpacing - use every frameSpacing trajectory point. Optional,
%                  the default value is 1, i.e. use all points
%
% Output: 
%    Plot for MSD

% if we consider every frame
if nargin < 6
    frameSpacing = 1;
end

% Convert time step to ms
timeStep = timeStep * frameSpacing / 1000;

% Make MSD plot(one for all trajectories)
sqDisplTotCell = cell(2, maxTrajLength-1);

% Run through all trajectories
for ii = 1:length(finalTraj)
    traj = finalTraj{ii}(1:frameSpacing:end, :) * pixelSize;
    
    if size(traj, 1)>=minTrajLength && size(traj, 1)<=maxTrajLength
        % Calculate all squared displacements
        for dt = 1:size(traj, 1)-1
            
            dCoords = traj(1+dt:end,1:2) - traj(1:end-dt,1:2);
            sqDispl = sum(dCoords.^2,2); % dx^2+dy^2
            
            sqDisplTotCell{1, dt} = [sqDisplTotCell{1, dt}; sqDispl];

            sqDisplX = dCoords(:, 1).^2;
            sqDisplTotCell{2, dt} = [sqDisplTotCell{2, dt}; sqDisplX];   
        end   
    end
end

% Make one general MSD list
msdTot = [length(sqDisplTotCell), 4];
% msdTotX = [length(sqDisplTotCell), 4];

for dt = 1:size(sqDisplTotCell,2)
    msdTot(dt,1) = nanmean(sqDisplTotCell{1, dt}); % average
    msdTot(dt,2) = nanstd(sqDisplTotCell{1, dt}); % std
    msdTot(dt,3) = length(sqDisplTotCell{1, dt}); % n
    
%     msdTotX(dt,1) = nanmean(sqDisplTotCell{2, dt}); % average
%     msdTotX(dt,2) = nanstd(sqDisplTotCell{2, dt}); % std
%     msdTotX(dt,3) = length(sqDisplTotCell{2, dt}); % n
end

% Calculate diffusion constant from the first 2 points
diffArr = msdTot(1:2, 1)';
xArr = [1, 2];
[p, ~] = polyfit(xArr.*timeStep, diffArr, 1);
diffOffset = p(2);
diffCoeff = p(1)/4;

% diffArr = msdTotX(1:2, 1)';
% xArr = [1, 2];
% [pX, ~] = polyfit(xArr.*timeStep, diffArr, 1);
% diffOffsetX = pX(2);
% diffCoeffX = pX(1)/2;

% Plot the MSD if desired

color = 'r';
fighand1 = figure();
set(fighand1, 'Name', 'MSD plot','NumberTitle','off');
hold on
xArr = find(msdTot(:, 1));
fplot(@(x)x*p(1)+p(2), [0, 5*timeStep], ['--' color]);
plot(xArr.*timeStep, msdTot(:, 1), ['-' color]);
errorbar(xArr.*timeStep, msdTot(:, 1), msdTot(:, 2)./(sqrt(msdTot(:, 3))-1), ['+' color]);
title('MSD XY');
xlabel('Time [s]');
ylabel('Mean square displacement [\mum^2]');
% xlim([0, 20*timeStep]);
text(0.1, 0.9, ['Diff. Coeff = ', num2str(diffCoeff), '\mum^2/s'], 'Units', 'normalized');
text(0.1, 0.85, ['Offset = ', num2str(diffOffset), '\mum^2'], 'Units', 'normalized');
hold off


% fighand2 = figure(667);
% set(fighand2, 'Name', 'MSD plot','NumberTitle','off');
% hold on
% xArr = find(msdTotX(:, 1));
% fplot(strcat('x*', num2str(pX(1)), '+', num2str(pX(2))), [0, 5*timeStep], ['--' color]);
% plot(xArr.*timeStep, msdTotX(:, 1), ['-' color]);
% errorbar(xArr.*timeStep, msdTotX(:, 1)./(1e6), msdTotX(:, 2)./(sqrt(msdTotX(:, 3))-1), ['+' color]);
% title('MSD X');
% xlabel('Time [s]');
% ylabel('Mean square displacement [\mum^2]');
% xlim([0, 20*timeStep]);
% text(0.1, 0.9, ['Diff. Coeff = ', num2str(diffCoeffX), '\mum^2/s'], 'Units', 'normalized');
% text(0.1, 0.85, ['Offset = ', num2str(diffOffsetX), '\mum^2'], 'Units', 'normalized');
% hold off

end
