function [diffCoeff, diffOffset] = getMSDSingleTraj(varargin)
% [DiffCoeff, DiffOffset] = getMSDSingleTraj(coord,nFitPoints)
% computes apparent diffusion coefficient DiffCoeff and offset DiffOffset 
% for single trajectory by using nFitPoints points to fit to get the slope.
%
% getMSDSingleTraj(AX,___) plots the errorbar of MSD and fitted line 
% into the axes with handle AX.
%
% Input:
%   coord - M x D matrix of M trajectory coordinates [in um] in D-dim space
%   nFitPoints - number of points to fit to get the slope
% Optional parameters:
%   timeStep - time between frames [in ms], the default value is 20.
%   throughZero - if 1, force that localization error is zero, i.e., 
%                 fit two first points such that fit goes through (0,0).
%                 The default value is 0.
%   debugFlag - if 1, make plot with MSD errorbar and fitted line. 
%               If no axes handle is provided, make plot in a new figure.
%               By default, 1 if axes handle is provided, 0 otherwise.
%
% Output:
%   diffCoeff - apparent diffusion coefficient
%   diffOffset - localization error
%% Parse parameters
if length(varargin)>1 && isa(varargin{1},'matlab.graphics.axis.Axes')
    ax = varargin{1};
    defDebugFlag=1;
    varargin = varargin(2:end);
else
    ax = [];
    defDebugFlag=0;
end
ip = inputParser();
ip.addRequired('coord',@(x) isnumeric(x) && length(size(x))==2);
ip.addRequired('nFitPoints',@(x) isnumeric(x) && isscalar(x));
ip.addOptional('timeStep',20,@(x) isnumeric(x) && isscalar(x));
ip.addOptional('throughZero',0,@(x) islogical(x) || x==0 || x==1);
ip.addOptional('debugFlag',defDebugFlag,@(x) islogical(x) || x==0 || x==1);
ip.parse(varargin{:});
coord = ip.Results.coord;
nFitPoints = ip.Results.nFitPoints;
timeStep = ip.Results.timeStep / 1000; % convert to seconds
throughZero = ip.Results.throughZero;
debugFlag = ip.Results.debugFlag;

if debugFlag
    if isempty(ax)
        figure
        ax = gca;
    end
end
%% Compute diffusion coefficient
if debugFlag
    trajLen = size(coord,1); % compute all msd
else
    trajLen = nFitPoints+1; % compute only msd used for fitting
end
dim = size(coord,2);
msd = zeros(1,trajLen-1);
stderr = zeros(1,trajLen-1);
for dt = 1:trajLen-1
    dCoord = coord(1+dt:end,:) - coord(1:end-dt,:);
    sqDispl = sum(dCoord.^2,2); % dx^2+dy^2+dz^2
    msd(dt) = nanmean(sqDispl);
    stderr(dt) = nanstd(sqDispl) / sqrt(sum(~isnan(sqDispl)));
end
if any(isnan(msd(1:nFitPoints)))
    diffCoeff = [];
    diffOffset = [];
    return
end

% Calculate diffusion coefficient D from r^2 = 2*dim*D*t, where r^2 is msd
if throughZero
    p = [(1:nFitPoints)'\msd(1:nFitPoints)' 0];
else
    p = polyfit(1:nFitPoints,msd(1:nFitPoints),1);
end
diffCoeff = p(1)/(2*dim*timeStep);
diffOffset = p(2);
%% Debugging
if debugFlag
    cla(ax);
    hold(ax,'on')
    plot(ax,(1:nFitPoints)*timeStep,msd(1:nFitPoints),'b-','Marker','*');
    ind = find(~isnan(msd(nFitPoints:end)))+nFitPoints-1;
    if ~isempty(ind)
        plot(ax,ind*timeStep,msd(ind),'b--','Marker','*');
    end
    msd_f = p(1)*(1:trajLen-1)+p(2);
    plot(ax,(1:trajLen-1)*timeStep,msd_f,'r--');
    errorbar(ax,(1:trajLen-1)*timeStep,msd,stderr,'b+');
    xlim([0.5 trajLen-0.5]*timeStep);
    title('MSD XY');
    xlabel('Time [s]');
    ylabel('Mean square displacement [\mum^2]');
    text(0.1, 0.9, ['Diff. Coeff = ', num2str(diffCoeff), '\mum^2/s'], 'Units', 'normalized');
    text(0.1, 0.85, ['Offset = ', num2str(diffOffset), '\mum^2'], 'Units', 'normalized');
    hold(ax,'off')
end