function [growthRates, fitRMSE] = computeGrowthRates(cellArray, framesPerMin)
% Compute the growth rate and corresponding root-mean square error for each
% cell in a cell array. Growth rates that cannot be computed are set to NaN
%
% Input:
%       cellArray  : input array of aCell objects.
%
% Output: 
%       growthRates: array of corresponding growth rates for each aCell.
%       fitRMSE    : array of corresponding root-mean squared error for
%                    each aCell.

if nargin < 2
    framesPerMin = 1;
end

parfor cellIdx = 1:length(cellArray)
    currCell = cellArray(cellIdx);
    currLen = currCell.cellLength;
    cellIdx
    
    if isempty(currLen) || length(currLen) < 30
        growthRates(cellIdx) = NaN;
        fitRMSE(cellIdx) = NaN;
        continue
    end

    timePoints = 1:length(currLen);
    timePoints = timePoints/framesPerMin;
    
    currLenLog=log(currLen);
    currLenLog=currLenLog/currLenLog(1);

    X=[ones(size(timePoints(:))) timePoints(:)];

    B=((X'*X)^-1)*X'*currLenLog(:);

    model='L0.*(exp(mu*t));';
    fitType = fittype( model, 'indep', 't', 'depend', 'y' );
    
    L0start=currLen(1);
    muStart=B(2);
    
    opts = fitoptions( fitType );
    opts.Display = 'Off';
    opts.MaxFunEvals = 1000;
    opts.MaxIter = 800;
    opts.TolFun = 1e-008;
    opts.TolX = 1e-008;
    opts.Weights = zeros(1,0);
    opts.StartPoint = [L0start,muStart]; %Fitting Initial Guess
    [Expfitresult, error] = fit(timePoints(:),currLen(:), fitType, opts);

    growthRates(cellIdx) = Expfitresult.mu;
    fitRMSE(cellIdx) = error.rmse;

    
end

end