function [intCoordsShortAxis, intCoordsLongAxis, dotCellAreas, dotCellLengths, ...
    dotCellWidths, cellCounts, dotCellFrames, cellFluoFrames] = getDotLocation(mCell,orientFlag,fluoIndices)
% This function extracts information about dot locations and is used for
% making fork plots. 
%
% Input:
%   mCell - MCell object
%   orientFlag - logical, if 1, reorient cell poles along the trap and
%                recompute internal long axis coordinates
%   fluoIndices - array of phase contrast frames corresponding to 
%                 fluorescence images, optional.
%
% Output:
%   intCoordsShortAxis - array with dot internal coordinates along 
%                        "short axis"
%   intCoordsLongAxis - array with dot internal coordinates along 
%                        "long axis"
%   dotCellAreas - array with cell areas in the dot detection frames
%   dotCellLengths - array with cell lengths in the dot detection frames
%   dotCellWidths - array with cell widths in the dot detection frames
%   cellCounts - array with cell counts (number of dots in cell in the 
%                correspodning dot detection frame)
%   dotCellFrames - array with dot detection frames, i.e. 
%                   indices of cellFluoFrames, optional. Requires
%                   fluoIndices.
%   cellFluoFrames - array with cell detection frames that have
%                    a corresponding fluo frame. Frame=1 means the cell
%                    birth frame.

if nargin<2
    orientFlag=0;
end

particles = mCell.particles;

cellDetectionFrames = [particles.cellDetectionFrames]-mCell.birthFrame+1;

cellAreas = mCell.areas(cellDetectionFrames);
cellLengths = mCell.lengths(cellDetectionFrames);
cellWidths = mCell.widths(cellDetectionFrames);
badSegs =  mCell.badSegmentations(cellDetectionFrames);

intCoordsShortAxis = zeros(size(cellDetectionFrames));
intCoordsLongAxis = zeros(size(cellDetectionFrames));
dotCellAreas  = zeros(size(cellDetectionFrames));
dotCellLengths = zeros(size(cellDetectionFrames));
dotCellWidths = zeros(size(cellDetectionFrames));
cellCounts = zeros(size(cellDetectionFrames));
dotCellFrames = zeros(size(cellDetectionFrames));

if nargin==3  && nargout>=7
    ind = fluoIndices>=mCell.birthFrame & fluoIndices<=mCell.lastFrame;
    cellFluoFrames = fluoIndices(ind)-mCell.birthFrame+1;
end

[~, ~, uidx] = unique(cellDetectionFrames);
uvalCount = accumarray(uidx,1);
frameCounts = uvalCount(uidx);

counter = 0;
for i = 1:length(particles)
    if badSegs(i) == 0 && ~any(isnan(particles(i).trajInternalCoord))
        counter = counter + 1;
        if ~isempty(particles(i).trajInternalCoord)
            cellCounts(counter) = frameCounts(i);
            intCoordsShortAxis(counter) = particles(i).trajInternalCoord(2);
            intCoordsLongAxis(counter) = particles(i).trajInternalCoord(1);
        else
            cellCounts(counter) = 1;
            intCoordsShortAxis(counter) = NaN;
            intCoordsLongAxis(counter) = NaN;
        end
        dotCellAreas(counter) = cellAreas(i);
        dotCellLengths(counter) = cellLengths(i);
        dotCellWidths(counter) = cellWidths(i);
        if nargin>=2 && nargout>=7
            dotCellFrames(counter) = find(cellFluoFrames == cellDetectionFrames(i),1);
        end
    end
end

if orientFlag
    counter = 0;
    cellPoles = mCell.poles;
    for i = 1:length(particles)
        if badSegs(i) == 0 && ~any(isnan(particles(i).trajInternalCoord))
            counter = counter + 1;
            curCellPoles = cellPoles{cellDetectionFrames(i)}; 
            if ~isempty(particles(i).trajInternalCoord)
                if curCellPoles(1,1) > curCellPoles(2,1)
                    intCoordsLongAxis(counter) = 1 - intCoordsLongAxis(counter);
                end
            end
        end
    end
end

intCoordsShortAxis = intCoordsShortAxis(1:counter);
intCoordsLongAxis = intCoordsLongAxis(1:counter);
dotCellAreas  = dotCellAreas(1:counter);
dotCellLengths = dotCellLengths(1:counter);
dotCellWidths = dotCellWidths(1:counter);
cellCounts = cellCounts(1:counter);
if nargin==3 && nargout>=7
    dotCellFrames = dotCellFrames(1:counter);
end