function [o_sa,o_la,o_ca,o_cl,o_cw,o_cf,df,o_a,o_l,o_a_f,o_l_f,dt,offset] = getJointTrack(mCells,lineage,fluorIndices,useParent)
% This function extracts dot coordinates and cell measurements from a cell
% lineage. The track can optionally include the parent cell of the first
% cell in the lineage.
%
% Input:
%   mCells - array of MCell objects
%   lineage - array with cell id, index of its daugther cell, index of its
%             daughters' daughter cell etc. The indices are either 1 or 2.
%   fluorIndices - array of fluorescence frames
%   useParent - logical. If 1, include the parent cell into the track. 
%               Optional, the default value is 1.
%
% Output:
%   o_sa - array with joint dot internal coordinates along "short axis"
%   o_la - array with joint dot internal coordinates along "long axis"
%   o_ca - array with joint cell areas in the dot detection frames
%   o_cl - array with joint cell lengths in the dot detection frames
%   o_cw - array with joint cell widths in the dot detection frames
%   o_cf - array with joint dot detection frames
%   df - array with division frames
%   o_a - array with joint cell area in cell lineage in fluo frames
%   o_l - array with joint cell length in cell lineage in fluo frames
%   o_a_f - array with joint fitted cell area in cell lineage in fluo frames
%   o_l - array with joint fitted cell length in cell lineage in fluo frames
%   dt - array with detection times of fluo frames
%   offset - array with length offsets used to compute o_la.
%   useParent - logical. If 1, include the parent cell of the lineage to
%               the track. Optional, the default value is 1.
%
%   See also: getDotLocation
%
% Author: Spartak Zikrin, 2020-04-20

    % Parse parameters
    if nargin<4
        useParent = true;
    end

    c = mCells([mCells.id]==lineage(1));
    N = length(lineage); % number of cell generations
    
    if useParent
        % Initialize with the parent of the first cell in the lineage
        pc = c.parent;
        if isempty(pc)
            error('The cell has no parent cell');
        end
    else
        % Initialize with the first cell in the lineage
        pc = c;
        N = N-1;
        lineage = lineage(2:end);
    end
    [p_sa, p_la, p_ca, p_cl, p_cw, ~, p_cf,p_cff] = getDotLocation(pc,0,fluorIndices);
    [p_areas,p_lengths,p_areas_f,p_lengths_f] = getCellStats(pc);
    p_det_times = pc.detectionTimes';
    
    % Initialize data in the joint track
    o_la = (p_la-0.5).*p_cl;
    o_sa = 0.5.*p_sa.*p_cw;
    o_ca = p_ca;
    o_cw = p_cw;
    o_cl = p_cl; 
    o_cf = p_cf;
    df = [p_cf(end) zeros(1,N-1)];
    o_a = p_areas;
    o_l = p_lengths;
    o_a_f = p_areas_f;
    o_l_f = p_lengths_f;
    fluoFrames = p_cff;
    dt = p_det_times;    
    offset = zeros(1,N+1);
    
    % temp variables for computing joint dot coordinates
    lastFrame = pc.lifeTime;
    p_last_area_f = p_areas_f(end);
    p_last_length_f = p_lengths_f(end);
    orientFlag = 1;
    
    if ~useParent
        % Move to the second cell in the lineage
        c = c.descendants(lineage(1));
    end
    
    % Main loop
    for i=1:N
        
        % Get cell info in current  generation
        [sa, la, ca, cl, cw, ~, cf, cff] = getDotLocation(c,0,fluorIndices);
        [areas,lengths,areas_f,lengths_f] = getCellStats(c);
        
        % Get sister cell info in current  generation
        if c.id == c.parent.descendants(1).id
            c_2 = c.parent.descendants(2);
        else
            c_2 = c.parent.descendants(1);
        end
        [~,~,areas_f_2,lengths_f_2] = getCellStats(c_2);
        
        % Compute joint internal dot coordinates
        % orientFlag = 1 if cell has the same orientation as
        % the very first cell
        orientFlag = orientFlag * isOldPole(c.parent, c);
        if orientFlag>0
            offset(i+1) = offset(i) - cl(1)/2;
        else
            offset(i+1) = offset(i) + cl(1)/2;
            la = 1-la;
            sa = -sa;
        end
        o_la = [o_la (la-0.5).*cl+offset(i+1)];
        o_sa = [o_sa 0.5.*sa.*cw];
        
        % Update cell measurements in dot detection frames
        o_ca = [o_ca ca];
        o_cw = [o_cw cw];
        o_cl = [o_cl cl]; 
        o_cf = [o_cf cf+o_cf(end)];
        
        % Update cell lineage area and length
        o_a = [o_a 2^i*areas];
        o_l = [o_l 2^i*lengths];
        
        % Compute fitted area and length in dot detection frames
        frac_a = areas_f(1) /(areas_f(1) + areas_f_2(1));
        o_a_f = [o_a_f, o_a_f(end) + (2^i)*(areas_f-frac_a*p_last_area_f)];
        frac_l = lengths_f(1) /(lengths_f(1) + lengths_f_2(1));
        o_l_f = [o_l_f, o_l_f(end) + (2^i)*(lengths_f-frac_l*p_last_length_f)];
        
        % Update detection times
        dt = [dt, c.detectionTimes'];
        
        % Update fluo frames in cell lineage 
        fluoFrames = [fluoFrames; cff+lastFrame];
        
        if i<N % if not the last cell in the lineage
            % Update division frames
            df(i+1) = df(i)+cf(end);
            % Update temp variables
            lastFrame = lastFrame + c.lifeTime;
            p_last_area_f = areas_f(end);
            p_last_length_f = lengths_f(end);
            % Update cell to the next cell in the lineage
            c = c.descendants(lineage(i+1));
        end
    end % main loop
    
    % Extract cell measurements in fluo frames.
    o_a = o_a(fluoFrames);
    o_l = o_l(fluoFrames);
    o_a_f = o_a_f(fluoFrames);
    o_l_f = o_l_f(fluoFrames);
    dt = dt(fluoFrames);

end

% -----------------------
% Utility functions
function [areas,lengths,areas_f,lengths_f] = getCellStats(c)
    areas = c.areas;
    areas(c.badSegmentations ~= 0) = NaN;
    lengths = c.lengths;
    lengths(c.badSegmentations ~= 0) = NaN;
    det_times = c.detectionTimes;
    non_nans = ~isnan(areas);
    fit_a = fit(det_times(non_nans) - det_times(1), areas(non_nans)', 'exp1');
    areas_f = fit_a(det_times - det_times(1))';
    non_nans = ~isnan(lengths);
    fit_l = fit(det_times(non_nans) - det_times(1), lengths(non_nans)', 'exp1');
    lengths_f = fit_l(det_times - det_times(1))';
end

function is_old_pole = isOldPole(mCellParent, mCellDaugther)
    p_poles = mCellParent.poles{end};
    p_old_pole = p_poles(1,:);
    p_new_pole = p_poles(2,:);

    d_poles = mCellDaugther.poles{1};
    d_old_pole = d_poles(1,:);

    if pdist([p_old_pole; d_old_pole]) < pdist([p_new_pole; d_old_pole])
        is_old_pole = 1;
    else
        is_old_pole = -1;
    end
end