function paramStruct = parseParameterFile(fileName, pipelinePath)
% paramStruct = parseParameterFile(fileName, pipelinePath)
%
% Parses a preferences file. parseParameterFile ignores all empty lines, 
% and all lines with comments which start with a '%' character.
%
% Input:
%   fileName    - character string containing the name of a
%                 preferences file. If this file is not on
%                 the path, then the name must also contain
%                 the full path to that file.
%   pipelinePath - path to the directory with ImAnalysis pipeline
%
% Output:
%   paramStruct - structure containing one field for each
%                 keyword found in the keyword list

% Last modified:
% SZ, 2018-04-23. Added optional emptyChannel parameters
% SZ, 2018-05-04. Removed parameter useSegMask. We must always it.
% SZ, 2018-08-09. Parse '%' as '\%'.
% SZ, 2018-08-23. New parameters for the wavelet dot detection algorithm -
% minSpotArea and maxAxesRatio.
% SZ, 2019-09-02. Use negative values in positionRange to skip positions.
% SZ, 2019-03-15. New parameter - parforPositions, a flag to use parfor
% loops between the positions. Also, save current matlab version in field
% matlabVersion
% SZ, 2019-03-28. Added new parameters for cell segmentation:
% minEllipseRatio, maxEllipseRatio with the default values 1.5 and 100.
% SZ, 2019-11-25. Removed parameter nExcludedRows, added saveCellBoundaries
% SZ, 2019-12-10. New parameter - pranetModelPath. To use pranet, set
% parameter segmentationAlg to "pranet". Renamed two parameters - 
% fitEllipseThresh -> segmentationThreshold
% segThreshWindow -> segmentationThresholdWindow
% Handle empty spaces in paths.
%% Create cell array of parameter names and their default values

keywords = {'phaseChannelNames', 'bfChannelNames', 'fluoChannelNames', ...
            'rangePositions', 'rangePhaseImages', 'rangeFluoStack', ...
            'parNumWorkers', 'parforPositions'};
        
localCluster = parcluster('local');
defaultParNumWorkers = localCluster.NumWorkers;

defaults = {('phase'), {}, {}, ...
            [], [], [], ...,
            defaultParNumWorkers, 0};
        
keywordsSections = ...
           {'doPreprocessing', 'doSegmentation', 'doBlobProcessing', ...
            'doCellMeasurements', 'doDotDetection'};
        
defaultsSections = ...
           {1, 1, 1, ...
            1, 1 };
      
keywordsPreprocessing = ...
           {'angularDisplacement', 'transMatPath', 'transMat', ...
            'flipFlags', 'roiInPhase', 'roiInFluo', 'barcodeROI', ...
            'registrationMaxOffset', 'blackShiftedBF', ...
            'isMothermachine', 'nGrowthChannels', 'emptyChannel', ...
            'channelWidth', 'minChannelDist', 'channelOffset'};

defaultsPreprocessing = ...
           {0, '', [], ...
            [], [], [], [], ...
            50, 1, ...
            0, [], [], ...
            [], [], 3};
       
keywordsSegmentation = ...
           {'segmentationAlg',...
            'minWidth', 'maxWidth', 'minLength', 'maxLength', ...
            'minCellArea', 'maxCellArea', ...
            'minEllipseRatio', 'maxEllipseRatio', 'segmentationThreshold', ...
            'segmentationThresholdWindow', 'gaussKernSigma', ...
            'discardCellsOnBorder', 'pranetModelPath',};
                
defaultsSegmentation = ...
           {'POE',...
            2, 25, 20, 145, ...
            70, 1500000, ...
            1.5, 100, [], ...
            [], 2, ...
            1, '',...
};

keywordsCellTracking = ...
           {'trackCells', ...
            'pCnt0', 'pCnt1', 'pCnt2', 'pCntExtrap',  ...
            'pDeath', 'deathShift', ...
            'pAppear', 'pDisappear', 'minScore', ...            
            'migJaccardMin', 'migJaccardMax', ...
            'migJaccardExp', 'migJaccardPostSplitMin', ...
            'splitJaccardMin', 'splitJaccardMax', 'splitJaccardPreCond', ...
            'JacMigCutOff', 'shortTrackCutOff', ...
            'maxGrowthRate', 'centerOfMassMovementCutOff', ...
            'deleteBlobFiles','saveCellBoundaries'};
                  
defaultsCellTracking = ...
           {1, ...
            1E-6, 0.9, 0.01, 0, ...
            0, 2E-5, ...
            1E-10, 1E-10, -20, ...
            0.3, 0.99, ...
            1, 0.6, ...
            0.55, 0.99, 0.01, ...
            0.7, 10, ...
            1.3, 10, ...
            1, 1};

keywordsCellMeasurements = {'distanceThreshold'};

defaultsCellMeasurements = {6 };

keywordsDotDetection = ...
           {'dotDetectionAlg', 'dotTrackingAlg', 'doLSQrefineDots', ...
            'excludeCellsFluorBoundary', 'cellInclusionMargin', ...
            'maxDotsInCell', ...
            'noiseThreshold', 'minSpotArea', 'maxAxesRatio'...
            'radii', 'alpha', 'beta', 'radThreshold', ...
            'radSymThreshold', 'gaussKernRadFactor', ...
            'maxDist', 'minDotsInTraj', 'removeDuplicatedDots'};
                  
defaultsDotDetection = ...
           {'wavelet', '', 0, ...
            0, -1, ...
            inf, ...
            3.5, 0, inf, ...
            [1 2 3 4 5], 1, 0, 2, ...
            4, 1, ...
            inf, 5, 0};

keywordsUTrack = ...
           {'uWindow', 'uMergeSplit', 'uMinTrackLen', 'uBrownStdMult', ...
            'uMinSearchRadius', 'uMaxSearchRadius', 'uLinearMotion',...
            'uMinGapSearchRadius','uMaxGapSearchRadius',...
            'uAmpRatioLimit','uGapPenalty'};
                
defaultsUTrack = ...
           {1, 0, 2, 3, ...
            2, 5, 1, ...
            [], [], ...
            [0.7 4], 1.5};

keywordsLSQrefineDots = ...
           {'LSQrefineDots_ROIwidth','LSQrefineDots_drMax',...                          
            'LSQrefineDots_SminPx','LSQrefineDots_SmaxPx',...
            'LSQrefineDots_Nmin',...
            'EMgain','EMCCD_noiseStd','EMCCD_offsetName',...
            'pxQuadPoints',... % number of quadrature points per pixel
            ...% imaging wavelength, units: MAPlambda/fluoPxSize = fluo pixel units
            'MAPlambda', 'fluoPxSize', ...
            'NA', 'MAPprior'}; % Numerical Aperture, MAP prior parameters
            % the form of the prior parameters depend on dotRefinementAlg:
            % for EMCCDsymmetricMAP : [lnB0 lnBstd lnN0 lnNstd dS0], recommend dS0=1. 
            % for EMCCDasymmetricMAP: [lnB0 lnBstd lnN0 lnNstd dS0], recommend dS0=0.5. 
            % EMCCDsymmetricMLE, LSQsymmetric: not used          
               
defaultsLSQrefineDots = ...
           {7, 3, ...
            0.333, 3, ...
            -1, ...
            60.4, 18.4, 'EMCCDcalibration_offset.tif',...
            5, ...
            638, 80, ...
            1.4, [log(1) log(10) log(300) inf 1]}; 
            % lnB0, lnBstd, lnN0, lnNstd, dS0
            % lnB0 and lnN0 are also used as initial guess for optimization
            % with lnStd=inf, there is no prior on lnN 
            % EMgain: can be read off camera spec sheets (see lab
            % wiki), but should be converted to pixel units.
            % EMCCD_noiseStd : either from camera specs, or from
            % dark-count statistics.

% Redundant legacy parameters that should not be used
keywordsLegacy = {'ellipseFitThresh', 'segThreshWindow'};
defaultsLegacy = {[],[]};
                
keywords = [keywords  keywordsSections keywordsPreprocessing ...
            keywordsSegmentation keywordsCellTracking ...
            keywordsCellMeasurements keywordsDotDetection keywordsUTrack ...
            keywordsLSQrefineDots keywordsLegacy];

defaults = [defaults defaultsSections defaultsPreprocessing ...
            defaultsSegmentation defaultsCellTracking ...
            defaultsCellMeasurements defaultsDotDetection defaultsUTrack ...
            defaultsLSQrefineDots defaultsLegacy];
%% Parse the parameter file

% Create paramStruct with default values
paramStruct = [];
if length(defaults) ~= length(keywords)
    error(['Number of parameters does not match '...
           'the number of provided default values']);
end

for i=1:length(defaults)
    paramStruct.(keywords{i}) = defaults{i};
end

if nargin < 1
    return;
end

fid = fopen(fileName,'r');
if fid < 0
    error(['File not found: ', fileName])
end

isMissingParameter = true(size(keywords));
rec = fgetl(fid);
while ischar(rec)
    rec = strtrim(rec);
    if ~isempty(rec)
        % ignore empty lines
        % grab the first token in the record
        [tok, rest] = strtok(rec,'=');
        % is this a comment (first token == '%')
        if ~strcmp(tok(1),'%')
            % check to see if this was a keyword in the list
            [key, idx] = getKeyword(tok,keywords);
            if ~isempty(key)                
                % Remove the equality sign and possible trailing comments
                rest(strfind(rest,'=')) = [];
                if contains(rest,'\%')
                    rest = strrep(rest,'\%','%');
                else
                    rest(strfind(rest,'%'):end) = [];
                end
                rest = strtrim(rest);                
                if ~isempty(rest)
                    defaultVal = paramStruct.(key);
                    if ischar(defaultVal) || iscell(defaultVal)
                        % get rid of possible brackets
                        rest = regexprep(rest,'\(|\)|\[|\]|\{|\}','');
                        % separate all string words by exactly one white space
                        rest = strtrim(strrep(rest,',',' '));
                        rest = regexprep(rest,'( )+',' ');
                        ind = strfind(rest,' ');
                        if isempty(ind)
                            paramStruct.(key) = rest;
                        else
                            cellString = cell(1,numel(ind)+1);
                            cellString{1} = rest(1:ind(1)-1);
                            for ii=1:numel(ind)-1
                                cellString{ii+1} = rest(ind(ii)+1:ind(ii+1)-1);
                            end
                            cellString{end} = rest(ind(end)+1:end);
                            paramStruct.(key) = cellString;
                        end
                        isMissingParameter(idx) = false; 
                    else
                        val = str2num(rest);
                        if any(isnan(val(:))) || isempty(val)
                            disp(['Provided value for property ' key ...
                                ' is not numeric, using default value']);
                        else
                            paramStruct.(key) = val;
                            isMissingParameter(idx) = false; 
                        end
                    end                              
                end
            end
        end
    end
    rec=fgetl(fid);
end

if any(isMissingParameter)
    disp('The following parameters were not provided, defaults are used:');
    for idx = find(isMissingParameter)
        disp(keywords{idx});
    end
end

fclose(fid);

%% Set the revision number of the used pipeline
if ~exist('pipelinePath','var')
    pipelinePath = fileparts(which('processCells.m'));
end
[status, gitRevision] = ...
    system(['pushd >> /dev/null "' pipelinePath '" >> /dev/null && '...
      'TERM=linux && git log -1 --pretty=format:"%h %ad" --date=short && '...
      'popd >> /dev/null']);
if status
    error('There was an error regarding the git revision number: %s', gitRevision);        
end
gitRevision = gitRevision(2:19);
paramStruct.gitRevision = gitRevision;

%% Set the version of the currently used MATLAB
paramStruct.matlabVersion = version('-release');

%% Handle possible empty spaces in the paths
paramStruct.pranetModelPath = handleEmptySpaceInPath(paramStruct.pranetModelPath);
paramStruct.EMCCD_offsetName = handleEmptySpaceInPath(paramStruct.EMCCD_offsetName);
paramStruct.transMatPath = handleEmptySpaceInPath(paramStruct.transMatPath);

%% Remove redundant legacy parameters
if isempty(paramStruct.segmentationThreshold)
    paramStruct.segmentationThreshold = paramStruct.ellipseFitThresh;
end
if isempty(paramStruct.segmentationThresholdWindow)
    paramStruct.segmentationThresholdWindow = paramStruct.segThreshWindow;
end
paramStruct = rmfield(paramStruct,keywordsLegacy);
end
% =================== begin sub-functions here ===================
% ================================================================
function [property, idx] = getKeyword(property,validList)
% checks the valid list of properties supplied, and returns the
% one which matches, first looking for an exact match, then looking
% to see if the user has only supplied the first few characters of
% a valid property
%
% Input:
%   property  - string to be checked for validity 
%   validList - cell array containing valid property strings
%
% Output:
%   property  - string containing the full property name from validList
%   idx       - scalar index of the property identified in validList

    idx = strcmpi(strtrim(property), validList);

    % is it in the list of legal properties?
    if ~any(idx)
        disp([property,' is not a valid option'])
        property = '';
    elseif sum(idx) > 1
        disp([property,' matches more than one possible option.', ...
               ' be more specific.'])
        property = '';
    else
        property = validList{idx};
    end

end

function outtxt = handleEmptySpaceInPath(cellstring)
    % If the path string contains empty space it will be parsed as several
    % strings separated by the empty spaces. This function concatenates the
    % strings back into the original path string.
    if iscell(cellstring)
        outtxt = cellfun(@(x) sprintf([x ' ']), cellstring,'UniformOutput',false);
        outtxt = [outtxt{:}];
        outtxt = outtxt(1:end-1);
    else
        outtxt = cellstring;
    end
end