function fluoIntensities = computeCellFluorescence(expInfoObj, posName, fluoChanName, sigma)
% fluoIntensities = computeCellFluorescence(expInfoObj, posName, fluoChanName, sigma)
%
% function to compute the fluorescence of a cell throughout its lifetime.
% The results are stored directly in MCell objects.
%
% Input:
%   expInfoObj - expInfo object
%   posName - string, position name
%   fluoChanName - string, fluo channel name. Optional, by default the
%                  first fluo channel is used
%   sigma - if sigma>0, do background substraction by applying an
%           isotropic Gaussian filter with kernel sigma and computing
%           the difference between the original and the filtered images.
%           Optional, the default value is 0.
%
% Output:
%   fluoIntensities - cell array, each cell is a numeric array
%                     corresponding to a cell track. Each element is an
%                     average fluorescence of the cell in the
%                     corresponding track frame. NaN otherwise.

fluoChannels = expInfoObj.getChannelNames('fluo');

if nargin<3
    fluoChanName = fluoChannels{1};
end

if nargin<4
    sigma = 0;
end

fluoChanId = find(strcmp(fluoChanName, fluoChannels));
if isempty(fluoChanId)
    disp('Wrong fluorescence channel name');
    fluoIntensities = [];
    return
end

sourceDir = expInfoObj.getPathName('source');
outputDir = expInfoObj.getPathName('output');
mCellsMatFile = expInfoObj.getMCellMatPath(posName);
mCells = Cell.MCell.loadMCells(mCellsMatFile);
mFrames = Cell.MCell.mFrames;
fluoIndices = expInfoObj.getIndices(posName, fluoChanName);
phaseChanName = expInfoObj.getChannelNames('phase');
phaseRange = expInfoObj.getRange(posName, phaseChanName);
Ts = expInfoObj.getAllT(posName, fluoChanName);

pathFluo = fullfile(sourceDir,posName,fluoChanName);
listFluo = dir(fullfile(pathFluo,'*.tif*'));
listFluo = {listFluo.name};

phaseListIdxs = zeros(1,numel(fluoIndices));
for fluoListIdx = 1:length(fluoIndices)
    phaseIdx = fluoIndices(fluoListIdx);
    phaseListIdxs(fluoListIdx) = find(phaseRange == phaseIdx,1);
end
pathSeg = fullfile(outputDir,posName,'SegmentedPhase');
listSeg = dir(fullfile(pathSeg,'*.tif*'));
listSeg = {listSeg(phaseListIdxs).name};
outData = cell(1,length(mCells));
% parfor fluoListIdx = 1:length(fluoIndices)
for fluoListIdx = 1:length(fluoIndices)
    phaseListIdx = phaseListIdxs(fluoListIdx);
    fluoIm = imread(fullfile(pathFluo,listFluo{fluoListIdx}));
    segIm = imread(fullfile(pathSeg,listSeg{fluoListIdx}));
    if sigma>0 % do background subtraction
        filtIm = imgaussfilt(fluoIm,sigma);
        fluoIm = fluoIm-filtIm;
    end
    T = Ts(:,:,fluoListIdx);
    transFluoIm = imwarp(fluoIm, affine2d(T), ...
        'outputView', imref2d(size(segIm)), 'FillValues', 0);
    
    blob_props = regionprops(segIm, transFluoIm, 'MeanIntensity');
    blobs_intensities = [blob_props.MeanIntensity];
    blobs_in_frame = mFrames.getBlobsInFrame(phaseListIdx);
    outData{fluoListIdx} = blobs_intensities(blobs_in_frame);
end

fluoIntensities = cell(1,length(mCells));
for cIdx = 1:length(mCells)
    fluoIntensities{cIdx} = nan(1,mCells(cIdx).lifeTime);
end
for ii = 1:length(fluoIndices)
    phaseListIdx = phaseListIdxs(ii);
    blobs = mFrames.getBlobsInFrame(phaseListIdx);
    for blobIdx = 1:numel(blobs)
        blobLabel = blobs(blobIdx);
        blobCells = mFrames.getCellsInBlob(phaseListIdx,blobLabel);
        for cIdx = 1:length(blobCells)
            c = blobCells(cIdx);
            t = c.getIndex(phaseListIdx);
            fluoIntensities{c.id}(t) = outData{ii}(blobIdx);
        end
    end
end

for cIdx = 1:length(mCells)
    mCells(cIdx).fluoIntensities(fluoChanId,:) = fluoIntensities{cIdx};
end
Cell.MCell.saveMCells(mCellsMatFile, mCells);