function displayCellsOnFluo(expInfoObj, posName,varargin)
% displayCellsOnFluo(expInfoObj, posName, varargin)
%
% Plots fluorescence or bright-field images with cell boundaries,
% backbones, poles, and dots representing cell particles.
%
% Use keyboard arrows to navigate between frames:
%  - press right or left to show the next or previous frames, respectively.
%  - press up or down to move by 10 frames.
%
% Use active data cursor to show particle internal coordinates by clicking
% on dots. Click in the middle of the cell to display cell id.
%
% IMPORTANT - figure must be active and zoom/pan/data cursor etc. not
% active while pressing keyboard arrows to switch frames.
%
% Input:
%   expInfoObj  - expInfo object
%   posName     - string, position name
%
% List of optional input arguments:
%   Channel             - name of a fluorescence or bright-field channel.
%                         By default, the first fluorescence channel.
%   Frame               - phase frame. By default the first phase frame
%                         selected for analysis.
%   Cells               - array of MCell objects. If [], mCells are loaded
%                         using expInfoObj. By default [].
%   SelectedCells       - array with id of selected cells. By default [].
%   PlotCells           - if 1, plot cell boundaries. By default 1.
%   PlotBackbones       - if 1, plot cell backbones. By default 0.
%   PlotDots            - if 1, plot circles around detected particles.
%                         By default 0.
%   PlotPoles           - if 1, plot circular markers in the "old" cell
%                         poles. By default 0.
%   UniqueCellColors    - if 1, use unique color per cell track, otherwise
%                         use green and red (for bad segmentations) colors.
%                         By default 0.
%   Radius              - radius of circles around detected dots, in
%                         pixels. By default 0
%   CLIM                - array with two values between 0 and 1, defines
%                         imagesc scaling. Default is [0.01 0.99]
%
% See also displayCellsOnPhase, Visualizer
    %% Initialization
    phaseChanName = expInfoObj.getChannelNames('phase');
    fluoChanNames = expInfoObj.getChannelNames('fluo');
    phaseFrames = expInfoObj.getIndices(posName,phaseChanName);
    
    ip = inputParser;
    defaultFrame = phaseFrames(1);
    validLogical = @(x) islogical(x) || (x==0) || (x==1);
    ip.addRequired('Position',@(x) ischar(x));
    ip.addParameter('Channel',[],@(x) ischar(x) || isempty(x));
    ip.addParameter('Frame',defaultFrame);
    ip.addParameter('PlotBackbones',0,validLogical);
    ip.addParameter('PlotPoles',0,validLogical);
    ip.addParameter('PlotCells',1,validLogical);
    ip.addParameter('PlotDots',1,validLogical);
    ip.addParameter('ShowEvents',0,validLogical);
    ip.addParameter('UniqueCellColors',0,validLogical);
    ip.addParameter('CLIM',[0.01 0.99],@(x) isnumeric(x) && length(x)==2)
    ip.addParameter('SelectedCells',[],@(x) isnumeric(x))
    ip.addParameter('Cells',[]);
    ip.addParameter('Radius',0,@(x) isnumeric(x))
    
    ip.parse(posName,varargin{:});
    posName = ip.Results.Position;
    if ~isempty(ip.Results.Channel)
        fluoChanName = ip.Results.Channel;
    else
        fluoChanName = fluoChanNames{1};        
    end
    phaseFrame = ip.Results.Frame;
    plotBackbones = ip.Results.PlotBackbones;
    plotPoles = ip.Results.PlotPoles;
    plotCells = ip.Results.PlotCells;
    plotDots = ip.Results.PlotDots;
    uniqueCellColors = ip.Results.UniqueCellColors;
    clim = ip.Results.CLIM;
    selectedCells = ip.Results.SelectedCells;
    radius = ip.Results.Radius;
    if ~isempty(ip.Results.Cells)
        mCells = ip.Results.Cells;
    else
        loadFile = expInfoObj.getMCellMatPath(posName);
        mCells = Cell.MCell.loadMCells(loadFile);
    end
    
    if uniqueCellColors
        rng('default');
        rng(1);
        cellColorMap = rand(length(mCells),3);
    end
    
    if ~isempty(selectedCells)
        mCells = mCells(selectedCells);
    end
    
    mFrames = Cell.MCell.mFrames;
    fluoIndices = expInfoObj.getIndices(posName,fluoChanName);
    fluoRange = expInfoObj.getRange(posName,fluoChanName);
    sourceDir   = expInfoObj.getPathName('source');
    imageDir = fullfile(sourceDir, posName, fluoChanName);
    imageList = dir(fullfile(imageDir, '*.tif*'));
    imageList = imageList(fluoRange);
    
    % Use segmented images if cell boundaries are not stored
    if isempty([mCells.boundaries])
        outputDir = expInfoObj.getPathName('output');
        segDir = fullfile(outputDir, posName, 'SegmentedPhase');
        segList = dir(fullfile(segDir, '*.tif*'));
    else
        segList = [];
    end
    
    fluoROI = expInfoObj.getParameters('roiInFluo');
    if isempty(fluoROI)
        fluoImInfo = imfinfo(fullfile(imageDir,imageList(1).name));
        fluoROI = [1 1 fluoImInfo(1).Width fluoImInfo(1).Height];
    end
    
    if plotDots
        pList = getParticleList(mCells,fluoChanName);
    end
    
    Ts = expInfoObj.getAllT(posName,fluoChanName);
    %% Create figure and show the first frame
    fig = figure;
    ax = gca;
    imageListIdx = find(fluoIndices>=phaseFrame,1);
    plotFrame(imageListIdx);
    %% Main plotting function
    function plotFrame(imageListIdx)
        if imageListIdx < 1
            imageListIdx = length(imageList);
        elseif imageListIdx > length(imageList)
            imageListIdx = 1;
        end
        % Plot image
        if ~isempty(get(ax,'Children'))
            xlimits = xlim(ax);
            ylimits = ylim(ax);
        else
            xlimits = [];
        end
        im = tiffread(fullfile(imageDir, imageList(imageListIdx).name));
        the_clim = quantile(double(im(:)), clim);
        imagesc(ax,im);
        colormap(ax,'gray');
        axis(ax,'image');
        set(ax,'Ydir','reverse','YTick',[],'XTick',[],...
            'Clipping','off','Visible','off','CLIM',the_clim);
        if ~isempty(xlimits)
            xlim(ax,xlimits);
            ylim(ax,ylimits);
        end
        hold(ax,'on');
        
        phaseFrame = fluoIndices(imageListIdx);
        phaseListIdx = find(phaseFrames == phaseFrame,1);
        if ~isempty(segList)
            segIm = imread(fullfile(segDir,segList(phaseListIdx).name));
            [B,L] = bwboundaries(segIm>0,'noholes');
        end
        
        T = reshape(Ts(:,:,imageListIdx),[3 3]);
        invT = inv(T);
        
        blobsInFrame = mFrames.getBlobsInFrame(phaseListIdx);
        nBlobs = length(blobsInFrame);
        cellBoundaries = cell(1,nBlobs);
        isOK = true(1,nBlobs);
        cellInfo = nan(nBlobs,3);
        if plotPoles
            cellPoles = nan(2*nBlobs,2);
        end
        counter = 0;
        for blobLabel = blobsInFrame
            counter = counter+1;
            cellsInBlob = mFrames.getCellsInBlob(phaseListIdx,blobLabel);
            c = cellsInBlob(1);            
            t = c.getIndex(phaseListIdx);
            if ~isempty(selectedCells) && ~any(selectedCells == c.id)
                continue
            end
            if isempty(segList)
                theBoundary = c.convertFromCellCoord(t,c.boundaries{t});
            else
                origo = round(c.centroids(t,:));
                label = L(origo(2),origo(1));
                if ~label
                    idx = find(segIm==c.blobLabels(t),1);
                    label = L(idx);
                end
                theBoundary = B{label}(:,[2 1]);
            end
            theBoundary = convertCoord(theBoundary, invT);
            if min(theBoundary(:, 1)) < fluoROI(1) || ...
               min(theBoundary(:, 2)) < fluoROI(2) || ...
               max(theBoundary(:, 1)) > fluoROI(1)+fluoROI(3)-1 || ...
               max(theBoundary(:, 2)) > fluoROI(2)+fluoROI(4)-1
                continue
            end
            cellInfo(counter,:) = [c.centroids(t,:) c.id];
            if plotCells
                if uniqueCellColors
                    line(theBoundary(:,1), theBoundary(:,2),...
                        'Color',cellColorMap(c.id,:));
                else
                    cellBoundaries{counter} = theBoundary(:,[2 1]);
                    if c.badSegmentations(t)
                        isOK(counter)=false;
                    end
                end
            end
            if ~c.badSegmentations(t)
                if plotBackbones && c.lengths(t) > 0
                    f = c.backbones(:,t);
                    poleCoord = c.convertToCellCoord(t,c.poles{t},1);
                    if ~c.isReoriented(t)
                        step = 0.2;
                    else
                        step = -0.2;
                    end
                    bb = (poleCoord(1,1):step:poleCoord(2,1))';
                    bb(:,2) = f(1)*bb(:,1).^2 + f(2)*bb(:,1) + f(3);
                    bb = c.convertFromCellCoord(t,bb,1);
                    bb = convertCoord(bb, invT);
                    line(bb(:,1),bb(:,2),'Color','y');
                end
                if plotPoles && c.lengths(t) > 0
                    cellPoles([2*counter-1 2*counter],:) = c.poles{t};
                end
            end
        end % loop through blobs
        if plotCells && ~uniqueCellColors
            visboundaries(cellBoundaries(isOK),...
                'Color','g','LineWidth',1,'EnhanceVisibility',false);
            visboundaries(cellBoundaries(~isOK),...
                'Color','r','LineWidth',1,'EnhanceVisibility',false);
        end
        if plotPoles && any(~isnan(cellPoles(:)))
            cellPoles = cellPoles(~isnan(cellPoles(:,1)),:);
            cellPoles = convertCoord(cellPoles, invT);
            plot(cellPoles(1:2:end,1),cellPoles(1:2:end,2),'g*');
        end
        cellInfo = convertCoord(cellInfo, invT);
        if plotDots && ~isempty(pList)
            ind = pList.detectionFrame == phaseListIdx;
            dotCoord = pList.fluoCoord(ind,:);
            if radius
                radii = radius*ones(size(dotCoord,1),1);
                viscircles(dotCoord,radii,'Color','r',...
                    'EnhanceVisibility',false);
            end
            plot(dotCoord(:,1), dotCoord(:,2), 'r.');
            dcm = datacursormode(fig);
            set(dcm, 'UpdateFcn', {@datatip_callback, cellInfo, ...
                pList.fluoCoord(ind, 1:2), pList.internalCoord(ind, 1:2)})
        else
            dcm = datacursormode(fig);
            set(dcm, 'UpdateFcn', {@datatip_callback,cellInfo})
        end
        hold(ax,'off');
        drawnow
        fprintf('Frame: %d\n',phaseFrame);
        set(fig,'KeyPressFcn', @(f,e) key_pressed_fcn(f,e,imageListIdx));
    end % plotFrame
    %% Navigation with keyboard arrows
    function key_pressed_fcn(src,~,imageListIdx)
        key = get(src,'CurrentKey');
        switch key
            case 'rightarrow'
                imageListIdx = imageListIdx+1;
            case 'leftarrow'
                imageListIdx = imageListIdx-1;
            case 'uparrow'
                imageListIdx = imageListIdx + 10;
            case 'downarrow'
                imageListIdx = imageListIdx - 10;
            otherwise
                return
        end
        delete(findall(src,'Type','hggroup'));
        plotFrame(imageListIdx);
    end
end