function displayCellsOnPhase(expInfoObj, posName, varargin)
% displayCellsOnPhase(expInfoObj, posName, varargin)
%
% Plots preprocessed phase contrast or segmented images with cell
% boundaries, backbones, poles, and dots representing cell particles.
%
% Use keyboard arrows to navigate between frames:
%  - press right or left to show the next or previous frames, respectively.
%  - press up or down to move by 10 frames.
%
% Use active data cursor to show particle internal coordinates.
%
% IMPORTANT - figure must be active and zoom/pan/data cursor etc. not
% active while pressing keyboard arrows to switch frames
%
% Inputs
%   expInfoObj  - expInfo object
%   posName     - string containing position name
%
% List of optional input arguments:
%   Frame               - phase frame. By default the first phase frame
%                         selected for analysis.
%   Cells               - array of MCell objects. If [], mCells are loaded
%                         using expInfoObj. By default [].
%   SelectedCells       - array with id of selected cells. By default [].
%   PlotCells           - if 1, plot cell boundaries. By default 1.
%   PlotBackbones       - if 1, plot cell backbones. By default 0.
%   PlotDots            - if 1, plot circles around detected particles.
%                         By default 0.
%   PlotPoles           - if 1, plot circular markers in the "old" cell
%                         poles. By default 0.
%   PlotSegmentation    - if 1, plot segmented images instead of phase
%                         images. By default 0.
%   ShowEvents          - if 1, use thick line for plotting cell boundary
%                         if the cell is born/appear in this frame or
%                         divide/disappear in the next frame. By default 0.
%   UniqueCellColors    - if 1, use unique color per cell track, otherwise
%                         use green and red (for bad segmentations) colors.
%                         By default 0.
%   Radius              - radius of circles around detected dots, in
%                         pixels. By default 0.
%
% See also displayCellsOnFluo, Visualizer
    %% Initialization
    phaseChanName = expInfoObj.getChannelNames('phase');
    fluoChanNames = expInfoObj.getChannelNames('fluo');
    phaseFrames = expInfoObj.getIndices(posName,phaseChanName);
    
    ip = inputParser;
    defaultFrame = phaseFrames(1);
    validLogical = @(x) islogical(x) || (x==0) || (x==1);
    ip.addRequired('Position',@(x) ischar(x));
    ip.addParameter('Channel',[],@(x) ischar(x) || isempty(x));
    ip.addParameter('Frame',defaultFrame);
    ip.addParameter('PlotBackbones',0,validLogical);
    ip.addParameter('PlotPoles',0,validLogical);
    ip.addParameter('PlotCells',1,validLogical);
    ip.addParameter('PlotDots',0,validLogical);
    ip.addParameter('PlotSegmentation',0,validLogical);
    ip.addParameter('UniqueCellColors',0,validLogical);
    ip.addParameter('ShowEvents',0,validLogical);
    ip.addParameter('SelectedCells',[],@(x) isnumeric(x))
    ip.addParameter('Radius',0,@(x) isnumeric(x))
    ip.addParameter('Cells',[]);
    
    ip.parse(posName,varargin{:});
    posName = ip.Results.Position;
    frame = ip.Results.Frame;
    plotBackbones = ip.Results.PlotBackbones;
    plotPoles = ip.Results.PlotPoles;
    plotCells = ip.Results.PlotCells;
    plotDots = ip.Results.PlotDots;
    plotSegmentation = ip.Results.PlotSegmentation;
    uniqueCellColors = ip.Results.UniqueCellColors;
    showEvents = ip.Results.ShowEvents;
    selectedCells = ip.Results.SelectedCells;
    radius = ip.Results.Radius;
    
    if ~isempty(ip.Results.Cells)
        mCells = ip.Results.Cells;
    else
        loadFile = expInfoObj.getMCellMatPath(posName);
        mCells = Cell.MCell.loadMCells(loadFile);
    end
    mFrames = Cell.MCell.mFrames;
    
    if uniqueCellColors
        rng('default');
        rng(1);
        cellColorMap = rand(length(mCells),3);
    end
    
    if ~isempty(selectedCells)
        mCells = mCells(selectedCells);
    end
    
    outputDir = expInfoObj.getPathName('output');
    if ~plotSegmentation
        imageDir = fullfile(outputDir, posName, 'PreprocessedPhase');
    else
        imageDir = fullfile(outputDir, posName, 'SegmentedPhase');
    end
    imageList = dir(fullfile(imageDir, '*.tif*'));
    
    % Use segmented images if cell boundaries are not stored
    if isempty([mCells.boundaries])
        segDir = fullfile(outputDir, posName, 'SegmentedPhase');
        segList = dir(fullfile(segDir, '*.tif*'));
    else
        segList = [];
    end

    if plotDots
        pList = getParticleList(mCells);
        dotColors = {'r','c','m'};
    end
    %% Create figure and show the first frame
    fig = figure;
    ax = gca;
    imageListIdx = find(phaseFrames==frame,1);
    plotFrame(imageListIdx);
    %% Main plotting function
    function plotFrame(imageListIdx)
        if imageListIdx < 1
            imageListIdx = length(imageList);
        elseif imageListIdx > length(imageList)
            imageListIdx = 1;
        end
        % Plot image
        if ~isempty(get(ax,'Children'))
            xlimits = xlim(ax);
            ylimits = ylim(ax);
        else
            xlimits = [];
        end
        phaseFrame = phaseFrames(imageListIdx);
        im = imread(fullfile(imageDir, imageList(imageListIdx).name));
        if plotSegmentation
            im = im>0;
        end
        imagesc(ax,im);
        colormap(ax,'gray');
        axis(ax,'image');
        set(ax,'Ydir','reverse','YTick',[],'XTick',[],...
            'Clipping','off','Visible','off');
        if ~isempty(xlimits)
            xlim(ax,xlimits);
            ylim(ax,ylimits);
        end
        hold(ax,'on');
        
        if ~isempty(segList)
            segIm = imread(fullfile(segDir,segList(imageListIdx).name));
            [B,L] = bwboundaries(segIm>0,'noholes');
        end
        
        countCellDiv = 0;
        countCellBirth = 0;
        countCellDeath = 0;
        
        blobsInFrame = mFrames.getBlobsInFrame(imageListIdx);
        nBlobs = length(blobsInFrame);
        cellBoundaries = cell(1,nBlobs);
        isOK = true(1,nBlobs);
        isEvent = false(1,nBlobs);
        cellInfo = nan(nBlobs,3);
        if plotPoles
            cellPoles = nan(2*nBlobs,2);
        end
        counter = 0;
        for blobLabel = blobsInFrame
            counter = counter+1;
            cellsInBlob = mFrames.getCellsInBlob(imageListIdx,blobLabel);
            c = cellsInBlob(1);
            t = c.getIndex(imageListIdx);
            if ~isempty(selectedCells) && ~any(selectedCells == c.id)
                continue
            end
            cellInfo(counter,:) = [c.centroids(t,:) c.id];
            if plotCells
                if isempty(segList)
                    theBoundary = c.convertFromCellCoord(t,c.boundaries{t});
                else
                    origo = round(c.centroids(t,:));
                    label = L(origo(2),origo(1));
                    if ~label
                        idx = find(segIm==c.blobLabels(t),1);
                        label = L(idx);
                    end
                    theBoundary = B{label}(:,[2 1]);
                end
                if uniqueCellColors
                    p = line(theBoundary(:,1), theBoundary(:,2),...
                        'Color',cellColorMap(c.id,:));
                else
                    cellBoundaries{counter} = theBoundary(:,[2 1]);
                    if c.badSegmentations(t)
                        isOK(counter)=false;
                    end
                end
                if t == 1 && imageListIdx>1
                    isEvent(counter)=true;
                    countCellBirth = countCellBirth +1;
                    if uniqueCellColors && showEvents
                        set(p,'LineWidth',2);
                    end
                elseif t == c.lifeTime && imageListIdx < mFrames.getNumOfFrames()
                    if isempty(c.descendants)
                        countCellDeath = countCellDeath +1;
                    else
                        countCellDiv = countCellDiv +1;
                    end
                    isEvent(counter)=true;
                    if uniqueCellColors && showEvents
                        set(p,'LineWidth',2);
                    end
               end
            end
            if ~c.badSegmentations(t)
                if plotBackbones && c.lengths(t) > 0
                    f = c.backbones(:,t);
                    poleCoord = c.convertToCellCoord(t,c.poles{t},1);
                    if ~c.isReoriented(t)
                        step = 0.2;
                    else
                        step = -0.2;
                    end
                    bb = (poleCoord(1,1):step:poleCoord(2,1))';
                    bb(:,2) = f(1)*bb(:,1).^2 + f(2)*bb(:,1) + f(3);
                    bb = c.convertFromCellCoord(t,bb,1);
                    line(bb(:,1),bb(:,2),'Color','y');
                end
                if plotPoles && c.lengths(t) > 0
                    cellPoles([2*counter-1 2*counter],:) = c.poles{t};
                end
            end
        end % loop through blobs
        if plotCells && ~uniqueCellColors
            if showEvents
                visboundaries(cellBoundaries(isOK & ~isEvent),...
                    'Color','g','LineWidth',1,'EnhanceVisibility',false);
                visboundaries(cellBoundaries(isOK & isEvent),...
                    'Color','g','LineWidth',2,'EnhanceVisibility',false);
                visboundaries(cellBoundaries(~isOK & ~isEvent),...
                    'Color','r','LineWidth',1,'EnhanceVisibility',false);
                visboundaries(cellBoundaries(~isOK & isEvent),...
                    'Color','r','LineWidth',2,'EnhanceVisibility',false);
            else
                visboundaries(cellBoundaries(isOK),...
                    'Color','g','LineWidth',1,'EnhanceVisibility',false);
                visboundaries(cellBoundaries(~isOK),...
                    'Color','r','LineWidth',1,'EnhanceVisibility',false);
            end
        end
        if plotPoles && any(~isnan(cellPoles(:)))
            cellPoles = cellPoles(~isnan(cellPoles(:,1)),:);
            if radius
                radii = radius*ones(size(cellPoles,1)/2,1);
                viscircles(cellPoles(1:2:end,:),radii,'Color','g',...
                   'LineWidth',1,'EnhanceVisibility',false);
            end
        end
        if plotDots && ~isempty(pList)
            ind = pList.detectionFrame == imageListIdx;
            for chIdx=1:numel(fluoChanNames)
                fluoChanName = fluoChanNames{chIdx};
                curInd = ind & strcmp(pList.chanName,fluoChanName);
                dotCoord = pList.phaseCoord(curInd, :);
                radii = 3*ones(size(dotCoord,1),1);
                viscircles(dotCoord,radii,'Color',dotColors{chIdx},...
                    'EnhanceVisibility',false);
                plot(dotCoord(:,1),dotCoord(:,2),[dotColors{chIdx},'.']);
            end
            dcm = datacursormode(fig);
            set(dcm, 'UpdateFcn', {@datatip_callback, cellInfo, ...
                pList.phaseCoord(ind, 1:2), pList.internalCoord(ind, 1:2)})
        else
            dcm = datacursormode(fig);
            set(dcm, 'UpdateFcn', {@datatip_callback,cellInfo})
        end
        hold(ax,'off');
        drawnow
        fprintf('Frame: %d, born: %d, divided: %d, disappeared: %d\n',...
            phaseFrame, countCellBirth, countCellDiv, countCellDeath);
        set(fig,'KeyPressFcn',@(f,e) key_pressed_fcn(f,e,imageListIdx));
    end % plotFrame
    %% Navigation with keyboard arrows
    function key_pressed_fcn(src,~,imageListIdx)
        key = get(src,'CurrentKey');
        switch key
            case 'rightarrow'
                imageListIdx = imageListIdx+1;
            case 'leftarrow'
                imageListIdx = imageListIdx-1;
            case 'uparrow'
                imageListIdx = imageListIdx + 10;
            case 'downarrow'
                imageListIdx = imageListIdx - 10;
            otherwise
                return
        end
        delete(findall(src,'Type','hggroup'));
        plotFrame(imageListIdx);
    end
end