function stats = displayHistogramPlots(varargin)
% displayHistogramPlots(X) creates a figure with histogram plot for
% each set of observations defined by X.
% The input parameter X is either a cell array of numeric arrays in case of
% multiple set of observations, or a numeric array in case of a single set.
%
% displayHistogramPlots(X,M), where M is a scalar, uses M bins.
%
% displayHistogramPlots(X,EDGES), where EDGES is a vector, specifies 
% the edges of the bins.
%
% displayHistogramPlots(...,'Legend',LEGEND) adds legend LEGEND to
% the plot, where LEGEND is a cell array with labels corresponding to the
% sets of observations, or a char array in case of a single data set.
%
% displayHistogramPlots(...,'Normalization',NM) specifies the
% normalization scheme of the histogram values returned in N. NM can be:
%                'count'   Each N value is the number of observations in 
%                          each bin. SUM(N) is generally equal to 
%                          NUMEL(X), but is less than if some of 
%                          the input data is not included in the bins. 
%                          This is the default.
%          'probability'   Each N value is the relative number of 
%                          observations (number of observations in bin / 
%                          total number of observations), and SUM(N) is 
%                          less than or equal to 1.
%         'countdensity'   Each N value is the number of observations in 
%                          each bin divided by the width of the bin. 
%                  'pdf'   Probability density function estimate. Each N 
%                          value is, (number of observations in bin) / 
%                          (total number of observations * width of bin).
%             'cumcount'   Each N value is the cumulative number of 
%                          observations in each bin and all previous bins. 
%                          N(end) is less than or equal to NUMEL(X).
%                  'cdf'   Cumulative density function estimate. Each N 
%                          value is the cumulative relative number of 
%                          observations in each bin and all previous bins. 
%                          N(end) is less than or equal to 1.
%
% displayHistogramPlots(...,'PlotStyle',PS) plots lines if PS='line' 
% or vertical bars if PS='bar'. The default value is 'line'.
%
% displayHistogramPlots(AX,...) plots into the axes with handle AX.
%
% Output:
%   stats - struct array with statistics for each set of observations:
%                 'mean'   mean value
%               'stddev'   standard deviation
%                   'cv'   coefficient of variation
%              'stdmean'   standard deviation of the mean
%                  'num'   number of nonnan data points

%%%%%%%%%%%%%%%%%%%
% Parse parameters
if length(varargin)>1 && isa(varargin{1},'matlab.graphics.axis.Axes')
    ax = varargin{1};
    varargin = varargin(2:end);
else
    ax = [];
end

ip = inputParser();
ip.addRequired('data',@(x) iscell(x) || isnumeric(x));
ip.addOptional('binMethod',[], @(x) isempty(x) || isnumeric(x));
ip.addParameter('Normalization','count');
ip.addParameter('Legend',{}, @(x) iscell(x) || ischar(x));
ip.addParameter('PlotStyle','line',@(x) ismember(lower(x),{'line','bar'}))
ip.parse(varargin{:});
data = ip.Results.data;
binMethod = ip.Results.binMethod;
normMethod = ip.Results.Normalization;
labels = ip.Results.Legend;
plotStyle = ip.Results.PlotStyle;

if isnumeric(data)
    data = num2cell(data,[1 2]);
end

if ischar(labels)
    labels = cellstr(labels);
elseif isempty(labels)
    labels = cellstr("data " + string(1:length(data)));
end

%%%%%%%%%%%%%%%%%%
% Compute statistics and plot histograms
if isempty(ax)
    figure, 
    ax = gca;
end
hold(ax,'on')
stats = struct;
for i=1:length(data)
    X = data{i};
    X = X(~isnan(X));
    stats(i).mean = mean(X);
    stats(i).stddev = std(X);
    stats(i).cv = num2str(std(X)/mean(X));
    stats(i).stdmean = num2str(std(X)/sqrt(length(X)));
    stats(i).num = length(X);
    disp([labels{i} ': Mean:' num2str(mean(X)) ...
        ', Std: ' num2str(std(X)) ... 
        ', CV: ' num2str(std(X)/mean(X)) ...
        ', Std of mean: ' num2str(std(X)/sqrt(length(X))) ... 
        ', Number of data-points: ' num2str(length(X))]); 
   
    if isempty(binMethod)
        [n, e] = histcounts(X,'Normalization',normMethod);
    else
        [n, e] = histcounts(X,binMethod,'Normalization',normMethod);
    end
    x = e(1:end-1)+ 0.5*diff(e);
    if strcmp(plotStyle,'line')
        plot(ax,x, n)
    else
        bar(ax,x,n,1)
    end

end
if strcmp(plotStyle,'bar') && length(data)>1
    alpha(ax,0.5)
end
hold(ax,'off')
legend(labels)