function displaySingleCellLineageOnFluo(expInfoObj, posName, cellID, varargin)
% displaySingleCellLineageOnFluo(expInfoObj, posName, cellID, varargin)
%
% Plots fluorescent or bright-field images with cell
% boundaries, backbones, poles, and dots representing cell particles.
% Only the cell with id=cellID and its descendants are displayed.
% 
% Use keyboard arrows to navigate between frames:
%  - press right or left to show the next or previous frames, respectively.
%  - press up or down to move by 10 frames.
%
% Use active data cursor to show particle internal coordinates by clicking
% on dots. Click in the middle of the cell to display cell id.
%
% IMPORTANT - figure must be active and zoom/pan/data cursor etc. not
% active while pressing keyboard arrows to switch frames.
%
% Input:
%   expInfoObj - expInfo object
%   posName - string,  position name
%   cellID - cell track id
%
% List of optional input arguments:
%   Channel             - name of a fluorescence or bright-field channel.
%                         By default, the first fluorescence channel.
%   Frame               - phase frame. By default the first phase frame
%                         selected for analysis.
%   Cells               - array of MCell objects. If [], mCells are loaded
%                         using expInfoObj. By default [].
%   PlotCells           - if 1, plot cell boundaries. By default 1.
%   PlotBackbones       - if 1, plot cell backbones. By default 0.
%   PlotDots            - if 1, plot circles around detected particles.
%                         By default 0.
%   PlotPoles           - if 1, plot circular markers in the "old" cell
%                         poles. By default 0.
%   UniqueCellColors    - if 1, use unique color per cell track, otherwise
%                         use green and red (for bad segmentations) colors.
%                         By default 0.
%   Radius              - radius of circles around detected dots, in
%                         pixels. By default 0.
%   CLIM                - array with two values between 0 and 1, defines
%                         imagesc scaling. Default is [0.01 0.99]
%
% See also displayCellsOnFluo, displaySingleCellLineageOnPhase

ip = inputParser;
validLogical = @(x) islogical(x) || (x==0) || (x==1);
ip.addRequired('Position',@(x) ischar(x));
ip.addRequired('ID',@(x) isnumeric(x) && x>0);
ip.addParameter('Channel',[],@(x) ischar(x) || isempty(x));
ip.addParameter('Cells',[]);
ip.addParameter('Frame',[]);
ip.addParameter('PlotBackbones',0,validLogical);
ip.addParameter('PlotPoles',0,validLogical);
ip.addParameter('PlotCells',1,validLogical);
ip.addParameter('PlotDots',1,validLogical);
ip.addParameter('UniqueCellColors',0,validLogical);
ip.addParameter('Radius',0,@(x) isnumeric(x))
ip.addParameter('CLIM',[0.01 0.99],@(x) isnumeric(x) && length(x)==2)
    
ip.parse(posName,cellID,varargin{:});
posName = ip.Results.Position;
cellID = ip.Results.ID;
frame = ip.Results.Frame;
plotBackbones = ip.Results.PlotBackbones;
plotPoles = ip.Results.PlotPoles;
plotCells = ip.Results.PlotCells;
plotDots = ip.Results.PlotDots;
uniqueCellColors = ip.Results.UniqueCellColors;
radius = ip.Results.Radius;
clim = ip.Results.CLIM;
if ~isempty(ip.Results.Cells)
    mCells = ip.Results.Cells;
else
    loadFile = expInfoObj.getMCellMatPath(posName);
    mCells = Cell.MCell.loadMCells(loadFile);
end
if ~isempty(ip.Results.Channel)
    fluoChanName = ip.Results.Channel;
else
    fluoChanNames = expInfoObj.getChannelNames('fluo');
    fluoChanName = fluoChanNames{1};        
end
mCell = mCells([mCells.id]==cellID);
if isempty(frame)
    phaseChanName = expInfoObj.getChannelNames('phase');
    phaseFrames = expInfoObj.getIndices(posName,phaseChanName);
    frame = phaseFrames(mCell.birthFrame);
end
cellIndices = getCellLineage(mCell);

displayCellsOnFluo(expInfoObj,posName,'Channel',fluoChanName,...
    'Cells',mCells,'Frame',frame,...
    'SelectedCells',cellIndices,'PlotCells',plotCells,...
    'PlotBackbones',plotBackbones,'PlotDots',plotDots,...
    'PlotPoles',plotPoles,'UniqueCellColors',uniqueCellColors,...
    'Radius',radius, 'CLIM',clim);