function displaySingleCellLineageOnPhase(expInfoObj, posName, cellID, varargin)
% displaySingleCellLineageOnPhase(expInfoObj, posName, cellID, varargin)
%
% Plots preprocessed phase contrast or segmented images with cell
% boundaries, backbones, poles, and dots representing cell particles.
% Only the cell with id=cellID and its descendants are displayed.
% 
% Use keyboard arrows to navigate between frames:
%  - press right or left to show the next or previous frames, respectively.
%  - press up or down to move by 10 frames.
%
% Use active data cursor to show particle internal coordinates by clicking
% on dots. Click in the middle of the cell to display cell id.
%
% IMPORTANT - figure must be active and zoom/pan/data cursor etc. not
% active while pressing keyboard arrows to switch frames.
%
% Input:
%   expInfoObj - expInfo object
%   posName - string,  position name
%   cellID - cell track id
%
% List of optional input arguments:
%   Frame               - phase frame. By default the first phase frame
%                         selected for analysis.
%   Cells               - array of MCell objects. If [], mCells are loaded
%                         using expInfoObj. By default [].
%   PlotCells           - if 1, plot cell boundaries. By default 1.
%   PlotBackbones       - if 1, plot cell backbones. By default 0.
%   PlotDots            - if 1, plot circles around detected particles.
%                         By default 0.
%   PlotPoles           - if 1, plot circular markers in the "old" cell
%                         poles. By default 0.
%   PlotSegmentation    - if 1, plot segmented images instead of phase
%                         images. By default 0.
%   ShowEvents          - if 1, use thick line for plotting cell boundary
%                         if the cell is born/appear in this frame or
%                         divide/disappear in the next frame. By default 0.
%   UniqueCellColors    - if 1, use unique color per cell track, otherwise
%                         use green and red (for bad segmentations) colors.
%                         By default 0.
%   Radius              - radius of circles around detected dots, in
%                         pixels. By default 0.
%
% See also displayCellsOnPhase, displaySingleCellLineageOnFluo

ip = inputParser;
validLogical = @(x) islogical(x) || (x==0) || (x==1);
ip.addRequired('Position',@(x) ischar(x));
ip.addRequired('ID',@(x) isnumeric(x) && x>0);
ip.addParameter('Cells',[]);
ip.addParameter('Frame',[]);
ip.addParameter('PlotBackbones',0,validLogical);
ip.addParameter('PlotPoles',0,validLogical);
ip.addParameter('PlotCells',1,validLogical);
ip.addParameter('PlotDots',0,validLogical);
ip.addParameter('PlotSegmentation',0,validLogical);
ip.addParameter('UniqueCellColors',0,validLogical);
ip.addParameter('ShowEvents',0,validLogical);
ip.addParameter('Radius',0,@(x) isnumeric(x))
    
ip.parse(posName,cellID,varargin{:});
posName = ip.Results.Position;
cellID = ip.Results.ID;
frame = ip.Results.Frame;
plotBackbones = ip.Results.PlotBackbones;
plotPoles = ip.Results.PlotPoles;
plotCells = ip.Results.PlotCells;
plotDots = ip.Results.PlotDots;
plotSegmentation = ip.Results.PlotSegmentation;
uniqueCellColors = ip.Results.UniqueCellColors;
showEvents = ip.Results.ShowEvents;
radius = ip.Results.Radius;
    
if ~isempty(ip.Results.Cells)
    mCells = ip.Results.Cells;
else
    loadFile = expInfoObj.getMCellMatPath(posName);
    mCells = Cell.MCell.loadMCells(loadFile);
end
mCell = mCells([mCells.id]==cellID);
if isempty(frame)
    phaseChanName = expInfoObj.getChannelNames('phase');
    phaseFrames = expInfoObj.getIndices(posName,phaseChanName);
    frame = phaseFrames(mCell.birthFrame);
end
cellIndices = getCellLineage(mCell);

displayCellsOnPhase(expInfoObj,posName,'Cells',mCells,'Frame',frame,...
    'SelectedCells',cellIndices,'PlotCells',plotCells,...
    'PlotBackbones',plotBackbones,'PlotDots',plotDots,...
    'PlotPoles',plotPoles,'PlotSegmentation',plotSegmentation,...
    'UniqueCellColors',uniqueCellColors','ShowEvents',showEvents,...
    'Radius',radius);