function [] = dotBrowser(analysisDir, posName, startFrameNum)
% This function plots detected and refined dots on the top of fluo images
%
% Input:
%   - analysisDir - path to the directory with output from ImAnalysis
%   - posName : string with the position name where you would like to
%               browse a movie
%   - startFrameNum : movie frame number to start with
%
% Author: Ivan Volkov, 2015-12-09

% Last modified: SZ, 2018-08-21. Adjusted to the newest pipeline.
    %% Load data
    expInfoObj = loadExpInfo(analysisDir);
    positionList = expInfoObj.getPositionList();
    movieN = find(strcmp(positionList,posName),1);
    if isempty(movieN)
        error('Position %s not found',posName);
    end
    sourceDir = expInfoObj.getPathName('source');
    pathPosOutput = fullfile(analysisDir,posName);
    fluoChanNames = expInfoObj.getChannelNames('fluo');
    fluoChanName = fluoChanNames{1};

    % Load LSQrefine data
    LSQfile = fullfile(pathPosOutput,['LSQdotCoordinates_' fluoChanName '.mat']);
    load(LSQfile,'totalDotCoord0','allMAPcoord','totalDotCoord');

    %% Set paths to fluo and segm images
    pathFlImages = fullfile(sourceDir,posName,fluoChanName);
    listFlImages = dir(fullfile(pathFlImages,'*.tif*'));
    imagePathFl = fullfile(pathFlImages,listFlImages(1).name);
    pathSegImages = fullfile(pathPosOutput,'SegmentedPhase');
    listSegImages = dir(fullfile(pathSegImages,'*.tif*'));
    imagePathSeg = fullfile(pathSegImages,listSegImages(1).name);

    imInfo = imfinfo(imagePathFl);
    numberOfFrames = numel(imInfo);
    frameRange = startFrameNum:numberOfFrames;

    %% Transform segmented image to fluo coordinates
    T = expInfoObj.getTByFrame(posName, 1, fluoChanName);
    invT = inv(T);
    invT(:,3) = [0;0;1];
    flSize = [imInfo(1).Height imInfo(1).Width];
    
    segIm = imread(imagePathSeg);
    segImScaled = imwarp(segIm,affine2d(invT),'OutputView',imref2d(flSize),'FillValues',0);
    %% Calculation of right color scaling for fluo pictures
    midleFrame = imread(imagePathFl,numberOfFrames,'Info', imInfo);
    maxInt = max(max(midleFrame));
    minInt = min(min(midleFrame));

    %% Main loop - showing the movie frame by frame
    frameIdx = min(frameRange);
    disp('Which frame number would you like?');
    disp('Will be next, if empty value; Will be previous, if 0:');
    while frameIdx <= max(frameRange) && frameIdx >0
        %% Plot fluo images
        currFrame = imread(imagePathFl,frameIdx,'Info', imInfo);
        ax1 = subplot(211); imshow(currFrame,[]); caxis(ax1, [minInt maxInt]);
        title(strcat('Frame number___', num2str(frameIdx)));
        hold on
        ax2 = subplot(212); imshow(currFrame,[]); caxis(ax2, [minInt maxInt]);
        title('\fontsize{11} \color{blue}BLUE - after Wavelet/RadSym; \color{yellow}YELLOW - survived after MAP fitting; \color{red}RED - survived after MAP filtration');
        hold on
        %% Drawing segmentation on the top of the images
        NumberOfCells = max(max(segImScaled));
        for cc = 1:NumberOfCells
            singleCell = segImScaled == cc;
            B = bwboundaries(singleCell,'noholes');

            for k = 1:length(B)
                boundary = B{k};
                plot(ax2, boundary(:,2), boundary(:,1), 'g', 'LineWidth', 1)
            end
        end
        %% Plot dots after wavelet/RadSym 
        AllDotsInthisMovie = totalDotCoord0{1};

        subArrayX = AllDotsInthisMovie(AllDotsInthisMovie(:,3) == frameIdx, 1);
        subArrayY = AllDotsInthisMovie(AllDotsInthisMovie(:,3) == frameIdx, 2);

        for kk = 1:length(subArrayX)
             myCircle(subArrayX(kk),subArrayY(kk),2, ax2, '-b')
             hold on
        end
     %% Plot dots survived after MAP fitting
        AllMAPDotsInthisMovie = allMAPcoord{1};

        subArrayX = AllMAPDotsInthisMovie(AllMAPDotsInthisMovie(:,3) == frameIdx, 1);
        subArrayY = AllMAPDotsInthisMovie(AllMAPDotsInthisMovie(:,3) == frameIdx, 2);

        for kk = 1:length(subArrayX)
             myCircle(subArrayX(kk),subArrayY(kk),2, ax2, '-y')
             hold on
        end
    %% Plot dots survived after MAP filtration
        AllFittedDotsInthisMovie = totalDotCoord{1};

        subArrayX = AllFittedDotsInthisMovie(AllFittedDotsInthisMovie(:,3) == frameIdx, 1);
        subArrayY = AllFittedDotsInthisMovie(AllFittedDotsInthisMovie(:,3) == frameIdx, 2);

        for kk = 1:length(subArrayX)
             myCircle(subArrayX(kk),subArrayY(kk),2, ax2, '-r')
             hold on
        end
        %% Select next frame
        %pause; %Wayt for a key to proceed to the next frame
        message = strcat('Current_frame_', num2str(frameIdx));
        prompt = (message);
        nextFrameIdx = input(prompt);

        if nextFrameIdx > 0
            frameIdx = nextFrameIdx; 
        elseif nextFrameIdx == 0
            frameIdx = frameIdx - 1;
        else
            frameIdx = frameIdx + 1;
        end
    end
end


function myCircle(x,y,r, axN, color)
%x and y are the coordinates of the center of the circle
%r is the radius of the circle
%0.01 is the angle step, bigger values will draw the circle faster but
%you might notice imperfections (not very smooth)
    ang=0:0.05:2*pi; 
    xp=r*cos(ang);
    yp=r*sin(ang);
    plot(axN, x+xp,y+yp, color);
end