classdef expInfo < handle
    % An object containing various info about the experiment
    
    % Last modified:
    % 2018-05-05, SZ. Access all transformation matrices in position and
    % channel names.
    % 2018-03-12, SZ. Access transformation matrices using
    % frame index as the 3rd dimension, not the first
    
    properties (SetAccess = private)
        metadataPath      = '';
        transfMatrices    = {};
        rois              = {};
        channels          = {};
        channelMap       = [];
        positions         = {};
        positionMap      = [];
        sourceFolderPath  = '.';
        outputFolderPath  = '.';
        phaseChannelNames = {};
        bfChannelNames    = {};
        fluoChannelNames  = {};
        saveName          = '';
        parameters        = [];
        imIndices         = {};
        imRange           = {};
        failedRegistration = [];
        channelLocations  = {};
    end
    
    %% Private methods
    methods (Access = private)
        function T = getT(obj, posIdx, frameIdx, chanIdx)
            try
                if ~isempty(frameIdx)
                    try
                        T = obj.transfMatrices{posIdx}{chanIdx}(:,:,frameIdx);
                        T = reshape(T,[3 3]);
                    catch
                        T = obj.transfMatrices{posIdx}{chanIdx}{frameIdx};
                    end
                else
                    T = obj.transfMatrices{posIdx}{chanIdx};
                end
            catch
                T = [];
            end
        end
        
        function setT(obj, posIdx, frameIdx, chanIdx, T)
            if numel(size(T)) ~= 2 || any(size(T) ~= 3)
                error('Setting transformation matrix in wrong format');
            end
            obj.transfMatrices{posIdx}{chanIdx}(:,:,frameIdx) = T;
        end
        
        function posIdx = getPosIndex(obj, posName)
            try
                posIdx = obj.positionMap(posName);
            catch
                posIdx = [];
            end
        end
        
        function setRoi(obj, chanName, inRoi)
            chanIdx = obj.getChanIndex(chanName);
            obj.rois{chanIdx} = inRoi;
        end
        
        function chanIdx = getChanIndex(obj, chanName)
            try
                chanIdx = obj.channelMap(chanName);
            catch
                chanIdx = [];
            end
        end
        
    end
    
    methods
        % Constructor
        function obj = expInfo(varargin)
            for i = 1 : 2 : length(varargin)
                obj.(varargin{i}) = varargin{i+1};
            end
            obj.failedRegistration = cell(numel(obj.positions),numel(obj.channels));
            for bfChIdx = 1:numel(obj.bfChannelNames)
                bfChanName = obj.bfChannelNames{bfChIdx};
                chanIdx = obj.channelMap(bfChanName);
                for posIdx = 1:numel(obj.positions)
                    obj.failedRegistration{posIdx,chanIdx} = ...
                        false(numel(obj.imRange{posIdx,chanIdx}));
                end
            end
        end
        
        function setParameters(obj,varargin)
            if length(varargin) == 1
                obj.parameters = varargin{1};
            elseif length(varargin) == 2
                if isfield(obj.parameters,varargin{1})
                    obj.parameters.(varargin{1}) = varargin{2};
                else
                    warning(['Setting a value to non-existing parameter ' varargin{1}]);
                end
            else
                warning('Too many input arguments');
            end        
        end
        %% Accessors
        % Folder related functions
        function positions = getPositionList(obj, posRange)
            positions = obj.positions;
            if nargin == 2 && ~isempty(posRange)
                positions = positions(posRange);
            end
        end
        
        function pathName = getPathName(obj,directoryType)
            if ~ischar(directoryType)
                error('Directory type must be a sting');
            end
            directoryType = lower(directoryType);
            switch(directoryType)
                case 'source'
                    pathName = obj.sourceFolderPath;
                case 'input'
                    pathName = obj.sourceFolderPath;
                case 'output'
                    pathName = obj.outputFolderPath;
                otherwise
                    pathName = '';
            end
        end
        
        function chanName = getChannelNames(obj,channelType)
            if nargin == 1
                chanName = obj.channels;
            else
                if ~ischar(channelType)
                    error('Channel type must be a sting');
                end
                channelType = lower(channelType);
                switch(channelType)
                    case 'phase'
                        chanName = obj.phaseChannelNames;
                    case 'bf'
                        chanName = obj.bfChannelNames;
                    case 'fluo'
                        chanName = obj.fluoChannelNames;
                    case 'fluor'
                        chanName = obj.fluoChannelNames;
                    otherwise
                        chanName = {};
                end
            end
        end
                
        % Get transformation matrices
        function T = getTByFrame(obj, posName, frameIdx, chanName)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            T = obj.getT(posIdx, frameIdx, chanIdx);
        end
        
        function T = getAllT(obj, posName, chanName)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            T = obj.getT(posIdx, [], chanIdx);
        end
        
        % Others
        function matPath = getMCellMatPath(obj, posName)
            outputDir = obj.getPathName('output');
            matPath = fullfile(outputDir,posName,'trackedCells.mat');
        end
        
        function roi = getRoi(obj, chanName)
            chanIdx = obj.getChanIndex(chanName);
            roi = obj.rois{chanIdx};
        end
        
        function imIndices = getIndices(obj, posName, chanName)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            try
                imIndices = obj.imIndices{posIdx,chanIdx};
            catch
                imIndices = [];
            end
        end
        
        function imRange = getRange(obj, posName, chanName)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            try
                imRange = obj.imRange{posIdx,chanIdx};
            catch
                imRange = [];
            end
        end
        
        function parameters = getParameters(obj,paramName)
            parameters = obj.parameters;
            if nargin == 2
                if isfield(parameters, paramName)
                    parameters=parameters.(paramName);
                else
                    error('parameter %s does not exist', paramName);
                end
            end
        end
        
        function channelLocations = getChannelLocations(obj,posName)
            posIdx = obj.getPosIndex(posName);
            channelLocations = obj.channelLocations{posIdx};
        end

        %% Mutators
        % Set transformation matrices
        function setTByFrame(obj, posName, chanName, frameIdx, T)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            obj.setT(posIdx, frameIdx, chanIdx, T);
        end
        
        function setAllT(obj, posName, chanName, T)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            for frameIdx = 1:size(T,3)
                currT = T(:,:,frameIdx);
                obj.setT(posIdx, frameIdx, chanIdx, currT);
            end
        end
        
        % Other 
        function setAllRois(obj, chanNames, inRois)
            % the channel names and inRois need to be in the same order
            for chanIdx = 1:length(chanNames)
                obj.setRoi(chanNames{chanIdx},inRois{chanIdx});
            end
        end
        
        function setFailedRegistration(obj, posName, chanName, dataArray)
            posIdx = obj.getPosIndex(posName);
            chanIdx = obj.getChanIndex(chanName);
            obj.failedRegistration{posIdx,chanIdx} = dataArray;
        end
        
        function setChannelLocations(obj, posName, dataArray)
            % dataArray - nCh x nF array, where nCh is number of growth 
            % channels and nF is number of frames
            posIdx = obj.getPosIndex(posName);
            nGrowthChannels = obj.getParameters('nGrowthChannels');
            nEmptyChannels = length(obj.getParameters('emptyChannel'));
            if nEmptyChannels
                nGrowthChannels = nGrowthChannels-nEmptyChannels+1;
            end
            phaseChanName = obj.getChannelNames('phase');
            nFrames = numel(obj.getRange(posName,phaseChanName));
            [k,m,n] = size(dataArray);
            if k == nGrowthChannels && m == 4 && n == nFrames
                obj.channelLocations{posIdx} = dataArray;
            else
                error('Wrong data matrix size');
            end
        end
        %% Other functions
        function saveExpInfo(expInfoObj, path)
            if nargin < 2
                save(fullfile(expInfoObj.outputFolderPath,expInfoObj.saveName),'expInfoObj');
            else
                save(fullfile(path,expInfoObj.saveName),'expInfoObj');
            end
        end
        
        function phaseCoord = convertCoordToPhase(obj, posName, fluoChanName, frameIdx, fluoCoord)
            if isempty(fluoCoord)
                phaseCoord = [];
                return;
            end
            posIdx = obj.positionMap(posName);
            chanIdx = obj.getChanIndex(fluoChanName);
            T = obj.getT(posIdx, frameIdx, chanIdx);
            inCoord(:,1:2) = fluoCoord(:,1:2);
            inCoord(:,3) = 1;

            tempCoord = inCoord * T;
            phaseCoord = tempCoord(:,1:2);
            phaseCoord(:,3:size(fluoCoord,2)) = fluoCoord(:,3:size(fluoCoord,2));
        end
        
        function fluoCoord = convertCoordToFluo(obj, posName, fluoChanName, frameIdx, phaseCoord)
            if isempty(phaseCoord)
                fluoCoord = [];
                return;
            end
            posIdx = obj.positionMap(posName);
            chanIdx = obj.getChanIndex(fluoChanName);
            T = obj.getT(posIdx, frameIdx, chanIdx);
            inCoord(:,1:2) = double(phaseCoord(:,1:2));
            inCoord(:,3) = 1;
            tempCoord = inCoord / T;
            fluoCoord = [tempCoord(:,1:2) phaseCoord(:,3:end)];
        end
    end % methods
end
