function outInfo = getCellMeasurements(expInfoObj,varargin)
% outInfo = getCellMeasurements(expInfoObj,posRange)
% extracts various cell measurements from the pipeline output defined by
% expInfoObj.
% 
% Input:
%   expInfoObj - expInfo object
%   posRange - array of position indices for analysis. Optional,
%              by default [], i.e. all positions are used.
%   lengthCutOff - cutoff threshold of cell track length, 
%                  optional, 5 by default.
%   parentLengthCutoff - cutoff threshold of parent cell track length,
%                        optional, 0 by default
%   daughterLengthCutoff - cutoff threshold of daughter cell track length,
%                          optional, 0 by default.
% 
% User can provide the following optional name-value pair arguments:
%   PixelSize - scaling factor between 1 pixel in phase contrast images and
%               um. It is required for fork plots and estimating initiation
%               size in bulk. Default value is 1.
%   BirthArea - logical flag to compute birth areas. Default value is 1.
%   DivisionArea - logical flag to compute division areas. 
%                  Default value is 1.
%   InitiationArea - logical flag to compute initiation areas.
%                    Default value is 1 if applicable.
%   GenerationTime - logical flag to compute generation times.
%                    Default value is 1.
%   InitiationAreaBulk - logical flag to estimate average initiation area
%                        in bulk. Default value is 1 if applicable.
%   GrowthRate - logical flag to compute growth rates. Default value is 1.
%   ForkPlot - logical flag to create figure with fork distribution plot.
%              Default is 1 if applicable.
%   BinScale - used in fork plot to adjust the number of bins. Higher
%   value gives narrower bins. Default value is 20.
%   HeatMapThreshold - used in fork plot for heatmap thresholding. Must be
%                      a scaler in (0,1]. Default value is 0.99.
%   UtrackParam - structure with u-Track parameters for dot tracking:
%                   - uWindow
%                   - uMergeSplit
%                   - uMinTrackLen
%                   - uBrownStdMult
%                   - uMinSearchRadius
%                   - uMaxSearchRadius
%                   - uLinearMotion
%                   - uMinGapSearchRadius
%                   - uMaxGapSearchRadius
%                   - uAmpRatioLimit
%                   - uGapPenalty
%   MinInitTrackLength - minimum length of a dot trajectory to count as
%                        the initiation event.
%
% Output:
%   outInfo - struct with following fields:
%       BirthAreas - array of cell birth areas
%       DivisionAreas - array of cell division areas
%       InitiationAreas - array of cell initiation areas
%       GrowthRates - array of cell growth rates
%       GenerationTimes - array of cell generation times
%       DotLongCoordinates - array of dot "long" internal coordinates
%       Areas - array of cell areas in dot detection frames 
%       Lengths - array of cell lengths in dot detection frames
%       CellCounts - array of cell counts
%       MeanBirthArea - average cell birth area
%       MeanDivisionArea - average cell division area
%       MeanInitiationArea - average cell initiation area
%       MeanGrowthRate - average cell growth rate
%       MeanGenerationTime - average cell generation time
%       MeanInitiationAreaBulk - average cell initiation area in bulk
%
% See also: getBirthAreas, getDivisionAreas, getGenerationTimes,
% getGrowthRates, getDotLocations, getInitiationAreaInBulk, displayForkPlot
% getInitationAreas, fitGaussianInitArea

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parse parameters
ip = inputParser;
validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x>0);
validLogical = @(x) islogical(x) || (x==0) || (x==1);
if expInfoObj.getParameters('doDotDetection') == 2
    defForkPlot = 1;
    defInitiationArea = 1;
    defInitiationAreaBulk = 1;
else
    defForkPlot = 0;
    defInitiationArea = 0;
    defInitiationAreaBulk = 0;
end
ip.addRequired('positionRange',@(x) isnumeric(x));
ip.addOptional('lengthCutOff',5,validScalarPosNum);
ip.addOptional('parentLengthCutOff',3,validScalarPosNum);
ip.addOptional('daughterLengthCutOff',3,validScalarPosNum);
ip.addParameter('PixelSize',1,validScalarPosNum)
ip.addParameter('BirthArea',1,validLogical);
ip.addParameter('DivisionArea',1,validLogical);
ip.addParameter('InitiationArea',defInitiationArea,validLogical);
ip.addParameter('GenerationTime',1,validLogical);
ip.addParameter('InitiationAreaBulk',defInitiationAreaBulk,validLogical);
ip.addParameter('GrowthRate',1,validLogical);
ip.addParameter('ForkPlot',defForkPlot,validLogical);
ip.addParameter('BinScale',20, @(x) isscalar(x) && isnumeric(x) && x>0);
ip.addParameter('HeatMapThreshold',1,@(x) isscalar(x) && x>0 && x<=1);
ip.addParameter('UtrackParam',[],@(x) isstruct(x));
ip.addParameter('MinInitTrackLength',[],@(x) isnumeric(x) & x>0);

ip.parse(varargin{:});
posRange = ip.Results.positionRange;
lengthCutOff = ip.Results.lengthCutOff;
parentLengthCutOff = ip.Results.parentLengthCutOff;
daughterLengthCutOff = ip.Results.daughterLengthCutOff;
pixelSize = ip.Results.PixelSize;
doBirthArea = ip.Results.BirthArea;
doDivisionArea = ip.Results.DivisionArea;
doInitiationArea = ip.Results.InitiationArea;
doInitiationAreaBulk = ip.Results.InitiationAreaBulk;
doGrowthRate = ip.Results.GrowthRate;
doGenerationTime = ip.Results.GenerationTime;
doForkPlot = ip.Results.ForkPlot;
binScale = ip.Results.BinScale;
heatMapThresh = ip.Results.HeatMapThreshold;
minInitTrackLength = ip.Results.MinInitTrackLength;
uParam = ip.Results.UtrackParam;

if doIinitiationArea
    if isempty(uParam)
        error('Missing u-track parameters');
    end
    if isempty(minInitTrackLength)
        error('Missing parameter MinInitTrackLength');
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make computations and plots
posList = expInfoObj.getPositionList(posRange);
nPositions = length(posList);
fluoChanNames = expInfoObj.getChannelNames('fluo');
fluoChanName = fluoChanNames{1};

tmpBirthAreas = cell(1,nPositions);
tmpDivisionAreas = cell(1,nPositions);
tmpInitiationAreas = cell(1,nPositions);
tmpGrowthRates = cell(1,nPositions);
tmpGenerationTimes = cell(1,nPositions);
tmpLongs = cell(1,nPositions);
tmpAreas = cell(1,nPositions);
tmpLengths = cell(1,nPositions);
tmpCounts = cell(1,nPositions);
for p=1:nPositions
    matFile = expInfoObj.getMCellMatPath(posList{p});
    mCells = Cell.MCell.loadMCells(matFile);
    fluorIndices = expInfoObj.getIndices(posList{p}, fluoChanName);

    if doBirthArea
        tmpBirthAreas{p} = getBirthAreas(mCells,lengthCutOff,...
            parentLengthCutOff,daughterLengthCutOff,'PixelSize',pixelSize);
    end
    
    if doDivisionArea
        tmpDivisionAreas{p} = getDivisionAreas(mCells,lengthCutOff,...
            parentLengthCutOff,daughterLengthCutOff,'PixelSize',pixelSize);
    end
    
    if doGrowthRate
        tmpGrowthRates{p} = getGrowthRates(mCells,lengthCutOff);
    end
    
    if doGenerationTime
        tmpGenerationTimes{p} = getGenerationTimes(mCells,lengthCutOff,...
            parentLengthCutOff,daughterLengthCutOff);
    end
    
    if doInitiationArea
        tmpInitiationAreas{p} = getInitiationAreas(mCells,fluorIndices,...
            1,lengthCutOff,parentLengthCutOff,daughterLengthCutOff,0,...
            1,1,uParam,minInitTrackLength,pixelSize,0);
    end
    
    if doInitiationAreaBulk || doForkPlot
        [~, lns, as, ls, ~, cs] = getDotLocations(mCells, lengthCutOff, ...
            parentLengthCutOff, daughterLengthCutOff,'PixelSize',pixelSize);
        tmpLongs{p} = lns;
        tmpAreas{p} = as;
        tmpLengths{p} = ls;
        tmpCounts{p} = cs;
    end
end

birthAreas = cell2mat(tmpBirthAreas);
divisionAreas = cell2mat(tmpDivisionAreas);
initiationAreas = cell2mat(tmpInitiationAreas);
growthRates = cell2mat(tmpGrowthRates);
generationTimes = cell2mat(tmpGenerationTimes);
longs = cell2mat(tmpLongs);
areas = cell2mat(tmpAreas);
lengths = cell2mat(tmpLengths);
counts = cell2mat(tmpCounts);

avgBirthArea = mean(birthAreas);
avgDivisionArea = mean(divisionAreas);
avgGrowthRate = mean(growthRates);
avgGenerationTime = mean(generationTimes);
avgInitiationArea = fitGaussianInitArea(initiationAreas/pixelSize^2,-1,pixelSize);

if doInitiationAreaBulk
    avgInitiationAreaBulk = getInitiationAreaInBulk(longs,areas,lengths,counts,avgDivisionArea);
else
    avgInitiationAreaBulk = [];
end

if doForkPlot
    if doInitiationArea
        avgIA = avgInitiationArea;
    else
        avgIA = avgInitiationAreaBulk;
    end
    displayForkPlot(longs,areas,lengths,counts,avgBirthArea,...
        avgDivisionArea,avgIA,'BinScale',binScale,...
        'HeatMapThreshold',heatMapThresh);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create output structure
outInfo = struct(...
    'BirthAreas', birthAreas, ...
    'DivisionAreas', divisionAreas, ...
	'InitiaionAreas', initiationAreas, ...
    'GrowthRates', growthRates, ...
	'GenerationTimes', generationTimes, ...
	'DotLongCoordinates', longs, ...
	'Areas', areas, ...
	'Lengths', lengths, ...
    'CellCounts', counts, ...
    'MeanBirthArea', avgBirthArea, ...
    'MeanDivisionArea', avgDivisionArea, ...
    'MeanInitiationArea', avgInitiationArea, ...
    'MeanGrowthRate', avgGrowthRate, ...
    'MeanGenerationTime', avgGenerationTime, ...
    'MeanInitiationAreaBulk', avgInitiationAreaBulk);