function [areas, cellIndices] = getDivisionAreas(mCells,varargin)
% This function extracts information about cell division areas. The cell
% should have a parent cell and two daughter cells, all the cell tracks
% should be sufficiently long. Then the division area is the area of the 
% cell in the last detection frame.
%
% Input:
%   mCells - array of MCell objects
%   lengthCutOff - cutoff threshold of cell track length,
%                  optional, 5 by default
%   parentLengthCutoff - cutoff threshold of parent cell track length,
%                        optional, 3 by default
%   daughterLengthCutoff - cutoff threshold of daughter cell track length,
%                          optional, 3 by default
%
% getDivisionAreas(...,'PixelSize',pixelSize) scales the output values from
% px^2 to um^2 according to pixelSize (in um). Optional, no scaling by
% default (pixelSize=1).

% Output:
%   areas - array with cell division areas
%   cellIndices - array with indices of the cells where 
%                 division area was measured

ip = inputParser;
validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x > 0);
ip.addOptional('lengthCutOff',5,validScalarPosNum);
ip.addOptional('parentLengthCutOff',3,validScalarPosNum);
ip.addOptional('daughterLengthCutOff',3,validScalarPosNum);
ip.addParameter('PixelSize',1,validScalarPosNum)
ip.parse(varargin{:});
lengthCutOff = ip.Results.lengthCutOff;
parentLengthCutOff = ip.Results.parentLengthCutOff;
daughterLengthCutOff = ip.Results.daughterLengthCutOff;
pixelSize = ip.Results.PixelSize;

areas = nan(1, length(mCells));

for i = 1:length(mCells)
    c = mCells(i);
    if ~isempty(c.parent) && c.parent.lifeTime >= parentLengthCutOff
        if length(c.descendants) == 2 && all([c.descendants.lifeTime] >= daughterLengthCutOff)
            if c.isBadCell == 0 && c.lifeTime >= lengthCutOff
                if c.badSegmentations(end) == 0
                    areas(i) = c.areas(end);
                end     
            end
        end
    end
end

cellIndices = find(~isnan(areas));
areas = areas(cellIndices);

if pixelSize~=1
    areas = areas * pixelSize^2;
end