function expInfoObj = getExpInfo(sourceDir,outputDir,imAdapterObj,parameters)

% Last modified:
% SZ, 2017-12-21. Handle multiple fluo channels
% SZ, 2019-02-07. Load existing expInfoObj only if doPreprocessing = 2. Use
% negative position indices to exclude them from the analysis.

filePath = fullfile(outputDir,'expInfoObj.mat');
if exist(filePath,'file') && parameters.doPreprocessing == 2
    [~, expInfoObj] = loadObject(filePath);
    disp('Loaded expInfo object');   
    expInfoObj.setParameters(parameters);
    expInfoObj.saveExpInfo();
    return
end

positionList = imAdapterObj.getPositionList();
if ~isempty(parameters.rangePositions)
    posInd = parameters.rangePositions>0;
    if any(posInd)
        ind = false(size(positionList));
        ind(parameters.rangePositions(posInd)) = true;
    else
        ind = true(size(positionList));
    end
    ind(-parameters.rangePositions(~posInd)) = false;
    positionList = positionList(ind);
end
nPositions = length(positionList);

phaseChanName = parameters.phaseChannelNames;
if iscellstr(parameters.bfChannelNames)
    bfChanNames = parameters.bfChannelNames;
else
    bfChanNames{:} = parameters.bfChannelNames;
end
if iscellstr(parameters.fluoChannelNames)
    flChanNames  = parameters.fluoChannelNames;
else
    flChanNames{:}  = parameters.fluoChannelNames;
end

channelList = [phaseChanName,flChanNames,bfChanNames];
nChannels = numel(channelList);

positionMap = containers.Map(positionList,1:nPositions);
channelMap = containers.Map(channelList,1:nChannels);

% Check that there is the same number of fluo and bf channels
nBfCh = numel(bfChanNames);
nFlCh = numel(flChanNames);
if nBfCh==1 && nFlCh>1
    [bfChanNames{1:nFlCh}] = deal(bfChanNames{1});
    nBfCh = nFlCh;
elseif nBfCh>1 && nFlCh>nBfCh
    error('Number of fluo and bf channels does not match!');
end

imRange = cell(nPositions,nChannels);
imIndices = cell(nPositions,nChannels);

phaseImRange = parameters.rangePhaseImages;

for pIdx = 1:nPositions
    currPos = positionList{pIdx};
    
    phaseIdxsTmp = imAdapterObj.getIndicesByPosAndChanName(currPos,phaseChanName);
    if ~isempty(phaseImRange)
        phaseIdxsTmp = phaseIdxsTmp(phaseImRange);
    end
    
    phChIdx = channelMap(phaseChanName);
    imIndices{pIdx,channelMap(phaseChanName)} = phaseIdxsTmp;
    imRange{pIdx,phChIdx} = phaseIdxsTmp;
     
    for chIdx = 1:max(nBfCh, nFlCh)

        if chIdx <= nBfCh
            bfChanName = bfChanNames{chIdx};
            bfIdxsTmp = imAdapterObj.getIndicesByPosAndChanName(currPos,bfChanName);
        else
            bfIdxsTmp = [];
        end
        
        if chIdx <= nFlCh
            fluoChanName = flChanNames{chIdx};
            flIdxsTmp = imAdapterObj.getIndicesByPosAndChanName(currPos,fluoChanName);
        else
            flIdxsTmp = [];
        end

        if ~isempty(bfIdxsTmp) && (numel(bfIdxsTmp) ~= numel(flIdxsTmp) || ...
                sum(bfIdxsTmp == flIdxsTmp) ~= numel(bfIdxsTmp))
            warning(['Dataset does not contain a brightfield image for ' ...
                'each fluorescence image']);
        end

        if size(intersect(bfIdxsTmp, phaseIdxsTmp)) ~= size(bfIdxsTmp)
            warning(['Each brightfield image does not have a corresponding ' ...
               'phase contrast image']);
        end

        [bfIdxsTmp, ~, imRangeBfTmp]   = intersect(phaseIdxsTmp, bfIdxsTmp);
        [flIdxsTmp, ~, imRangeFluoTmp] = intersect(phaseIdxsTmp, flIdxsTmp);

        if ~isempty(flIdxsTmp)
            flChIdx = channelMap(fluoChanName);
            imIndices{pIdx,flChIdx} = flIdxsTmp;
            imRange{pIdx,flChIdx} = imRangeFluoTmp;
        end
        
        if ~isempty(bfIdxsTmp)
            bfChIdx = channelMap(bfChanName);
            imIndices{pIdx,bfChIdx} = bfIdxsTmp;
            imRange{pIdx,bfChIdx} = imRangeBfTmp;
        end

    end
end

expInfoObj = expInfo('metadataPath',      fullfile(sourceDir,'metadata.txt'), ...
                     'positions',         positionList, ...
                     'positionMap',       positionMap, ...
                     'channels',          channelList, ...
                     'channelMap',        channelMap, ...
                     'sourceFolderPath',  sourceDir, ...
                     'outputFolderPath',  outputDir, ...
                     'phaseChannelNames', phaseChanName, ...
                     'bfChannelNames',    bfChanNames, ...
                     'fluoChannelNames',  flChanNames, ...
                     'parameters',        parameters, ...
                     'saveName',          'expInfoObj.mat', ...
                     'imIndices',         imIndices, ...
                     'imRange',           imRange);

expInfoObj.setAllRois(channelList,imAdapterObj.getCroppingRegions(channelList));

% Set initial transformation matrix from bf images to phase images
 if ~strcmp(parameters.transMatPath, '')
    if ~exist(parameters.transMatPath, 'file')
        error('Path to transformation matrix set but file does not exist')
    else
        load(parameters.transMatPath);
        if ~exist('T','var') || size(T, 1) ~= 3 || size(T, 2) ~= 3
            error('Transformation matrix is not of correct format')
        end
    end
elseif isempty(parameters.transMat)
    phaseDir = fullfile(sourceDir,positionList{1},phaseChanName);
    phaseList = dir(fullfile(phaseDir,'*.tif*'));
    im = tiffread(fullfile(phaseDir,phaseList(1).name));
    angularDisplacement = parameters.angularDisplacement;
    if isempty(angularDisplacement)
        angularDisplacement = 0;
    end
    [~, rotatedIm, Trot] = getAngularDisplacement(im,angularDisplacement);
    cropROI =  parameters.roiInPhase;
    if isempty(cropROI)
        cropROI = size(rotatedIm);
    end
    Tcrop = getTranslationMatrix(-(cropROI-1));
    T = Trot*Tcrop;
    expInfoObj.setParameters('transMat',T);
end
disp('Created an expInfo object');
expInfoObj.saveExpInfo();