function [fluorImages, frames] = getFluorPixels(expInfoObj, PosName, mCell, fluoChanName, frames, margin, getBox)
%   [fluorImages, frames] = getFluorPixels(expInfoObj, PosName, mCell, fluoChanName)
%   Returns the fluorescent image of a cell within its outline. Pixels
%   outside the outline are replaced by zero.
%   
%   Inputs
%   expInfoObj, PosName, mCell - mandatory. 
%   PosName         May be either name or number. 
%   fluoChanName    Fluo channel, optional, defaults to the first channel.
%                   May be name or channel number. 
%   frames          Find for only these frames, optional. 
%   margin          Add or remove pixels around the border, optional.
%   getBox          If true, the entire rectangular box is returned. 
%
%   Outputs
%   fluorImages     cell array of fluor images. 
%   frames          the experiment frame numbers corresponding to the
%                   images in flourImages. 
%   Arvid Gynnå, dec 2019

if nargin < 7
    getBox = false; 
end 
if nargin < 6
    margin = 0;
else 
    se = strel('diamond',abs(margin)); 
end 
if nargin < 5
    frames = mCell.birthFrame:mCell.lastFrame;
end

if nargin < 4 
    fluoChanName = expInfoObj.fluoChannelNames{1}; 
end

if isnumeric(PosName)
    PosName = expInfoObj.positions{PosName}; 
end
if sum(strcmp(expInfoObj.positions,PosName)) == 0
    error('getFluorPixels: Invalid position.')
end

if ischar(fluoChanName)
    fluoChanNo = find(strcmp(expInfoObj.channels,fluoChanName));
    if isempty(fluoChanNo)
        error('getFluorPixels: Invalid channel name.')
    end
elseif isnumeric(fluoChanName)
    fluoChanNo = fluoChanName; 
    fluoChanName = expInfoObj.channels{fluoChanName}; 
end

listFluo = dir(fullfile(expInfoObj.sourceFolderPath, PosName, fluoChanName, '*.tiff')); 
listSeg = dir(fullfile(expInfoObj.outputFolderPath, PosName, 'SegmentedPhase', '*.tiff')); 
imIndices = expInfoObj.imIndices{1,fluoChanNo};

fluorImages = cell(length(frames),1); 
for i = 1:length(frames)
    seg_f = frames(i); 
    fluo_frame_no = intersect(imIndices,seg_f);
    if isempty(fluo_frame_no) 
        continue; 
    end
    fluo_f = find(imIndices == fluo_frame_no); 
    
    currFluoIm = imread(fullfile( listFluo(fluo_f).folder, listFluo(fluo_f).name )); 
    currSegIm = imread(fullfile( listSeg(seg_f).folder, listSeg(seg_f).name ));
    T = expInfoObj.getTByFrame(PosName, fluo_f, fluoChanName);
    transFluoIm = imwarp(currFluoIm, affine2d(T), 'OutputView', imref2d(size(currSegIm)), 'FillValues', 0);
    
    cellMask = currSegIm == mCell.blobLabels(seg_f-mCell.birthFrame+1);
    
    if margin > 0
        cellMask = imdilate(cellMask, se); 
    elseif margin < 0 
        cellMask = imerode(cellMask, se); 
    end
    
    if ~getBox
        transFluoIm(~cellMask) = 0; 
    end
    
    croppedFluoIm = transFluoIm(sum(cellMask,2)>0,sum(cellMask)>0);
    fluorImages{i} = croppedFluoIm; 
end
    
end