function [generationTimes, cellIndices] = getGenerationTimes(mCells,varargin)
% This function extracts information about cell generation times. The cell
% should have a parent cell and two daughter cells, all the cell tracks
% should be sufficiently long. Then the generation time is the difference
% between the acquisition times of the last and the first detection frames.
%
% Input:
%   mCells - array of MCell objects
%   lengthCutOff - cutoff threshold of cell track length, 
%                  optional, 5 by default
%   parentLengthCutoff - cutoff threshold of parent cell track length,
%                        optional, 3 by default
%   daughterLengthCutoff - cutoff threshold of daughter cell track length,
%                          optional, 3 by default
%
% Output:
%   generationTimes - array with cell generation times, in minutes
%   cellIndices - array with indices of the cells where 
%                 generation time was measured

ip = inputParser;
validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x > 0);
ip.addOptional('lengthCutOff',5,validScalarPosNum);
ip.addOptional('parentLengthCutOff',3,validScalarPosNum);
ip.addOptional('daughterLengthCutOff',3,validScalarPosNum);
ip.parse(varargin{:});
lengthCutOff = ip.Results.lengthCutOff;
parentLengthCutOff = ip.Results.parentLengthCutOff;
daughterLengthCutOff = ip.Results.daughterLengthCutOff;

generationTimes = nan(1, length(mCells));

for i = 1:length(mCells)
    c = mCells(i);
    if ~isempty(c.parent) && c.parent.lifeTime >= parentLengthCutOff
        if length(c.descendants) == 2 && all([c.descendants.lifeTime] >= daughterLengthCutOff)
            if c.isBadCell == 0 && c.lifeTime >= lengthCutOff
                generationTimes(i) = c.detectionTimes(end)-c.detectionTimes(1); 
            end
        end
    end
end
% Convert to minutes
generationTimes = generationTimes / (60 * 1000);

cellIndices = find(~isnan(generationTimes));
generationTimes = generationTimes(cellIndices);