function [grs, cellIndices, grStds] = getGrowthRates(mCells, varargin)
% This function extracts information about cell growth rates. 
% 
% The cell tracks must be sufficiently long and contain at least 75% frames
% where they are labelled as "OK" (badSegmentation = 0).
% Pose a constraint that a cell should have a parent cell and/or daughter
% cells by setting parameters parentLengthCutoff, daughterLengthCutoff > 0.
%
% Input:
%   mCells - array of MCell objects
%   lengthCutOff - cutoff threshold of cell track length, 
%                  optional, 5 by default.
%   parentLengthCutoff - cutoff threshold of parent cell track length,
%                        optional, 0 by default
%   daughterLengthCutoff - cutoff threshold of daughter cell track length,
%                          optional, 0 by default.
%   nCores - number of parallel workers, optional, 0 by default.
%
% Output:
%   grs - array with growth rates
%   cellIndices - array with indices of the cells where 
%                 growth rate was measured
%   grStds - array with standard deviation from 66.(6)% confidence interval
%
% See also getGrowthRate. 

ip = inputParser;
validScalarPosNum = @(x) isnumeric(x) && isscalar(x) && (x > 0);
validScalarNonNegNum = @(x) isnumeric(x) && isscalar(x) && (x >= 0);
ip.addOptional('lengthCutOff',5,validScalarPosNum);
ip.addOptional('parentLengthCutOff',0,validScalarNonNegNum);
ip.addOptional('daughterLengthCutOff',0,validScalarNonNegNum);
ip.addOptional('nCores',0,validScalarNonNegNum);

ip.parse(varargin{:});
lengthCutOff = ip.Results.lengthCutOff;
parentLengthCutOff = ip.Results.parentLengthCutOff;
daughterLengthCutOff = ip.Results.daughterLengthCutOff;
nCores = ip.Results.nCores;

grs = nan(1, length(mCells));
grStds = nan(1, length(mCells));

parfor (i = 1:length(mCells),nCores)
    c = mCells(i);
    if ~parentLengthCutOff || ...
            (~isempty(c.parent) && c.parent.lifeTime >= parentLengthCutOff)
        if ~daughterLengthCutOff || ...
                (length(c.descendants) == 2 && all([c.descendants.lifeTime] >= daughterLengthCutOff))
            if c.isBadCell == 0 && c.lifeTime >= lengthCutOff
                [grs(i), grStds(i)] = getGrowthRate(c);
            end
        end
    end
end

cellIndices = find(~isnan(grs));
grs = grs(cellIndices);
grStds = grStds(cellIndices);