function [initArea, initFrame, inits] = getInitEventsPerLineage(varargin)
% [initArea, initFrame, inits] = getInitEventsPerLineage(tracks,minLength,id,la,cf,df,a,l,a_f,l_f,dt,useParent)
% Use u-track trajectories with length >= minLength to identify initiation
% events (inits) in cell lineage. The function also returns
% cell area (initArea) and detection frame (initFrame) at the first
% initiation event that happens in the first cell of the lineage.
%
% getInitEventsPerLineage(AX,___) plots into the axes with handle AX.
%
% Input:
%   tracks - output trajectories from u-track   
%   minLength - trajectory length cutoff
%   id - id of the first cell in the lineage
%   la - array with joint dot internal coordinates along "long axis"
%   cf - array with joint dot detection frames
%   df - array with division frames
%   a - array with joint cell area in cell lineage in fluo frames
%   l - array with joint cell length in cell lineage in fluo frames
%   a_f - array with joint fitted cell area in cell lineage in fluo frames
%   l - array with joint fitted cell length in cell lineage in fluo frames
%   dt - array with detection times of fluo frames
%   useParent - if 1, parent of the first cell in the lineage is included
%               to the joint track.
%   offset - array with length offsets used to compute l. Optional, used
%            only for debugging plots.
%   debugFlag - logical, if 1 make plot with dot trajectories and
%               initiation events. If no axes handle is provided, make plot
%               in new figure. Optional, the default value is 1 if axes
%               handle is provided, 0 otherwise.
%
% Output:
%   initArea - cell area at the first initiation event
%   initFrame - detection frame of the first initiation event
%   inits - cell array with the following elements:
%               1 - cell id 
%               2 - array with detection times of initiation events
%               3 - array with cell areas at initiation events
%               4 - array with cell lengths at initiation events
%               5 - array with fitted cell areas at initiation events
%               6 - array with fitted cell lengths at initiation events
%
%
% See also: getJointTrack

% Parse parameters
if length(varargin)>1 && isa(varargin{1},'matlab.graphics.axis.Axes')
    ax = varargin{1};
    defDebugFlag=1;
    varargin = varargin(2:end);
else
    ax = [];
    defDebugFlag=0;
end
ip = inputParser();
ip.addRequired('tracks',@(x) isstruct(x));
ip.addRequired('lengthCutOff',@(x) isscalar(x) && (x>0));
ip.addRequired('id',@(x) isscalar(x) && (x>0));
ip.addRequired('la',@(x) isnumeric(x));
ip.addRequired('cf',@(x) isnumeric(x));
ip.addRequired('df',@(x) isnumeric(x));
ip.addRequired('a',@(x) isnumeric(x));
ip.addRequired('l',@(x) isnumeric(x));
ip.addRequired('a_f',@(x) isnumeric(x));
ip.addRequired('l_f',@(x) isnumeric(x));
ip.addRequired('dt',@(x) isnumeric(x));
ip.addRequired('useParent',@(x) islogical(x) || (x==0) || (x==1));
ip.addOptional('offset',[],@(x) isnumeric(x));
ip.addOptional('debugFlag',defDebugFlag,@(x) islogical(x) || isscalar(x));
ip.parse(varargin{:});
tracks = ip.Results.tracks;
lengthCutOff = ip.Results.lengthCutOff;
id = ip.Results.id;
la = ip.Results.la;
cf = ip.Results.cf;
df = ip.Results.df;
a = ip.Results.a;
l = ip.Results.l;
a_f = ip.Results.a_f;
l_f = ip.Results.l_f;
dt = ip.Results.dt;
useParent = ip.Results.useParent;
offset = ip.Results.offset;
debugFlag  = ip.Results.debugFlag;

nTracks = length(tracks);
if debugFlag
    styles = {'s', 'o', 'd', '+', 'x'};
    nStyles = length(styles);
    nColors = min(nTracks, max(5, nTracks/nStyles));
    colorspec = colormap(jet(nColors));
    
    if isempty(ax)
        figure
        ax = gca;
    end
    cla(ax);
    hold(ax,'on');
    
    % Plot long internal coordinates of all dots
    plot(ax,cf,la,'rs')
end

init_cf = [];
trackLengths = zeros(1,nTracks);
startFrames = zeros(1,nTracks);
endFrames = zeros(1,nTracks);
for i=1:length(tracks)
    % Extract dot trajectories
    theSeq = tracks(i).seqOfEvents;
    glob_start = min(theSeq(:,1));
    glob_stop = max(theSeq(:,1));
    % minInitFrame & minTrackLength
    if (glob_start > 2) && (glob_stop - glob_start) > lengthCutOff
        init_cf = [init_cf glob_start];
    end
    
    trackLengths(i) = glob_stop-glob_start+1;
    startFrames(i) = glob_start;
    endFrames(i) = glob_stop;%startFrames+trackLengths-1;
    
    for j=1:size(tracks(i).tracksCoordAmpCG,1)
        theFound = find(theSeq(:,3) == j);
        theFound2 = find(theSeq(theFound, 2)==1);
        start_x = theSeq(theFound(theFound2),1);
        theFound2 = find(theSeq(theFound, 2)==2);
        stop_x = theSeq(theFound(theFound2),1);
        
        y_coords = tracks(i).tracksCoordAmpCG(j,1:8:end);
        x_coords_loc_start = start_x-glob_start +1;
        x_coords_loc_stop = stop_x-glob_start +1;
        
        if debugFlag % plot dot trajectory
            coloridx = 1 + mod(i-1, nColors);
            styleidx = 1 + floor((i-1)/nColors);
            plot(ax, start_x:stop_x, ...
                y_coords(x_coords_loc_start:x_coords_loc_stop), ...
                styles{styleidx}, 'Color', colorspec(coloridx,:), ...
                'MarkerFaceColor', colorspec(coloridx,:) ,'MarkerSize', 6);
        end
    end
end    

if debugFlag
    % Plot black lines to show cell length
    minY = -67;
    maxY = 172;
    dfTemp = [0 df length(l)];
    for i=1:length(dfTemp)-1
        frames = dfTemp(i)+1:dfTemp(i+1);
        I = ~isnan(l(frames));
        plot(frames(I), l(frames(I))/(2^i)+offset(i),'k','LineWidth',2); 
        plot(frames(I), -l(frames(I))/(2^i)+offset(i),'k','LineWidth',2);
        minY = min([minY -l(frames(I))/(2^i)+offset(i)]);
        maxY = max([maxY l(frames(I))/(2^i)+offset(i)]);
    end
    % Plot red lines in initiation frames
    for k=1:length(init_cf)
        plot(ax,[init_cf(k) init_cf(k)], [minY maxY], 'r--', 'LineWidth', 2)
    end
    % Plot blue lines in division frames
    for k=1:length(df)
        plot(ax,[df(k) df(k)], [minY maxY], 'b','LineWidth', 2)
    end
    hold(ax,'off')
end

% Find cell area at the first initiation event
dfTemp = [0 df length(l)];
if ~useParent
    idx = find(init_cf>dfTemp(1) & init_cf<=dfTemp(2),1);
else
    idx = find(init_cf>dfTemp(2) & init_cf<=dfTemp(3),1);
end
if ~isempty(idx)
    initFrame = init_cf(idx);
    initArea = a(initFrame);
else
    initFrame = nan;
    initArea = nan;
end
if useParent
    initArea = initArea/2;
end

if nargout>2
    % Store output
    inits = {};
    inits{1} = id;
    inits{2} = dt(init_cf);
    inits{3} = a(init_cf);
    inits{4} = l(init_cf);
    inits{5} = a_f(init_cf);
    inits{6} = l_f(init_cf);
end