function [initArea, theFit] = getInitiationAreaInBulk(varargin)
% getInitiationAreaInBulk(longs,areas,lengths,counts,avgDivisionArea,minLength)
%
% Estimates average initiation area in bulk using an error function based
% fitting of average dot counts per cell area bin.
% 
% Input:
%   longs - array of dot internal coordinates along the long axis
%   areas - array of cell areas (in um^2)
%   lengths - array of cell lengths (in um)
%   counts - array of cell counts
%   avgDivisionArea - average cell area at division (in um^2).
%                     Optional, by default [].
%   minLength - threshold, in um. Select dots with distance to the 
%               cell center along the long axis above the threshold: 
%               (x-0.5) * L > minLegth, 
%               where x is dot internal coordinate along the long axis and
%               L is cell length in um. Optional, by default 0.44.
%
% getInitiationAreaInBulk(AX,...) plots into the axes with handle AX.
% 
% getInitiationAreaInBulk(...,'ShowStatistics',1) display statistics about
% the estimage of initiation area. Optional, by default 0.
%
% getInitiationAreaInBulk(...,'MakePlot',1) creates a new figure and 
% plots the fitted curve. Optional, by default 0.
%
% Output:
%   initArea - estimate of average cell area by
%              replication initiation (in um^2)
%   theFit - fit object

    if length(varargin)>1 && isa(varargin{1},'matlab.graphics.axis.Axes')
        ax = varargin{1};
        varargin = varargin(2:end);
    else
        ax = [];
    end

    ip = inputParser;
    validNonEmpty = @(x) ~isempty(x) && isnumeric(x);
    ip.addRequired('longs',validNonEmpty);
    ip.addRequired('areas',validNonEmpty);
    ip.addRequired('lengths',validNonEmpty);
    ip.addRequired('counts',validNonEmpty);
    ip.addOptional('avgDivisionArea',[]);
    ip.addOptional('MinLength',0.44);
    ip.addParameter('ShowStatistics',0);
    ip.addParameter('MakePlot',0);

    ip.parse(varargin{:});
    areas = ip.Results.areas;
    counts = ip.Results.counts;
    longs = ip.Results.longs;
    lengths = ip.Results.lengths;
    avgDivisionArea = ip.Results.avgDivisionArea;
    minLength = ip.Results.MinLength;
    showStatistics = ip.Results.ShowStatistics;
    makePlot = ip.Results.MakePlot;

    [dotsPerCell, areaBins] = getDotsPerCell(areas, counts, longs, lengths, minLength);

    areaBins = areaBins(~isnan(dotsPerCell));
    dotsPerCell = dotsPerCell(~isnan(dotsPerCell));

    if ~isnan(avgDivisionArea)
        selAreaBins = areaBins < avgDivisionArea;
        areaBinsFit = areaBins(selAreaBins);
        dotsPerCellFit = dotsPerCell(selAreaBins);
    else
        areaBinsFit = areaBins;
        dotsPerCellFit = dotsPerCell;
    end

    dpc_fit = @(b,x0,s,x) b*(1+erf((x-x0)/(sqrt(2)*s)));
    startPoint = [2 2 0.2];
    theFit = fit(areaBinsFit', dotsPerCellFit', dpc_fit, 'StartPoint', startPoint);
    initArea = theFit.x0;
    if ~isempty(ax) || makePlot
        if isempty(ax)
            figure,
            ax = gca;
        end
        hold(ax,'on')
        plot(ax,areaBins, dotsPerCell, 'bs')
        plot(ax,areaBinsFit, feval(theFit, areaBinsFit)', 'r')
        ylims = ylim(ax);
        plot(ax, [theFit.x0 theFit.x0], [0 ylims(2)],'k--','Linewidth',2);
        hold(ax,'off')

        xlabel('Area (um2)')
        ylabel('Dots per Cell');
    end

    if showStatistics
        disp('*********Statistics****************')
        disp(['Initiation volume: ' num2str(theFit.x0)])
        disp(['Std initaition volume: ' num2str(theFit.s)])
        disp('***********************************')
    end
end

function [dotsPerCell, areaBins] = getDotsPerCell(areas,counts,long,lengths,minLength)

    binScale = 50;    
    aminmax = quantile(areas, [0.005 0.98]);
    amin = aminmax(1);
    amax = aminmax(2);
    anrbins = round((amax-amin) * binScale);
    abins = linspace(amin, amax, anrbins);
    long = long - 0.5;
    absLongLengths = abs(long.*lengths);
    selLong = absLongLengths >= minLength;

    dotsPerCell = zeros(1, anrbins-1);
    for i = 1:anrbins - 1
        selDots = areas > abins(i) & areas <= abins(i+1);
        selCounts = counts(selDots);
        normFactor = sum(1./selCounts);
        dotsPerCell(i) = sum(selDots & selLong)./normFactor;    
    end
    areaBins = 0.5*(abins(1:end-1)+abins(2:end));
end