function [initAreas, initEvents] = getInitiationAreas(mCells,...
    fluoIndices, N, lengthCutOff, parentLengthCutOff,...
    daughterLengthCutOff, sisterLengthCutOff, useParent, useDaughters,...
    uParam, minInitTrackLen, pixelSize, debugFlag)
% [initAreas, initEvents] = getInitiationAreas(mCells,...
%   fluoIndices, N, lengthCutOff, parentLengthCutOff,...
%   daughterLengthCutOff, sisterLengthCutOff, useParent, useDaughters,...
%   uParam, minInitTrackLen, pixelSize, debugFlag)
%
% Computes estimates of single cell initiation areas (initAreas) and
% initiation events (initEvents) per cell lineage with N generations.
% Each cell in the lineage must satisfy the following conditions:
%   1) to be an "OK" cell.
%   2) If parentLengthCutOff>0, the cell has an "OK" parent cell.
%   3) If daughterLengthCutOff>0, the cell has two "OK" daughter cells.
%   4) If sisterLengthCutOff>0, the cell has an "OK" sister cell.
%
% A cell track is considered "OK" if
%   - it has labelled isBadCell=0;
%   - its length >= given length cutoff.
%   - it has labels badSegmentations=0 in >= 90 % frames.
%
% Input:
%   mCells - array of MCell objects
%   fluoIndices - array of fluo frames
%   N - number of cell generations in a lineage
%   lengthCutOff - cutoff threshold of cell track length.
%   parentLengthCutoff - cutoff threshold of parent cell track length
%   daughterLengthCutoff - cutoff threshold of daughter cell track length
%   sisterLengthCutoff - cutoff threshold of sister cell track length
%   useParent - if 1, include the parent cell into the track.
%   useDaughters - if 1, include to the lineages daughter cells of the N-th
%                  cell in the lineage. Could be handy for computing single
%                  cell initiation area.
%   uParam - structure array with u-Track parameters:
%       - uWindow
%       - uMergeSplit
%       - uMinTrackLen
%       - uBrownStdMult
%       - uMinSearchRadius
%       - uMaxSearchRadius
%       - uLinearMotion
%       - uMinGapSearchRadius
%       - uMaxGapSearchRadius
%       - uAmpRatioLimit
%       - uGapPenalty
%   minInitTrackLen - minimum length of a dot trajectory to count as
%                     the initiation event
%   pixelSize - pixel size in um, scales the output cell area to um^2.
%               Optional, no scaling by default (pixelSize=1).
%   debugFlag - if 1, create figure and display dot trajectories and
%               initiation events for each cell lineage. Press any button
%               to move to the next cell lineage.
%
% Output:
%   initAreas - array with unique cell birth areas.
%   initEvents - cell array, each element in turn is a cell array with 
%                the following information about correspond cell lineage:
%               1 - cell id 
%               2 - array with detection times of initiation events
%               3 - array with cell areas at initiation events
%               4 - array with cell lengths at initiation events
%               5 - array with fitted cell areas at initiation events
%               6 - array with fitted cell lengths at initiation events
%
% See also: selectCellLineages, getJointTrack, getInitEventsPerLineage

% Author - Spartak Zikrin, 2020-04-20.

%% Parse parameters

if nargin<10
    pixelSize = 1;
end

if nargin<11
    debugFlag=0;
end

% Set u-track parameters
timeWindow = uParam.uWindow;
mergeSplit = uParam.uMergeSplit;
minTrackLen = uParam.uMinTrackLen;
minSearchRadius = uParam.uMinSearchRadius;
maxSearchRadius = uParam.uMaxSearchRadius;
brownStdMult = uParam.uBrownStdMult;
linearMotion = uParam.uLinearMotion;
minGapSearchRadius = uParam.uMinGapSearchRadius;
maxGapSearchRadius = uParam.uMaxSearchRadius;
ampRatioLimit = uParam.uAmpRatioLimit;
gapPenalty = uParam.uGapPenalty;

if debugFlag
    figure;
    ax = gca;
end
%% Select cell lineages and find initiation events in each lineage
lineages = selectCellLineages(mCells,N,lengthCutOff,parentLengthCutOff,...
    daughterLengthCutOff,sisterLengthCutOff,useDaughters);

initEvents = cell(1,length(lineages));
areas = nan(1,length(lineages));
frames = nan(1,length(lineages));
if debugFlag
    for r=1:length(lineages)
        lineage = lineages(r,:);
        cid = lineage(1);
        [sa,la,~,~,~,cf,df,a,l,a_f,l_f,dt,lo] = ...
            getJointTrack(mCells,lineage,fluoIndices,useParent);
        movieInfo = BuildTrajectories.prepareDataForUTrack([la; sa; cf]');
        tracks = runUTrack(movieInfo,timeWindow,mergeSplit,...
            minTrackLen,minSearchRadius,maxSearchRadius,brownStdMult,...
            linearMotion,minGapSearchRadius,maxGapSearchRadius,...
            ampRatioLimit,gapPenalty);
        [areas(r),frames(r),initEvents{r}] = getInitEventsPerLineage(ax,tracks,...
            minInitTrackLen,cid,la,cf,df,a,l,a_f,l_f,dt,useParent,lo);
        title(ax,sprintf('Initiation events in lineage [%s]',num2str(lineage,'%d')))
        pause
    end
else
    parfor r=1:length(lineages)
        lineage = lineages(r,:);
        cid = lineage(1);
        [sa,la,~,~,~,cf,df,a,l,a_f,l_f,dt,lo] = ...
            getJointTrack(mCells,lineage,fluoIndices,useParent);
        movieInfo = BuildTrajectories.prepareDataForUTrack([la; sa; cf]');
        tracks = runUTrack(movieInfo,timeWindow,mergeSplit,...
            minTrackLen,minSearchRadius,maxSearchRadius,brownStdMult,...
            linearMotion,minGapSearchRadius,maxGapSearchRadius,...
            ampRatioLimit,gapPenalty);
        [areas(r),frames(r),initEvents{r}] = getInitEventsPerLineage(tracks,...
            minInitTrackLen,cid,la,cf,df,a,l,a_f,l_f,dt,useParent,lo);
    end
end
%% Find single cell initiation area
ind = ~isnan(areas);
areas = areas(ind);
cids = lineages(ind,1)';
frames = frames(ind);
ucids = unique(cids); 
initAreas = nan(1,length(ucids));
for i=1:length(ucids)
    idx1 = find(cids == ucids(i));
    [~, idx2] = min(frames(idx1)); 
    initAreas(i) = areas(idx1(idx2));
end
%% Scale output
if pixelSize ~= 1
    initAreas = initAreas * pixelSize^2;
end