function [T, phaseROI, angularDisplacement, barcodeROI, Treg, phaseIm] = ...
    setInitialParameters(sourceDir, outputDir, parameterFile, varargin)
% T = setInitialParamters(sourceDir, outputDir, parameterFile, varargin) 
% 
% This function will read the imaging data in *sourceDir*, 
% parameter file *parameterFile* and then generate a transformation matrix
% T between brightfield and preprocessed phase contrast image. 
% The function will end by writing a new parameter file in folder
% *outputDir*
% 
% Pair-wise optional arguments:
%   Orientation - logical 2-tuple [f1 f2]. If f1=1, the phase image is
%                 flipped in up-down direction. If f2=1. the phase image is
%                 flipped in left-right direction. By default, [0 0].
%   Rotation - If a scalar, the image is rotated by *Rotation* degrees. 
%              If 'manual', the function prompts you a GUI to define an
%              angle by which the images will be rotated.
%              If 'auto', the function will automatically detect lines
%              created by growth channels and find the angle to
%              rotae the channels horizontally.
%              By default, if mother-machine, then 'auto', otherwise
%              'manual'.
%   CroppingRegion - If [] the function will prompt a GUI in which the
%                    the ROI can be defined. If supplied the image will
%                    be cropped accordingly.
%   SelectBarcode - if 1, the function will prompt a GUI in which the
%                   the barcode ROI can be defined.
%                   If 'manual', the function prompts you a GUI in which
%                   the barcode ROI can be defined.
%                   If 'auto', the function will automatically find ROI by
%                   detectind barcode circles.
%                   By default, 'auto'.
%   Treg - If [] the function prompts a GUI in which landmarks
%          in phase contrast and brightfield can be chooseen to
%          use for registration. By default [].
% 
% Output:
%   T - the transformation matrix to use when mapping from an
%       rotated and cropped phase contrast (for example) as the
%       result of pre-processing.
%   phaseROI - roi in phase images after reorientation.
%   angularDisplacement - angle in degrees, by which the images are rotated
%   barcodeROI - roi in the reoriented rotated image
%               (before cropping) with barcode dots.
%   Treg - see above, output from landmark-based image registation
%   phaseIm - preprocessed phase image. 
%% Parse parameters
disp('Parsing parameter file');
param = parseParameterFile(parameterFile);
if param.isMothermachine
    defRotation = 'auto';
else
    defRotation = 'manual';
end
optValues = {'auto','manual'};
ip = inputParser();
ip.addParameter('Orientation',[0 0],@(x) isnumeric(x) && length(x)==2);
ip.addParameter('Rotation',defRotation, @(x) any(strcmpi(x,optValues)) || isscalar(x));
ip.addParameter('CroppingRegion',[],@(x) isnumeric(x) && length(x)==4)
ip.addParameter('SelectBarcode','auto',@(x) any(strcmpi(x,optValues)) || isscalar(x))
ip.addParameter('Treg',[]);
ip.parse(varargin{:});
flipFlags = ip.Results.Orientation;
angularDisplacement = ip.Results.Rotation;
phaseROI = ip.Results.CroppingRegion;
selectBarcode = ip.Results.SelectBarcode;
Treg = ip.Results.Treg;
%% Initialization
disp('Parsing metadata file');
imAdapterObj = getImAdapter(sourceDir, outputDir);
expInfoObj = getExpInfo(sourceDir, outputDir, imAdapterObj, param);

positionList = expInfoObj.getPositionList();
phaseChanName = expInfoObj.getChannelNames('phase');
bfChanNames = expInfoObj.getChannelNames('bf');
if isempty(bfChanNames)
    bfFlag = false;
else
    bfFlag = true;
end
pos = positionList{1};
phaseImDir = fullfile(sourceDir, pos, phaseChanName);
listPhase = dir(fullfile(phaseImDir,'*.tif*'));
if bfFlag
    bfIndices = imAdapterObj.getIndicesByPosAndChanName(pos, bfChanNames{1});
    phaseIm = tiffread(fullfile(phaseImDir, listPhase(bfIndices(1)).name));
    bfImDir = fullfile(sourceDir, pos, bfChanNames{1});
    listBf = dir(fullfile(bfImDir,'*.tif*'));
    bfIm = imread(fullfile(bfImDir, listBf(1).name));
    bfIm = FastSeg.Preprocess.rescaleSingleImage(bfIm);
else
    phaseIm = tiffread(fullfile(phaseImDir, listPhase(1).name));
end
phaseIm = FastSeg.Preprocess.rescaleSingleImage(phaseIm);
%% Flip phase image to have the same orientation as bf image
phaseImMod = phaseIm;
if flipFlags(1)
    phaseImMod = flipud(phaseImMod);
end
if flipFlags(2)
    phaseImMod = fliplr(phaseImMod);
end
%% Landmark-based image registration
phaseROIOnCamera = imAdapterObj.getCroppingRegions(phaseChanName);
if bfFlag
    if isempty(Treg) && bfFlag
        Treg = registerWithLandmarks(phaseImMod, bfIm);
        if ~isnan(phaseROIOnCamera(1))
            Treg = Treg * getTranslationMatrix(phaseROIOnCamera-1);
        end
    end
    T = Treg;
    if ~isnan(phaseROIOnCamera(1))
        T = T * getTranslationMatrix(-(int16(phaseROIOnCamera) - 1));
    end
else
    T = eye(3);
end
%% Rotate phasecontrast image and , phase ROI
if strcmpi(angularDisplacement,'auto')
     [angularDisplacement, phaseImMod, Trot] = getAngularDisplacementMM(phaseImMod,1);
elseif strcmpi(angularDisplacement,'manual')
    [angularDisplacement, phaseImMod, Trot] = getAngularDisplacement(phaseImMod);
else
    [angularDisplacement, phaseImMod, Trot] = getAngularDisplacement(phaseImMod, angularDisplacement);
end
T = T * Trot;
%% Locate barcode ROI
if strcmpi(selectBarcode,'auto')
    radiusRange = [param.minWidth round(param.maxWidth*1.5)];
    barcodeROI = getBarcodeLocation(phaseImMod,radiusRange,1);
    pause(0.2);
elseif strcmpi(selectBarcode,'manual')
    barcodeROI = getROI(phaseImMod,'Select barcode ROI');
else
    barcodeROI = [];
end
%% Crop image
if isempty(phaseROI)
    if bfFlag
        bfImTrans = imwarp(bfIm, affine2d(T), 'OutputView', imref2d(size(phaseImMod)),'FillValues',0);
        phaseROI = getROI(phaseImMod, 'Select ROI for phase image',bfImTrans);
    else
        phaseROI = getROI(phaseImMod, 'Select ROI for phase image');
    end
end
T = T * getTranslationMatrix(-(int16(phaseROI)-1));
%% Improve registration accuracy with cross-correlation algorithm
if bfFlag
    phaseImMod = FastSeg.Preprocess.adjustSingleImage(phaseIm, flipFlags(1), ...
        flipFlags(2), angularDisplacement, phaseROI);

    bfImTrans = imwarp(bfIm, affine2d(T), 'OutputView', imref2d(size(phaseImMod)),'FillValues',0);
    [~, T, c, xpeak, ypeak] = FastSeg.registerWithCrossCorrelation(phaseImMod, bfImTrans, T, param.registrationMaxOffset, param.blackShiftedBF);

    [ync, xnc] = find(c);
    figure;imagesc(c); axis equal; hold on;
    xlim([ xnc(1) xnc(end)]); ylim([ync(1) ync(end)]);
    plot(xpeak, ypeak, 'wx', 'MarkerFaceColor', 'w', 'MarkerSize', 10)

    bfIm = imwarp(bfIm, affine2d(T), 'OutputView', imref2d(size(phaseImMod)),'FillValues',0);
    figure, imshow(imfuse(phaseImMod,bfIm,'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));
end
%% Store parameters in a new file
param.angularDisplacement = angularDisplacement;
param.flipFlags = flipFlags;
param.roiInPhase = phaseROI;
param.transMatPath = fullfile(outputDir, 'transMat.mat');
param.barcodeROI = barcodeROI;

save(param.transMatPath, 'T');

[the_path, the_name, the_suffix] = fileparts(parameterFile);
saveName = fullfile(the_path, [the_name 'Init' the_suffix]);
saveParameterFile(param, saveName);