function processCells(sourceDir, outputDir, varargin)
% Function which does the following steps: preprocessing, segmentation, dot
% detection and tracking, cell tracking, and some cell measurements.
% Executes inner for-loops along frames (time-points) in parallel.
%
% Input:
%   sourceDir       - path to directory with microscopy images.
%   outpuDir        - path to directory where the results will be saved.
%   parameterFile   - path to the parameter file (.txt) or
%                     a structure array.
%   pipelineDir     - path to directory with ImAnalysis pipeline, optional.
%% Initialization
totalStartTime = tic;

if length(varargin)<1
    error('Please supply parameters');
end

if length(varargin)==2
    pipelineDir = varargin{2};
else
    pipelineDir = fileparts(which('processCells.m'));
end

if isa(varargin{1}, 'struct')
    parameters = varargin{1};
    saveName = fullfile(outputDir,'SavedParameters.txt');
    saveParameterFile(parameters, saveName);
elseif isa(varargin{1}, 'char')
    disp('----------------------------------------');
    disp('Parsing parameter file');
    parameterFile = varargin{1};
    parameters = parseAndSaveParameters(outputDir, pipelineDir, parameterFile);
else
    error('Please supply parameters as a structure array or a path to a file with parameters');
end

disp('----------------------------------------');
disp('Parsing metadata file');
imAdapterObj = getImAdapter(sourceDir, outputDir);
expInfoObj = getExpInfo(sourceDir, outputDir, imAdapterObj, parameters);
bfChanNames = expInfoObj.getChannelNames('bf');
fluoChanNames = expInfoObj.getChannelNames('fluo');

positionList = expInfoObj.getPositionList();
nPositions = numel(positionList);

if parameters.parforPositions
    nCores = parameters.parNumWorkers;
else
    nCores = 0;
end
%% Preprocessing
if parameters.doPreprocessing == 1
    disp('----------------------------------------');
    disp('Image preprocessing stage');
    startTime = tic;
    bfTs = cell(1,nPositions);
    fluoTs = cell(1,nPositions);
    regFlags = cell(1,nPositions);
    growthChannels = cell(1,nPositions);
    parfor (posIdx = 1:nPositions, nCores)
        fprintf('Position %d/%d\n', posIdx, nPositions);
        currPos = positionList{posIdx};
        [bfTs{posIdx}, fluoTs{posIdx}, regFlags{posIdx}, growthChannels{posIdx}] = ...
            FastSeg.preprocessImages(expInfoObj, currPos);
    end
    % Update expInfo
    for posIdx = 1:nPositions
        posName = positionList{posIdx};
        for chIdx = 1:length(bfChanNames)
            bfChanName = bfChanNames{chIdx};
            expInfoObj.setAllT(posName, bfChanName, bfTs{posIdx}{chIdx});
            expInfoObj.setFailedRegistration(posName,bfChanName, regFlags{posIdx}{chIdx});
        end
        for chIdx = 1:length(fluoChanNames)
            fluoChanName = fluoChanNames{chIdx};
            expInfoObj.setAllT(posName, fluoChanName, fluoTs{posIdx}{chIdx});
        end
        if parameters.isMothermachine
            %expInfoObj.setChannelLocations(posName,growthChannels{posIdx});
        end
    end
    endTime = toc(startTime);
    fprintf('Total image preprocessing time: %.2fs\n', endTime);
    expInfoObj.setParameters('doPreprocessing', 2);
    expInfoObj.saveExpInfo();
end
%% Segmentation
if parameters.doSegmentation == 1
    disp('----------------------------------------');
    disp('Image segmentation stage');
    startTime = tic;
    parfor (posIdx = 1:nPositions, nCores)
        fprintf('Position %d/%d\n', posIdx, nPositions);
        currPos = positionList{posIdx};
        FastSeg.segmentCells(expInfoObj,currPos,pipelineDir);
    end
    endTime = toc(startTime);
    fprintf('Total image segmentation time: %.2fs\n', endTime);
    expInfoObj.setParameters('doSegmentation', 2);
    expInfoObj.saveExpInfo();    
end
%% Cell Tracking
if parameters.doBlobProcessing == 1
    disp('----------------------------------------');
    disp('Blob processing stage');
    startTime = tic; 
    if parameters.trackCells
        disp('Tracking cells using Baxter Algorithm');
        parfor (posIdx = 1:nPositions, nCores)
%         for posIdx = 1:nPositions
            fprintf('Position %d/%d\n', posIdx, nPositions);
            currPos = positionList{posIdx};
            Cell.Track.trackCells(expInfoObj,imAdapterObj,currPos);
        end
    else
        disp('No cell tracking, initializing MCell objects');
        parfor (posIdx = 1:nPositions, nCores)
            fprintf('Position %d/%d\n', posIdx, nPositions);
            currPos = positionList{posIdx};
            Cell.initializeOutputStructure(expInfoObj,imAdapterObj,currPos);
        end
    end
    endTime = toc(startTime);
    fprintf('Total cell tracking time: %.2fs\n', endTime);    
    expInfoObj.setParameters('doBlobProcessing',2);
    expInfoObj.saveExpInfo();    
end
%% Cell Measurements & Internal Coordinates
if parameters.doCellMeasurements == 1
    disp('----------------------------------------');
    disp('Cell measurement stage');
    startTime = tic;
%     parfor (posIdx = 1:nPositions, nCores)
    for posIdx = 1:nPositions

        fprintf('Position %d/%d\n', posIdx, nPositions);
        currPos = positionList{posIdx};
        mCellsMatFile = fullfile(outputDir,currPos,'trackedCells.mat');
        [mCells, mFrames] = Cell.MCell.loadMCells(mCellsMatFile);
        if ~isempty(mCells)
            Cell.Measure.computeCellMeasurements(expInfoObj,currPos,mFrames);
            Cell.Measure.orientCellPoles(mCells);
            Cell.MCell.saveMCells(mCellsMatFile,mCells);
        end
    end
    endTime = toc(startTime);
    fprintf('Total cell measurement time: %.2fs\n', endTime);
    expInfoObj.setParameters('doCellMeasurements', 2);
    expInfoObj.saveExpInfo();
end
%% Dot Detection & Tracking
if parameters.doDotDetection == 1
    disp('----------------------------------------');
    disp('Particle processing stage');
    startTime=tic;
    if parameters.doLSQrefineDots
        EMCCDfit.getLogL_EMCCD_LookupTable(expInfoObj,fluoChanNames{1});
    end
    computeInternalCoord = parameters.doCellMeasurements > 0;
    parfor (posIdx = 1:nPositions, nCores)
        fprintf('Position %d/%d\n', posIdx, nPositions);
        currPos = positionList{posIdx};
        mCellsMatFile = fullfile(outputDir,currPos,'trackedCells.mat');
        [mCells, mFrames] = Cell.MCell.loadMCells(mCellsMatFile);
        if ~isempty(mCells)
            Cell.initializeParticleList(mCells);
            for chanIdx = 1:numel(fluoChanNames)
                currFluoChanName = fluoChanNames{chanIdx};
                Spot.processDots(expInfoObj, currPos, currFluoChanName, mFrames);
            end
            if computeInternalCoord
                Cell.Measure.computeParticleInternalCoord(mCells);
            end
            Cell.MCell.saveMCells(mCellsMatFile, mCells);
        end
    end
    endTime = toc(startTime);
    fprintf('Total particle processing time: %.2fs\n', endTime);    
    expInfoObj.setParameters('doDotDetection',2);
    expInfoObj.saveExpInfo();
end
%% Display execution time
totalTime = datestr(seconds(toc(totalStartTime)),'DD:HH:MM:SS');
disp('----------------------------------------');
fprintf('Total execution time: %s\n', totalTime);
end