function T = DSBselectorGUI(T,randomizeIt,channels_names)
% Interactive figure to select mcherry degradation and repair events.
% Function accepts 'T' structure, output of 'getFluoSlices' function. The
% function allows for clicking the events on fluorescent images of mCherry
% channel (bottom row), displays CFP channel for reference. Chaning cells
% is done with left/right arrows. Selection can be erased with backspace.
% Function dumps the 'cherry_events' into the workspace upon closing. If
% there was no visible 'cherry_event' in the life of a cell, the returned
% matrix is [nan nan].
%
% godspeed.

% part of 'selectInducedCellsGUI'

%initialise graphic handles

fig = figure();
fig.KeyPressFcn = @changeCellFunction;
fig.CloseRequestFcn = @my_closereq;
fig.Color = [0 0 0];

ax_1 = axes('Parent',fig);
im_cherry = image(ax_1,'CDataMapping','scaled','ButtonDownFcn',@imageClickFunction);
ax_1.XTick = [];
ax_1.YTick = [];
ax_1.Units = 'pixels';
ax_1.Colormap = gray;
ax_1.Color = [0 0 0];

selection  = line(ax_1);

ax_2 = axes('Parent',fig);
ax_2.Units = 'pixels';
ax_2.Colormap = gray;
ax_2.XTick = [];
ax_2.YTick = [];
ax_2.Color = [0 0 0];

im_CFP = image(ax_2,'CDataMapping','scaled');

%initialise variables
%randomize cell display
if randomizeIt
    cells_vector = randperm(length(T.T));
else
    cells_vector = 1:length(T.T);
end

cellnum = 1;
imboxes = [];
%load selections if there were fields already
if isfield(T.T,'loss') 
    cherry_events = [[T.T.loss]' [T.T.recovery]']; %first degradation, then recovery
    T.T=rmfield(T.T,'loss');
    T.T=rmfield(T.T,'recovery');
else
    cherry_events = nan(size(T.T,1),2);
end

sel_strings = {'loss','recovery'};
%get screen size for resizing of images;
set(0,'units','pixels')
resolution = get(0,'ScreenSize');

updateFigure()
uiwait()

    function updateFigure(src,event)
        %make drawing of each cell
        
        [canvasCherry, imboxes] = drawCanvasAndBoxes( T.T(cells_vector(cellnum)).(channels_names{1}));
        [canvasCFP, ~] = drawCanvasAndBoxes( T.T(cells_vector(cellnum)).(channels_names{2}));
        
        if 0.9*(size(canvasCherry,2)*2) < resolution(3)
            canvasCherry = imresize(canvasCherry, 2, 'nearest');
            canvasCFP = imresize(canvasCFP, 2, 'nearest');
            imboxes = imboxes.*2;
        elseif size(canvasCherry,2) > resolution(3)
            res_factor = round(resolution(3)/size(canvasCherry,2));
            canvasCherry = imresize(canvasCherry,res_factor, 'nearest');
            canvasCFP = imresize(canvasCFP, res_factor, 'nearest');
        end
        
        im_cherry.CData = canvasCherry;
%         im_cherry.CData = imfuse(canvasCherry,canvasCFP);
        ax_1.Position = [0 0 size(canvasCherry,2)*1, size(canvasCherry,1)*1];
        
        %draw selected cells if any
        for p = find((~isnan(cherry_events(cells_vector(cellnum),:))))
            tb = imboxes(cherry_events(cells_vector(cellnum),p),:);
            text(ax_1,  tb(1), tb(2)+tb(4)+10, sel_strings{p},...
                        'Color',[1 1 1],...
                        'tag','sel_text');
            line(ax_1,[tb(1) tb(1)       tb(1)+tb(3) tb(1)+tb(3) tb(1)],...
                [tb(2)  tb(2)+tb(4) tb(2)+tb(4) tb(2)       tb(2)],...
                'Color',[1 1 1],...
                'lineStyle','-',...
                'tag', 'selection_box');
        end
        
%         tmp = imfuse(canvasCherry,canvasCFP,'falsecolor','Scaling','joint');
%         im_CFP.CData = tmp;
        im_CFP.CData = canvasCFP;
        ax_2.Position = [0 size(canvasCherry,1)*1 size(canvasCFP,2)*1, size(canvasCFP,1)*1];
        
        %pos fig in the middle
        x_pos = round(resolution(3) - size(canvasCherry,2))/2;
        y_pos = round(resolution(4) - 2*size(canvasCherry,1))/2;
        fig.Position = [x_pos y_pos size(canvasCherry,2)*1, size(canvasCherry,1)*2];
        %fig.Name = ['cell ' num2str(cells_vector(cellnum)) ' / ' num2str(length(T.T))];
        %this way I dont know which cell I analyse
        fig.Name = ['cell ' num2str(cellnum) ' / ' num2str(length(T.T))];
        
    end

    function [canvas,imboxes] = drawCanvasAndBoxes(ims)
        %reconstruct the images and get bounding boxes of each. Because
        %imboxes are initialised as all zeros, the resulting repair times
        %dont have to be adjusted if the imaging was not done every minute.
        
        sz_y = cellfun(@(x) size(x,1),ims);
        sz_x = cellfun(@(x) size(x,2),ims);
        step_sz = [0 cumsum(sz_x)];

        imboxes = zeros(size(ims,2),4);
        canvas = zeros(max(sz_y),sum(sz_x));
        offset = 1;
        for imc = 1:size(ims,2)
            if all(all(ims{imc} == 0)), continue, end %change this to remove black spaces!
            sz_yt = floor((max(sz_y)-size(ims{imc},1))/2);
            canvas(1:sz_y(imc),offset:offset+sz_x(imc)-1) = ims{imc};
%             canvas(sz_yt+1:sz_yt+size( ims{imc},1), step_sz(imc)+1:step_sz(imc+1)) = ims{imc};
            imboxes(imc,:) = [offset-0.5 0.5 sz_x(imc) sz_y(imc)];
%             imboxes(imc,:) = [offset-0.5 sz_yt sz_x(imc) sz_y(imc)];
            offset = offset + sz_x(imc);
        end
        imboxes(:,2) = imboxes(:,2)+0.5;
        canvas(:,find(all(isnan(canvas)),1,'first'):end) = [];
        canvas(canvas==0) = nan;
    end

    function changeCellFunction(src,event)
        %switch the cell with arrows, erase with backspace
        keyPress = event.Key;
        switch keyPress
            case 'leftarrow'
                cellnum = cellnum - 1;
                if cellnum < 1
                    cellnum = length(T.T);
                end
            case 'rightarrow'
                cellnum = cellnum + 1;
                if cellnum > length(T.T)
                    cellnum = 1;
                end
            case 'backspace'
                %clear selections
                cherry_events(cells_vector(cellnum),:) = nan;
                updateFigure()
        end
        f_line = findobj('tag','selection_box');
        delete(f_line)
        f_text = findobj('tag','sel_text');
        delete(f_text)
        updateFigure()
    end

    function imageClickFunction(src,event)
        %find where we clicked on mChery cell image. Remember first click
        %and the second click.
        if all(~isnan(cherry_events(cells_vector(cellnum),:)))
            return
        else
            cpt = get(ax_1, 'CurrentPoint');
            this_x = cpt(1,1);
            this_y = cpt(1,2);
            %this works
            this_rect = find(rectint(imboxes,[this_x this_y 1 1]),1,'first'); %no loop
            if isnan(cherry_events(cells_vector(cellnum),1))
                cherry_events(cells_vector(cellnum),1) = this_rect;
            elseif isnan(cherry_events(cells_vector(cellnum),2))
                cherry_events(cells_vector(cellnum),2) = this_rect;
            end
        end
        updateFigure()
    end

    function my_closereq(src,callbackdax_posta)
        % Close request function
        % to display a question dialog box
        selection = questdlg('Close This Figure?',...
            'Close Request Function',...
            'Yes','No','Yes');
        switch selection
            case 'Yes'
                %add fields to the old structure
                temp_struct=repmat(struct('loss',0,'recovery',0),length(T.T),1);
                C = num2cell(cherry_events);
                [temp_struct.loss] = C{:,1};
                [temp_struct.recovery] = C{:,2};
                mergestructs = @(x,y) cell2struct([struct2cell(x);struct2cell(y)],[fieldnames(x);fieldnames(y)]);
                T.T = mergestructs(T.T,temp_struct);
                uiresume()
                delete(fig)
            case 'No'
                return
        end
    end
end